#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx001.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX001(object):
	category = '''Dynamic'''
	description = '''Stereo Compressor'''
	short_description = '''Stereo Compressor'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX001 Stereo Compressor object'''
		bytes = struct.unpack('<10B',raw[:10])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.envelope_select = (bytes[3] >> 5) & 0x1
		self.sensitivity = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		self.attack = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		data = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x3) << 4)
		if data & 0x20:
			data -= 0x40
		self.pre_leq_gain = data
		data = (bytes[6] >> 2) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.pre_heq_gain = data
		self.output_level = bytes[7] & 0x7F
		self.output_level_mod_source = ((bytes[7] & 0x80) >> 7) | ((bytes[8] & 0x1F) << 1)
		data = ((bytes[8] & 0xE0) >> 5) | ((bytes[9] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.output_level_mod_intensity = data
		self.pre_leq_fc = (bytes[9] >> 5) & 0x1
		self.pre_heq_fc = (bytes[9] >> 6) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX001 Stereo Compressor object into raw data'''
		bytes = [0] * 10
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.envelope_select & 0x1) << 5) | ((self.sensitivity << 6) & 0xC0)
		bytes[4] = ((self.sensitivity >> 2) & 0x1F) | ((self.attack << 5) & 0xE0)
		bytes[5] = ((self.attack >> 3) & 0xF) | ((self.pre_leq_gain << 4) & 0xF0)
		bytes[6] = ((self.pre_leq_gain >> 4) & 0x3) | ((self.pre_heq_gain & 0x3F) << 2)
		bytes[7] = (self.output_level & 0x7F) | ((self.output_level_mod_source << 7) & 0x80)
		bytes[8] = ((self.output_level_mod_source >> 1) & 0x1F) | ((self.output_level_mod_intensity << 5) & 0xE0)
		bytes[9] = ((self.output_level_mod_intensity >> 3) & 0x1F) | ((self.pre_leq_fc & 0x1) << 5) | ((self.pre_heq_fc & 0x1) << 6)
		return struct.pack('<10B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX001 Stereo Compressor object'''
		json = []
		json.append(('name','FX001'))
		json.append(('description','Stereo Compressor'))
		json.append(['envelope_select',self.envelope_select])
		json.append(['sensitivity',self.sensitivity])
		json.append(['attack',self.attack])
		json.append(['input_trim',self.input_trim])
		json.append(['pre_leq_gain',self.pre_leq_gain])
		json.append(['pre_heq_gain',self.pre_heq_gain])
		json.append(['output_level',self.output_level])
		json.append(['output_level_mod_source',self.output_level_mod_source])
		json.append(['output_level_mod_intensity',self.output_level_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['pre_leq_fc',self.pre_leq_fc])
		json.append(['pre_heq_fc',self.pre_heq_fc])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_select(self):
		'''Envelope Select'''
		return self._envelope_select

	@envelope_select.setter
	def envelope_select(self,value):
		if 0 <= value <= 1:
			self._envelope_select = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def sensitivity(self):
		'''Sensitivity'''
		return self._sensitivity

	@sensitivity.setter
	def sensitivity(self,value):
		if 1 <= value <= 100:
			self._sensitivity = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def attack(self):
		'''Attack'''
		return self._attack

	@attack.setter
	def attack(self,value):
		if 1 <= value <= 100:
			self._attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_leq_gain(self):
		'''Pre LEQ Gain'''
		return self._pre_leq_gain

	@pre_leq_gain.setter
	def pre_leq_gain(self,value):
		if -30 <= value <= 30:
			self._pre_leq_gain = value
		else:
			raise ValueError('%s not -30 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_heq_gain(self):
		'''Pre HEQ Gain'''
		return self._pre_heq_gain

	@pre_heq_gain.setter
	def pre_heq_gain(self,value):
		if -30 <= value <= 30:
			self._pre_heq_gain = value
		else:
			raise ValueError('%s not -30 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level(self):
		'''Output Level'''
		return self._output_level

	@output_level.setter
	def output_level(self,value):
		if 0 <= value <= 100:
			self._output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level_mod_source(self):
		'''Output Level Mod.Source'''
		return self._output_level_mod_source

	@output_level_mod_source.setter
	def output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level_mod_intensity(self):
		'''Output Level Mod.Intensity'''
		return self._output_level_mod_intensity

	@output_level_mod_intensity.setter
	def output_level_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._output_level_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_leq_fc(self):
		'''Pre LEQ Fc'''
		return self._pre_leq_fc

	@pre_leq_fc.setter
	def pre_leq_fc(self,value):
		if 0 <= value <= 1:
			self._pre_leq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_heq_fc(self):
		'''Pre HEQ Fc'''
		return self._pre_heq_fc

	@pre_heq_fc.setter
	def pre_heq_fc(self,value):
		if 0 <= value <= 1:
			self._pre_heq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

