#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx002.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX002(object):
	category = '''Dynamic'''
	description = '''Red Comp'''
	short_description = '''Red Comp'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX002 Red Comp object'''
		bytes = struct.unpack('<8B',raw[:8])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.sensitivity = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.attack = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.level = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		self.level_mod_source = (bytes[6] >> 2) & 0x3F
		data = bytes[7] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.level_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX002 Red Comp object into raw data'''
		bytes = [0] * 8
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.sensitivity << 5) & 0xE0
		bytes[4] = ((self.sensitivity >> 3) & 0xF) | ((self.attack << 4) & 0xF0)
		bytes[5] = ((self.attack >> 4) & 0x7) | ((self.level << 3) & 0xF8)
		bytes[6] = ((self.level >> 5) & 0x3) | ((self.level_mod_source & 0x3F) << 2)
		bytes[7] = self.level_mod_intensity & 0xFF
		return struct.pack('<8B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX002 Red Comp object'''
		json = []
		json.append(('name','FX002'))
		json.append(('description','Red Comp'))
		json.append(['sensitivity',self.sensitivity])
		json.append(['attack',self.attack])
		json.append(['level',self.level])
		json.append(['level_mod_source',self.level_mod_source])
		json.append(['level_mod_intensity',self.level_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def sensitivity(self):
		'''Sensitivity'''
		return self._sensitivity

	@sensitivity.setter
	def sensitivity(self,value):
		if 0 <= value <= 99:
			self._sensitivity = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def attack(self):
		'''Attack'''
		return self._attack

	@attack.setter
	def attack(self,value):
		if 0 <= value <= 99:
			self._attack = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def level(self):
		'''Level'''
		return self._level

	@level.setter
	def level(self,value):
		if 0 <= value <= 100:
			self._level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def level_mod_source(self):
		'''Level Mod.Source'''
		return self._level_mod_source

	@level_mod_source.setter
	def level_mod_source(self,value):
		if 0 <= value <= 37:
			self._level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def level_mod_intensity(self):
		'''Level Mod.Intensity'''
		return self._level_mod_intensity

	@level_mod_intensity.setter
	def level_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._level_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

