#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx004.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX004(object):
	category = '''Dynamic'''
	description = '''Multiband Limiter'''
	short_description = '''Multiband Limiter'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX004 Multiband Limiter object'''
		bytes = struct.unpack('<13B',raw[:13])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.ratio = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		data = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.threshold = data
		self.attack = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		self.release = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		data = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		if data & 0x40:
			data -= 0x80
		self.low_offset = data
		data = (bytes[8] >> 1) & 0x7F
		if data & 0x40:
			data -= 0x80
		self.mid_offset = data
		data = bytes[9] & 0x7F
		if data & 0x40:
			data -= 0x80
		self.high_offset = data
		data = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.gain_adjust = data
		self.gain_adjust_mod_source = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0xF) << 2)
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.gain_adjust_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX004 Multiband Limiter object into raw data'''
		bytes = [0] * 13
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.ratio << 5) & 0xE0
		bytes[4] = ((self.ratio >> 3) & 0x1F) | ((self.threshold << 5) & 0xE0)
		bytes[5] = ((self.threshold >> 3) & 0xF) | ((self.attack << 4) & 0xF0)
		bytes[6] = ((self.attack >> 4) & 0x7) | ((self.release << 3) & 0xF8)
		bytes[7] = ((self.release >> 5) & 0x3) | ((self.low_offset << 2) & 0xFC)
		bytes[8] = ((self.low_offset >> 6) & 0x1) | ((self.mid_offset & 0x7F) << 1)
		bytes[9] = (self.high_offset & 0x7F) | ((self.gain_adjust << 7) & 0x80)
		bytes[10] = ((self.gain_adjust >> 1) & 0x3F) | ((self.gain_adjust_mod_source << 6) & 0xC0)
		bytes[11] = ((self.gain_adjust_mod_source >> 2) & 0xF) | ((self.gain_adjust_mod_intensity << 4) & 0xF0)
		bytes[12] = (self.gain_adjust_mod_intensity >> 4) & 0x7
		return struct.pack('<13B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX004 Multiband Limiter object'''
		json = []
		json.append(('name','FX004'))
		json.append(('description','Multiband Limiter'))
		json.append(['ratio',self.ratio])
		json.append(['threshold',self.threshold])
		json.append(['attack',self.attack])
		json.append(['release',self.release])
		json.append(['low_offset',self.low_offset])
		json.append(['mid_offset',self.mid_offset])
		json.append(['high_offset',self.high_offset])
		json.append(['gain_adjust',self.gain_adjust])
		json.append(['gain_adjust_mod_source',self.gain_adjust_mod_source])
		json.append(['gain_adjust_mod_intensity',self.gain_adjust_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def ratio(self):
		'''Ratio'''
		return self._ratio

	@ratio.setter
	def ratio(self,value):
		if 0 <= value <= 131:
			self._ratio = value
		else:
			raise ValueError('%s not 0 <= value <= 131' % value)

#-------------------------------------------------------------------------------
	@property
	def threshold(self):
		'''Threshold'''
		return self._threshold

	@threshold.setter
	def threshold(self,value):
		if -40 <= value <= 0:
			self._threshold = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def attack(self):
		'''Attack'''
		return self._attack

	@attack.setter
	def attack(self,value):
		if 1 <= value <= 100:
			self._attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def release(self):
		'''Release'''
		return self._release

	@release.setter
	def release(self,value):
		if 1 <= value <= 100:
			self._release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_offset(self):
		'''Low Offset'''
		return self._low_offset

	@low_offset.setter
	def low_offset(self,value):
		if -40 <= value <= 0:
			self._low_offset = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_offset(self):
		'''Mid Offset'''
		return self._mid_offset

	@mid_offset.setter
	def mid_offset(self,value):
		if -40 <= value <= 0:
			self._mid_offset = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def high_offset(self):
		'''High Offset'''
		return self._high_offset

	@high_offset.setter
	def high_offset(self,value):
		if -40 <= value <= 0:
			self._high_offset = value
		else:
			raise ValueError('%s not -40 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def gain_adjust(self):
		'''Gain Adjust'''
		return self._gain_adjust

	@gain_adjust.setter
	def gain_adjust(self,value):
		if -39 <= value <= 24:
			self._gain_adjust = value
		else:
			raise ValueError('%s not -39 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def gain_adjust_mod_source(self):
		'''Gain Adjust Mod.Source'''
		return self._gain_adjust_mod_source

	@gain_adjust_mod_source.setter
	def gain_adjust_mod_source(self,value):
		if 0 <= value <= 37:
			self._gain_adjust_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def gain_adjust_mod_intensity(self):
		'''Gain Adjust Mod.Intensity'''
		return self._gain_adjust_mod_intensity

	@gain_adjust_mod_intensity.setter
	def gain_adjust_mod_intensity(self,value):
		if -63 <= value <= 63:
			self._gain_adjust_mod_intensity = value
		else:
			raise ValueError('%s not -63 <= value <= 63' % value)

