#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx005.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX005(object):
	category = '''Dynamic'''
	description = '''Stereo Mastering Limiter'''
	short_description = '''St.MasteringLimtr'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX005 Stereo Mastering Limiter object'''
		bytes = struct.unpack('<8B',raw[:8])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		data = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x7F) << 3)
		if data & 0x200:
			data -= 0x400
		self.threshold = data
		data = ((bytes[4] & 0x80) >> 7) | (bytes[5] << 1) | ((bytes[6] & 0x1) << 9)
		if data & 0x200:
			data -= 0x400
		self.out_ceiling = data
		self.release = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x3) << 7)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX005 Stereo Mastering Limiter object into raw data'''
		bytes = [0] * 8
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.threshold << 5) & 0xE0
		bytes[4] = ((self.threshold >> 3) & 0x7F) | ((self.out_ceiling << 7) & 0x80)
		bytes[5] = (self.out_ceiling >> 1) & 0xFF
		bytes[6] = ((self.out_ceiling >> 1) & 0x9) | ((self.release << 1) & 0xFE)
		bytes[7] = (self.release >> 7) & 0x3
		return struct.pack('<8B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX005 Stereo Mastering Limiter object'''
		json = []
		json.append(('name','FX005'))
		json.append(('description','Stereo Mastering Limiter'))
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['threshold',self.threshold])
		json.append(['out_ceiling',self.out_ceiling])
		json.append(['release',self.release])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def threshold(self):
		'''Threshold'''
		return self._threshold

	@threshold.setter
	def threshold(self,value):
		if -300 <= value <= 0:
			self._threshold = value
		else:
			raise ValueError('%s not -300 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def out_ceiling(self):
		'''Out Ceiling'''
		return self._out_ceiling

	@out_ceiling.setter
	def out_ceiling(self,value):
		if -300 <= value <= 0:
			self._out_ceiling = value
		else:
			raise ValueError('%s not -300 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def release(self):
		'''Release'''
		return self._release

	@release.setter
	def release(self,value):
		if 13 <= value <= 290:
			self._release = value
		else:
			raise ValueError('%s not 13 <= value <= 290' % value)

