#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx006.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX006(object):
	category = '''Dynamic'''
	description = '''Stereo Gate'''
	short_description = '''Stereo Gate'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX006 Stereo Gate object'''
		bytes = struct.unpack('<13B',raw[:13])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		data = (bytes[3] >> 5) & 0x7
		if data & 0x4:
			data -= 0x8
		self.envelope_source = data
		self.fx_control_trim = bytes[4] & 0x7F
		self.envelope_select = ((bytes[4] & 0x80) >> 7) | ((bytes[5] & 0x1) << 1)
		self.envelope_select_mod_source = (bytes[5] >> 1) & 0x3F
		self.polarity = (bytes[5] >> 7) & 0x1
		self.threshold = bytes[6] & 0x7F
		self.attack = ((bytes[6] & 0x80) >> 7) | ((bytes[7] & 0x3F) << 1)
		self.release = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0x1F) << 2)
		self.delay_time = ((bytes[8] & 0xE0) >> 5) | ((bytes[9] & 0xF) << 3)
		self.side_peq_insert = (bytes[9] >> 4) & 0x1
		self.trigger_monitor = (bytes[9] >> 5) & 0x1
		self.side_peq_cutoff = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0xFF) << 2)
		self.side_peq_q = bytes[11] & 0x7F
		data = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.side_peq_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX006 Stereo Gate object into raw data'''
		bytes = [0] * 13
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.envelope_source & 0x7) << 5
		bytes[4] = (self.fx_control_trim & 0x7F) | ((self.envelope_select << 7) & 0x80)
		bytes[5] = ((self.envelope_select >> 1) & 0x1) | ((self.envelope_select_mod_source & 0x3F) << 1) | ((self.polarity & 0x1) << 7)
		bytes[6] = (self.threshold & 0x7F) | ((self.attack << 7) & 0x80)
		bytes[7] = ((self.attack >> 1) & 0x3F) | ((self.release << 6) & 0xC0)
		bytes[8] = ((self.release >> 2) & 0x1F) | ((self.delay_time << 5) & 0xE0)
		bytes[9] = ((self.delay_time >> 3) & 0xF) | ((self.side_peq_insert & 0x1) << 4) | ((self.trigger_monitor & 0x1) << 5) | ((self.side_peq_cutoff << 6) & 0xC0)
		bytes[10] = (self.side_peq_cutoff >> 2) & 0xFF
		bytes[11] = (self.side_peq_q & 0x7F) | ((self.side_peq_gain << 7) & 0x80)
		bytes[12] = (self.side_peq_gain >> 1) & 0x3F
		return struct.pack('<13B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX006 Stereo Gate object'''
		json = []
		json.append(('name','FX006'))
		json.append(('description','Stereo Gate'))
		json.append(['envelope_select',self.envelope_select])
		json.append(['envelope_select_mod_source',self.envelope_select_mod_source])
		json.append(['polarity',self.polarity])
		json.append(['threshold',self.threshold])
		json.append(['attack',self.attack])
		json.append(['release',self.release])
		json.append(['delay_time',self.delay_time])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['envelope_source',self.envelope_source])
		json.append(['fx_control_trim',self.fx_control_trim])
		json.append(['side_peq_insert',self.side_peq_insert])
		json.append(['trigger_monitor',self.trigger_monitor])
		json.append(['side_peq_cutoff',self.side_peq_cutoff])
		json.append(['side_peq_q',self.side_peq_q])
		json.append(['side_peq_gain',self.side_peq_gain])
		json.append(['side_peq_gain_display',self.side_peq_gain_display])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_source(self):
		'''Envelope Source'''
		return self._envelope_source

	@envelope_source.setter
	def envelope_source(self,value):
		if -1 <= value <= 2:
			self._envelope_source = value
		else:
			raise ValueError('%s not -1 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def fx_control_trim(self):
		'''Fx Control Trim'''
		return self._fx_control_trim

	@fx_control_trim.setter
	def fx_control_trim(self,value):
		if 0 <= value <= 100:
			self._fx_control_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_select(self):
		'''Envelope Select'''
		return self._envelope_select

	@envelope_select.setter
	def envelope_select(self,value):
		if 1 <= value <= 3:
			self._envelope_select = value
		else:
			raise ValueError('%s not 1 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_select_mod_source(self):
		'''Envelope Select Mod.Source'''
		return self._envelope_select_mod_source

	@envelope_select_mod_source.setter
	def envelope_select_mod_source(self,value):
		if 0 <= value <= 37:
			self._envelope_select_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def polarity(self):
		'''Polarity'''
		return self._polarity

	@polarity.setter
	def polarity(self,value):
		if 0 <= value <= 1:
			self._polarity = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def threshold(self):
		'''Threshold'''
		return self._threshold

	@threshold.setter
	def threshold(self,value):
		if 0 <= value <= 100:
			self._threshold = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def attack(self):
		'''Attack'''
		return self._attack

	@attack.setter
	def attack(self,value):
		if 1 <= value <= 100:
			self._attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def release(self):
		'''Release'''
		return self._release

	@release.setter
	def release(self,value):
		if 1 <= value <= 100:
			self._release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 100:
			self._delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def side_peq_insert(self):
		'''Side PEQ Insert'''
		return self._side_peq_insert

	@side_peq_insert.setter
	def side_peq_insert(self,value):
		if 0 <= value <= 1:
			self._side_peq_insert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def trigger_monitor(self):
		'''Trigger Monitor'''
		return self._trigger_monitor

	@trigger_monitor.setter
	def trigger_monitor(self,value):
		if 0 <= value <= 1:
			self._trigger_monitor = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def side_peq_cutoff(self):
		'''Side PEQ Cutoff'''
		return self._side_peq_cutoff

	@side_peq_cutoff.setter
	def side_peq_cutoff(self,value):
		if 1 <= value <= 600:
			self._side_peq_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 600' % value)

#-------------------------------------------------------------------------------
	@property
	def side_peq_q(self):
		'''Side PEQ Q'''
		return self._side_peq_q

	@side_peq_q.setter
	def side_peq_q(self,value):
		if 5 <= value <= 100:
			self._side_peq_q = value
		else:
			raise ValueError('%s not 5 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def side_peq_gain(self):
		'''Side PEQ Gain'''
		return self._side_peq_gain

	@side_peq_gain.setter
	def side_peq_gain(self,value):
		if -36 <= value <= 36:
			self._side_peq_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def side_peq_gain_display(self):
		'''Side PEQ Gain display value'''
		dv = self._side_peq_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

