#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx007.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX007(object):
	category = '''Eq/Filter'''
	description = '''Stereo Parametric 4-Band EQ'''
	short_description = '''St.Parametric4EQ'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX007 Stereo Parametric 4-Band EQ object'''
		bytes = struct.unpack('<17B',raw[:17])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.band1_type = (bytes[3] >> 5) & 0x1
		self.band4_type = (bytes[3] >> 6) & 0x1
		self.band1_cutoff = ((bytes[3] & 0x80) >> 7) | ((bytes[4] & 0x3F) << 1)
		self.band1_q = ((bytes[4] & 0xC0) >> 6) | ((bytes[5] & 0x1F) << 2)
		data = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.band1_gain = data
		self.band2_cutoff = ((bytes[6] & 0xF0) >> 4) | ((bytes[7] & 0x3F) << 4)
		self.band2_q = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0x1F) << 2)
		data = ((bytes[8] & 0xE0) >> 5) | ((bytes[9] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.band2_gain = data
		self.band2_gain_mod_source = ((bytes[9] & 0xF0) >> 4) | ((bytes[10] & 0x3) << 4)
		data = ((bytes[10] & 0xFC) >> 2) | ((bytes[11] & 0x1) << 6)
		if data & 0x40:
			data -= 0x80
		self.band2_gain_mod_intensity = data
		self.band3_cutoff = (bytes[11] >> 1) & 0x7F
		self.band3_q = bytes[12] & 0x7F
		data = ((bytes[12] & 0x80) >> 7) | ((bytes[13] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.band3_gain = data
		self.band4_cutoff = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0x3F) << 2)
		self.band4_q = ((bytes[14] & 0xC0) >> 6) | ((bytes[15] & 0x1F) << 2)
		data = ((bytes[15] & 0xE0) >> 5) | ((bytes[16] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.band4_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX007 Stereo Parametric 4-Band EQ object into raw data'''
		bytes = [0] * 17
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.band1_type & 0x1) << 5) | ((self.band4_type & 0x1) << 6) | ((self.band1_cutoff << 7) & 0x80)
		bytes[4] = ((self.band1_cutoff >> 1) & 0x3F) | ((self.band1_q << 6) & 0xC0)
		bytes[5] = ((self.band1_q >> 2) & 0x1F) | ((self.band1_gain << 5) & 0xE0)
		bytes[6] = ((self.band1_gain >> 3) & 0xF) | ((self.band2_cutoff << 4) & 0xF0)
		bytes[7] = ((self.band2_cutoff >> 4) & 0x3F) | ((self.band2_q << 6) & 0xC0)
		bytes[8] = ((self.band2_q >> 2) & 0x1F) | ((self.band2_gain << 5) & 0xE0)
		bytes[9] = ((self.band2_gain >> 3) & 0xF) | ((self.band2_gain_mod_source << 4) & 0xF0)
		bytes[10] = ((self.band2_gain_mod_source >> 4) & 0x3) | ((self.band2_gain_mod_intensity << 2) & 0xFC)
		bytes[11] = ((self.band2_gain_mod_intensity >> 6) & 0x1) | ((self.band3_cutoff & 0x7F) << 1)
		bytes[12] = (self.band3_q & 0x7F) | ((self.band3_gain << 7) & 0x80)
		bytes[13] = ((self.band3_gain >> 1) & 0x3F) | ((self.band4_cutoff << 6) & 0xC0)
		bytes[14] = ((self.band4_cutoff >> 2) & 0x3F) | ((self.band4_q << 6) & 0xC0)
		bytes[15] = ((self.band4_q >> 2) & 0x1F) | ((self.band4_gain << 5) & 0xE0)
		bytes[16] = (self.band4_gain >> 3) & 0xF
		return struct.pack('<17B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX007 Stereo Parametric 4-Band EQ object'''
		json = []
		json.append(('name','FX007'))
		json.append(('description','Stereo Parametric 4-Band EQ'))
		json.append(['input_trim',self.input_trim])
		json.append(['band1_type',self.band1_type])
		json.append(['band4_type',self.band4_type])
		json.append(['band2_gain_mod_source',self.band2_gain_mod_source])
		json.append(['band2_gain_mod_intensity',self.band2_gain_mod_intensity])
		json.append(['band2_gain_mod_intensity_display',self.band2_gain_mod_intensity_display])
		json.append(['band1_cutoff',self.band1_cutoff])
		json.append(['band1_cutoff_display',self.band1_cutoff_display])
		json.append(['band1_q',self.band1_q])
		json.append(['band1_q_display',self.band1_q_display])
		json.append(['band1_gain',self.band1_gain])
		json.append(['band1_gain_display',self.band1_gain_display])
		json.append(['band2_cutoff',self.band2_cutoff])
		json.append(['band2_cutoff_display',self.band2_cutoff_display])
		json.append(['band2_q',self.band2_q])
		json.append(['band2_q_display',self.band2_q_display])
		json.append(['band2_gain',self.band2_gain])
		json.append(['band2_gain_display',self.band2_gain_display])
		json.append(['band3_cutoff',self.band3_cutoff])
		json.append(['band3_cutoff_display',self.band3_cutoff_display])
		json.append(['band3_q',self.band3_q])
		json.append(['band3_q_display',self.band3_q_display])
		json.append(['band3_gain',self.band3_gain])
		json.append(['band3_gain_display',self.band3_gain_display])
		json.append(['band4_cutoff',self.band4_cutoff])
		json.append(['band4_cutoff_display',self.band4_cutoff_display])
		json.append(['band4_q',self.band4_q])
		json.append(['band4_q_display',self.band4_q_display])
		json.append(['band4_gain',self.band4_gain])
		json.append(['band4_gain_display',self.band4_gain_display])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def band1_type(self):
		'''Band1 Type'''
		return self._band1_type

	@band1_type.setter
	def band1_type(self,value):
		if 0 <= value <= 1:
			self._band1_type = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def band4_type(self):
		'''Band4 Type'''
		return self._band4_type

	@band4_type.setter
	def band4_type(self,value):
		if 0 <= value <= 1:
			self._band4_type = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def band1_cutoff(self):
		'''Band1 Cutoff'''
		return self._band1_cutoff

	@band1_cutoff.setter
	def band1_cutoff(self,value):
		if 0 <= value <= 98:
			self._band1_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 98' % value)

	@property
	def band1_cutoff_display(self):
		'''Band1 Cutoff display value'''
		dv = 20 + self._band1_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band1_q(self):
		'''Band1 Q'''
		return self._band1_q

	@band1_q.setter
	def band1_q(self,value):
		if 0 <= value <= 95:
			self._band1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

	@property
	def band1_q_display(self):
		'''Band1 Q display value'''
		dv = 0.5 + self._band1_q * 0.1
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band1_gain(self):
		'''Band1 Gain'''
		return self._band1_gain

	@band1_gain.setter
	def band1_gain(self,value):
		if -36 <= value <= 36:
			self._band1_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band1_gain_display(self):
		'''Band1 Gain display value'''
		dv = self._band1_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band2_cutoff(self):
		'''Band2 Cutoff'''
		return self._band2_cutoff

	@band2_cutoff.setter
	def band2_cutoff(self,value):
		if 0 <= value <= 995:
			self._band2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 995' % value)

	@property
	def band2_cutoff_display(self):
		'''Band2 Cutoff display value'''
		dv = 50 + self._band2_cutoff * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band2_q(self):
		'''Band2 Q'''
		return self._band2_q

	@band2_q.setter
	def band2_q(self,value):
		if 0 <= value <= 95:
			self._band2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

	@property
	def band2_q_display(self):
		'''Band2 Q display value'''
		dv = 0.5 + self._band2_q * 0.1
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band2_gain(self):
		'''Band2 Gain'''
		return self._band2_gain

	@band2_gain.setter
	def band2_gain(self,value):
		if -36 <= value <= 36:
			self._band2_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band2_gain_display(self):
		'''Band2 Gain display value'''
		dv = self._band2_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band2_gain_mod_source(self):
		'''Band2 Gain Mod.Source'''
		return self._band2_gain_mod_source

	@band2_gain_mod_source.setter
	def band2_gain_mod_source(self,value):
		if 0 <= value <= 37:
			self._band2_gain_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def band2_gain_mod_intensity(self):
		'''Band2 Gain Mod.Intensity'''
		return self._band2_gain_mod_intensity

	@band2_gain_mod_intensity.setter
	def band2_gain_mod_intensity(self,value):
		if -36 <= value <= 36:
			self._band2_gain_mod_intensity = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band2_gain_mod_intensity_display(self):
		'''Band2 Gain Mod.Intensity display value'''
		dv = self._band2_gain_mod_intensity * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band3_cutoff(self):
		'''Band3 Cutoff'''
		return self._band3_cutoff

	@band3_cutoff.setter
	def band3_cutoff(self,value):
		if 0 <= value <= 97:
			self._band3_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 97' % value)

	@property
	def band3_cutoff_display(self):
		'''Band3 Cutoff display value'''
		dv = 300 + self._band3_cutoff * 100
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band3_q(self):
		'''Band3 Q'''
		return self._band3_q

	@band3_q.setter
	def band3_q(self,value):
		if 0 <= value <= 95:
			self._band3_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

	@property
	def band3_q_display(self):
		'''Band3 Q display value'''
		dv = 0.5 + self._band3_q * 0.1
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band3_gain(self):
		'''Band3 Gain'''
		return self._band3_gain

	@band3_gain.setter
	def band3_gain(self,value):
		if -36 <= value <= 36:
			self._band3_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band3_gain_display(self):
		'''Band3 Gain display value'''
		dv = self._band3_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band4_cutoff(self):
		'''Band4 Cutoff'''
		return self._band4_cutoff

	@band4_cutoff.setter
	def band4_cutoff(self,value):
		if 0 <= value <= 195:
			self._band4_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 195' % value)

	@property
	def band4_cutoff_display(self):
		'''Band4 Cutoff display value'''
		dv = 500 + self._band4_cutoff * 100
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band4_q(self):
		'''Band4 Q'''
		return self._band4_q

	@band4_q.setter
	def band4_q(self,value):
		if 0 <= value <= 95:
			self._band4_q = value
		else:
			raise ValueError('%s not 0 <= value <= 95' % value)

	@property
	def band4_q_display(self):
		'''Band4 Q display value'''
		dv = 0.5 + self._band4_q * 0.1
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band4_gain(self):
		'''Band4 Gain'''
		return self._band4_gain

	@band4_gain.setter
	def band4_gain(self,value):
		if -36 <= value <= 36:
			self._band4_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band4_gain_display(self):
		'''Band4 Gain display value'''
		dv = self._band4_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

