#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx008.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX008(object):
	category = '''Eq/Filter'''
	description = '''Stereo Graphic 7-Band EQ'''
	short_description = '''St. Graphic 7EQ'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX008 Stereo Graphic 7-Band EQ object'''
		bytes = struct.unpack('<11B',raw[:11])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.type = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1) << 3)
		data = (bytes[4] >> 1) & 0x7F
		if data & 0x40:
			data -= 0x80
		self.band1_gain = data
		data = bytes[5] & 0x7F
		if data & 0x40:
			data -= 0x80
		self.band2_gain = data
		data = ((bytes[5] & 0x80) >> 7) | ((bytes[6] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.band3_gain = data
		data = ((bytes[6] & 0xC0) >> 6) | ((bytes[7] & 0x1F) << 2)
		if data & 0x40:
			data -= 0x80
		self.band4_gain = data
		data = ((bytes[7] & 0xE0) >> 5) | ((bytes[8] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.band5_gain = data
		data = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.band6_gain = data
		data = ((bytes[9] & 0xF8) >> 3) | ((bytes[10] & 0x3) << 5)
		if data & 0x40:
			data -= 0x80
		self.band7_gain = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX008 Stereo Graphic 7-Band EQ object into raw data'''
		bytes = [0] * 11
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.type << 5) & 0xE0)
		bytes[4] = ((self.type >> 3) & 0x1) | ((self.band1_gain & 0x7F) << 1)
		bytes[5] = (self.band2_gain & 0x7F) | ((self.band3_gain << 7) & 0x80)
		bytes[6] = ((self.band3_gain >> 1) & 0x3F) | ((self.band4_gain << 6) & 0xC0)
		bytes[7] = ((self.band4_gain >> 2) & 0x1F) | ((self.band5_gain << 5) & 0xE0)
		bytes[8] = ((self.band5_gain >> 3) & 0xF) | ((self.band6_gain << 4) & 0xF0)
		bytes[9] = ((self.band6_gain >> 4) & 0x7) | ((self.band7_gain << 3) & 0xF8)
		bytes[10] = (self.band7_gain >> 5) & 0x3
		return struct.pack('<11B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX008 Stereo Graphic 7-Band EQ object'''
		json = []
		json.append(('name','FX008'))
		json.append(('description','Stereo Graphic 7-Band EQ'))
		json.append(['type',self.type])
		json.append(['input_trim',self.input_trim])
		json.append(['band1_gain',self.band1_gain])
		json.append(['band1_gain_display',self.band1_gain_display])
		json.append(['band2_gain',self.band2_gain])
		json.append(['band2_gain_display',self.band2_gain_display])
		json.append(['band3_gain',self.band3_gain])
		json.append(['band3_gain_display',self.band3_gain_display])
		json.append(['band4_gain',self.band4_gain])
		json.append(['band4_gain_display',self.band4_gain_display])
		json.append(['band5_gain',self.band5_gain])
		json.append(['band5_gain_display',self.band5_gain_display])
		json.append(['band6_gain',self.band6_gain])
		json.append(['band6_gain_display',self.band6_gain_display])
		json.append(['band7_gain',self.band7_gain])
		json.append(['band7_gain_display',self.band7_gain_display])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def type(self):
		'''Type'''
		return self._type

	@type.setter
	def type(self,value):
		if 0 <= value <= 11:
			self._type = value
		else:
			raise ValueError('%s not 0 <= value <= 11' % value)

#-------------------------------------------------------------------------------
	@property
	def band1_gain(self):
		'''Band1 Gain'''
		return self._band1_gain

	@band1_gain.setter
	def band1_gain(self,value):
		if -36 <= value <= 36:
			self._band1_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band1_gain_display(self):
		'''Band1 Gain display value'''
		dv = self._band1_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band2_gain(self):
		'''Band2 Gain'''
		return self._band2_gain

	@band2_gain.setter
	def band2_gain(self,value):
		if -36 <= value <= 36:
			self._band2_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band2_gain_display(self):
		'''Band2 Gain display value'''
		dv = self._band2_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band3_gain(self):
		'''Band3 Gain'''
		return self._band3_gain

	@band3_gain.setter
	def band3_gain(self,value):
		if -36 <= value <= 36:
			self._band3_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band3_gain_display(self):
		'''Band3 Gain display value'''
		dv = self._band3_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band4_gain(self):
		'''Band4 Gain'''
		return self._band4_gain

	@band4_gain.setter
	def band4_gain(self,value):
		if -36 <= value <= 36:
			self._band4_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band4_gain_display(self):
		'''Band4 Gain display value'''
		dv = self._band4_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band5_gain(self):
		'''Band5 Gain'''
		return self._band5_gain

	@band5_gain.setter
	def band5_gain(self,value):
		if -36 <= value <= 36:
			self._band5_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band5_gain_display(self):
		'''Band5 Gain display value'''
		dv = self._band5_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band6_gain(self):
		'''Band6 Gain'''
		return self._band6_gain

	@band6_gain.setter
	def band6_gain(self,value):
		if -36 <= value <= 36:
			self._band6_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band6_gain_display(self):
		'''Band6 Gain display value'''
		dv = self._band6_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def band7_gain(self):
		'''Band7 Gain'''
		return self._band7_gain

	@band7_gain.setter
	def band7_gain(self,value):
		if -36 <= value <= 36:
			self._band7_gain = value
		else:
			raise ValueError('%s not -36 <= value <= 36' % value)

	@property
	def band7_gain_display(self):
		'''Band7 Gain display value'''
		dv = self._band7_gain * 0.5
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

