#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx010.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX010(object):
	category = '''Eq/Filter'''
	description = '''Stereo Isolator'''
	short_description = '''Stereo Isolator'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX010 Stereo Isolator object'''
		bytes = struct.unpack('<13B',raw[:13])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.crossover_low_mid = (bytes[3] >> 5) & 0x7
		self.crossover_mid_high = bytes[4] & 0x7
		data = ((bytes[4] & 0xF8) >> 3) | ((bytes[5] & 0x3) << 5)
		if data & 0x40:
			data -= 0x80
		self.low_gain = data
		self.low_gain_mod_source = (bytes[5] >> 2) & 0x3F
		data = bytes[6] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.low_gain_mod_intensity = data
		data = bytes[7] & 0x7F
		if data & 0x40:
			data -= 0x80
		self.mid_gain = data
		self.mid_gain_mod_source = ((bytes[7] & 0x80) >> 7) | ((bytes[8] & 0x1F) << 1)
		data = ((bytes[8] & 0xE0) >> 5) | ((bytes[9] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.mid_gain_mod_intensity = data
		data = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.high_gain = data
		self.high_gain_mod_source = ((bytes[10] & 0xF0) >> 4) | ((bytes[11] & 0x3) << 4)
		data = ((bytes[11] & 0xFC) >> 2) | ((bytes[12] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.high_gain_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX010 Stereo Isolator object into raw data'''
		bytes = [0] * 13
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.crossover_low_mid & 0x7) << 5)
		bytes[4] = (self.crossover_mid_high & 0x7) | ((self.low_gain << 3) & 0xF8)
		bytes[5] = ((self.low_gain >> 5) & 0x3) | ((self.low_gain_mod_source & 0x3F) << 2)
		bytes[6] = self.low_gain_mod_intensity & 0xFF
		bytes[7] = (self.mid_gain & 0x7F) | ((self.mid_gain_mod_source << 7) & 0x80)
		bytes[8] = ((self.mid_gain_mod_source >> 1) & 0x1F) | ((self.mid_gain_mod_intensity << 5) & 0xE0)
		bytes[9] = ((self.mid_gain_mod_intensity >> 3) & 0x1F) | ((self.high_gain << 5) & 0xE0)
		bytes[10] = ((self.high_gain >> 3) & 0xF) | ((self.high_gain_mod_source << 4) & 0xF0)
		bytes[11] = ((self.high_gain_mod_source >> 4) & 0x3) | ((self.high_gain_mod_intensity << 2) & 0xFC)
		bytes[12] = (self.high_gain_mod_intensity >> 6) & 0x3
		return struct.pack('<13B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX010 Stereo Isolator object'''
		json = []
		json.append(('name','FX010'))
		json.append(('description','Stereo Isolator'))
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['input_trim',self.input_trim])
		json.append(['crossover_low_mid',self.crossover_low_mid])
		json.append(['crossover_mid_high',self.crossover_mid_high])
		json.append(['low_gain',self.low_gain])
		json.append(['low_gain_mod_source',self.low_gain_mod_source])
		json.append(['low_gain_mod_intensity',self.low_gain_mod_intensity])
		json.append(['mid_gain',self.mid_gain])
		json.append(['mid_gain_mod_source',self.mid_gain_mod_source])
		json.append(['mid_gain_mod_intensity',self.mid_gain_mod_intensity])
		json.append(['high_gain',self.high_gain])
		json.append(['high_gain_mod_source',self.high_gain_mod_source])
		json.append(['high_gain_mod_intensity',self.high_gain_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def crossover_low_mid(self):
		'''Crossover Low/Mid'''
		return self._crossover_low_mid

	@crossover_low_mid.setter
	def crossover_low_mid(self,value):
		if 0 <= value <= 4:
			self._crossover_low_mid = value
		else:
			raise ValueError('%s not 0 <= value <= 4' % value)

#-------------------------------------------------------------------------------
	@property
	def crossover_mid_high(self):
		'''Crossover Mid/High'''
		return self._crossover_mid_high

	@crossover_mid_high.setter
	def crossover_mid_high(self,value):
		if 0 <= value <= 4:
			self._crossover_mid_high = value
		else:
			raise ValueError('%s not 0 <= value <= 4' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -60 <= value <= 12:
			self._low_gain = value
		else:
			raise ValueError('%s not -60 <= value <= 12' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain_mod_source(self):
		'''Low Gain Mod.Source'''
		return self._low_gain_mod_source

	@low_gain_mod_source.setter
	def low_gain_mod_source(self,value):
		if 0 <= value <= 37:
			self._low_gain_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain_mod_intensity(self):
		'''Low Gain Mod.Intensity'''
		return self._low_gain_mod_intensity

	@low_gain_mod_intensity.setter
	def low_gain_mod_intensity(self,value):
		if -72 <= value <= 72:
			self._low_gain_mod_intensity = value
		else:
			raise ValueError('%s not -72 <= value <= 72' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain(self):
		'''Mid Gain'''
		return self._mid_gain

	@mid_gain.setter
	def mid_gain(self,value):
		if -60 <= value <= 12:
			self._mid_gain = value
		else:
			raise ValueError('%s not -60 <= value <= 12' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain_mod_source(self):
		'''Mid Gain Mod.Source'''
		return self._mid_gain_mod_source

	@mid_gain_mod_source.setter
	def mid_gain_mod_source(self,value):
		if 0 <= value <= 37:
			self._mid_gain_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_gain_mod_intensity(self):
		'''Mid Gain Mod.Intensity'''
		return self._mid_gain_mod_intensity

	@mid_gain_mod_intensity.setter
	def mid_gain_mod_intensity(self,value):
		if -72 <= value <= 72:
			self._mid_gain_mod_intensity = value
		else:
			raise ValueError('%s not -72 <= value <= 72' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -60 <= value <= 12:
			self._high_gain = value
		else:
			raise ValueError('%s not -60 <= value <= 12' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain_mod_source(self):
		'''High Gain Mod.Source'''
		return self._high_gain_mod_source

	@high_gain_mod_source.setter
	def high_gain_mod_source(self,value):
		if 0 <= value <= 37:
			self._high_gain_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain_mod_intensity(self):
		'''High Gain Mod.Intensity'''
		return self._high_gain_mod_intensity

	@high_gain_mod_intensity.setter
	def high_gain_mod_intensity(self,value):
		if -72 <= value <= 72:
			self._high_gain_mod_intensity = value
		else:
			raise ValueError('%s not -72 <= value <= 72' % value)

