#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx011.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX011(object):
	category = '''Eq/Filter'''
	description = '''Stereo Wah/Auto Wah'''
	short_description = '''St. Wah/Auto Wah'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX011 Stereo Wah/Auto Wah object'''
		bytes = struct.unpack('<20B',raw[:20])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.frequency_bottom = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.frequency_top = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.sweep_mode = (bytes[5] >> 3) & 0x3
		self.sweep_mode_mod_source = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x7) << 3)
		self.response = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		self.envelope_sens = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		data = ((bytes[8] & 0xFE) >> 1) | ((bytes[9] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.envelope_shape = data
		self.lfo_type = (bytes[9] >> 1) & 0x3
		data = ((bytes[9] & 0xF8) >> 3) | ((bytes[10] & 0x1) << 5)
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_frequency = ((bytes[10] & 0xFE) >> 1) | ((bytes[11] & 0x1) << 7)
		self.lfo_frequency_mod_source = (bytes[11] >> 1) & 0x3F
		data = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.lfo_frequency_mod_intensity = data
		self.lfo_bpm_midi_sync = (bytes[12] >> 7) & 0x1
		self.lfo_bpm = (bytes[13] & 0xFF) | ((bytes[14] & 0x7F) << 8)
		self.lfo_bpm_base_note = ((bytes[14] & 0x80) >> 7) | ((bytes[15] & 0x7) << 1)
		self.lfo_bpm_times = ((bytes[15] & 0xF8) >> 3) | ((bytes[16] & 0x1) << 5)
		self.resonance = (bytes[16] >> 1) & 0x7F
		self.low_pass_filter_switch = bytes[17] & 0x1
		self.output_level = (bytes[17] >> 1) & 0x7F
		self.output_level_mod_source = bytes[18] & 0x3F
		data = ((bytes[18] & 0xC0) >> 6) | ((bytes[19] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.output_level_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX011 Stereo Wah/Auto Wah object into raw data'''
		bytes = [0] * 20
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.frequency_bottom << 5) & 0xE0
		bytes[4] = ((self.frequency_bottom >> 3) & 0xF) | ((self.frequency_top << 4) & 0xF0)
		bytes[5] = ((self.frequency_top >> 4) & 0x7) | ((self.sweep_mode & 0x3) << 3) | ((self.sweep_mode_mod_source << 5) & 0xE0)
		bytes[6] = ((self.sweep_mode_mod_source >> 3) & 0x7) | ((self.response << 3) & 0xF8)
		bytes[7] = ((self.response >> 5) & 0x3) | ((self.envelope_sens << 2) & 0xFC)
		bytes[8] = ((self.envelope_sens >> 6) & 0x1) | ((self.envelope_shape << 1) & 0xFE)
		bytes[9] = ((self.envelope_shape >> 7) & 0x1) | ((self.lfo_type & 0x3) << 1) | ((self.common_lfo_phase_offset << 3) & 0xF8)
		bytes[10] = ((self.common_lfo_phase_offset >> 5) & 0x1) | ((self.lfo_frequency << 1) & 0xFE)
		bytes[11] = ((self.lfo_frequency >> 7) & 0x1) | ((self.lfo_frequency_mod_source & 0x3F) << 1) | ((self.lfo_frequency_mod_intensity << 7) & 0x80)
		bytes[12] = ((self.lfo_frequency_mod_intensity >> 1) & 0x7F) | ((self.lfo_bpm_midi_sync & 0x1) << 7)
		bytes[13] = self.lfo_bpm & 0xFF
		bytes[14] = ((self.lfo_bpm >> 8) & 0x7F) | ((self.lfo_bpm_base_note << 7) & 0x80)
		bytes[15] = ((self.lfo_bpm_base_note >> 1) & 0x7) | ((self.lfo_bpm_times << 3) & 0xF8)
		bytes[16] = ((self.lfo_bpm_times >> 5) & 0x1) | ((self.resonance & 0x7F) << 1)
		bytes[17] = (self.low_pass_filter_switch & 0x1) | ((self.output_level & 0x7F) << 1)
		bytes[18] = (self.output_level_mod_source & 0x3F) | ((self.output_level_mod_intensity << 6) & 0xC0)
		bytes[19] = (self.output_level_mod_intensity >> 2) & 0x3F
		return struct.pack('<20B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX011 Stereo Wah/Auto Wah object'''
		json = []
		json.append(('name','FX011'))
		json.append(('description','Stereo Wah/Auto Wah'))
		json.append(['frequency_bottom',self.frequency_bottom])
		json.append(['frequency_top',self.frequency_top])
		json.append(['sweep_mode',self.sweep_mode])
		json.append(['sweep_mode_mod_source',self.sweep_mode_mod_source])
		json.append(['response',self.response])
		json.append(['envelope_sens',self.envelope_sens])
		json.append(['envelope_shape',self.envelope_shape])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_frequency_mod_source',self.lfo_frequency_mod_source])
		json.append(['lfo_frequency_mod_intensity',self.lfo_frequency_mod_intensity])
		json.append(['lfo_frequency_mod_intensity_display',self.lfo_frequency_mod_intensity_display])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		json.append(['resonance',self.resonance])
		json.append(['low_pass_filter_switch',self.low_pass_filter_switch])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		json.append(['output_level',self.output_level])
		json.append(['output_level_mod_source',self.output_level_mod_source])
		json.append(['output_level_mod_intensity',self.output_level_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def frequency_bottom(self):
		'''Frequency Bottom'''
		return self._frequency_bottom

	@frequency_bottom.setter
	def frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def frequency_top(self):
		'''Frequency Top'''
		return self._frequency_top

	@frequency_top.setter
	def frequency_top(self,value):
		if 0 <= value <= 100:
			self._frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def sweep_mode(self):
		'''Sweep Mode'''
		return self._sweep_mode

	@sweep_mode.setter
	def sweep_mode(self,value):
		if 0 <= value <= 2:
			self._sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def sweep_mode_mod_source(self):
		'''Sweep Mode Mod.Source'''
		return self._sweep_mode_mod_source

	@sweep_mode_mod_source.setter
	def sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def response(self):
		'''Response'''
		return self._response

	@response.setter
	def response(self,value):
		if 0 <= value <= 100:
			self._response = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_sens(self):
		'''Envelope Sens'''
		return self._envelope_sens

	@envelope_sens.setter
	def envelope_sens(self,value):
		if 0 <= value <= 100:
			self._envelope_sens = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_shape(self):
		'''Envelope Shape'''
		return self._envelope_shape

	@envelope_shape.setter
	def envelope_shape(self,value):
		if -100 <= value <= 100:
			self._envelope_shape = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_source(self):
		'''LFO Frequency Mod.Source'''
		return self._lfo_frequency_mod_source

	@lfo_frequency_mod_source.setter
	def lfo_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._lfo_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_intensity(self):
		'''LFO Frequency Mod.Intensity'''
		return self._lfo_frequency_mod_intensity

	@lfo_frequency_mod_intensity.setter
	def lfo_frequency_mod_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def lfo_frequency_mod_intensity_display(self):
		'''LFO Frequency Mod.Intensity display value'''
		return rng_8d_73_m20_p20_decode(self._lfo_frequency_mod_intensity)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def resonance(self):
		'''Resonance'''
		return self._resonance

	@resonance.setter
	def resonance(self,value):
		if 0 <= value <= 100:
			self._resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_pass_filter_switch(self):
		'''Low Pass Filter Switch'''
		return self._low_pass_filter_switch

	@low_pass_filter_switch.setter
	def low_pass_filter_switch(self,value):
		if 0 <= value <= 1:
			self._low_pass_filter_switch = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level(self):
		'''Output Level'''
		return self._output_level

	@output_level.setter
	def output_level(self,value):
		if 0 <= value <= 100:
			self._output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level_mod_source(self):
		'''Output Level Mod.Source'''
		return self._output_level_mod_source

	@output_level_mod_source.setter
	def output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level_mod_intensity(self):
		'''Output Level Mod.Intensity'''
		return self._output_level_mod_intensity

	@output_level_mod_intensity.setter
	def output_level_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._output_level_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

