#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx012.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX012(object):
	category = '''Eq/Filter'''
	description = '''Stereo Vintage/Custom Wah'''
	short_description = '''St. Vintage Wah'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX012 Stereo Vintage/Custom Wah object'''
		bytes = struct.unpack('<22B',raw[:22])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.mode = (bytes[3] >> 5) & 0x1
		self.frequency_bottom = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		self.frequency_top = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.resonance_bottom = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		self.resonance_top = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		data = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.shape = data
		self.invert = (bytes[8] >> 2) & 0x1
		self.sweep_mode = (bytes[8] >> 3) & 0x3
		self.manual = ((bytes[8] & 0xE0) >> 5) | ((bytes[9] & 0xF) << 3)
		self.sweep_mod_source = ((bytes[9] & 0xF0) >> 4) | ((bytes[10] & 0x3) << 4)
		self.response = ((bytes[10] & 0xFC) >> 2) | ((bytes[11] & 0x1) << 6)
		self.envelope_sens = (bytes[11] >> 1) & 0x7F
		self.lfo_type = bytes[12] & 0x3
		data = (bytes[12] >> 2) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_frequency = bytes[13] & 0xFF
		self.lfo_frequency_mod_source = bytes[14] & 0x3F
		data = ((bytes[14] & 0xC0) >> 6) | ((bytes[15] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.lfo_frequency_mod_intensity = data
		self.lfo_bpm_midi_sync = (bytes[15] >> 6) & 0x1
		self.lfo_bpm = ((bytes[15] & 0x80) >> 7) | (bytes[16] << 1) | ((bytes[17] & 0x3F) << 9)
		self.lfo_bpm_base_note = ((bytes[17] & 0xC0) >> 6) | ((bytes[18] & 0x3) << 2)
		self.lfo_bpm_times = (bytes[18] >> 2) & 0x3F
		self.output_level = bytes[19] & 0x7F
		self.output_level_mod_source = ((bytes[19] & 0x80) >> 7) | ((bytes[20] & 0x1F) << 1)
		data = ((bytes[20] & 0xE0) >> 5) | ((bytes[21] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.output_level_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX012 Stereo Vintage/Custom Wah object into raw data'''
		bytes = [0] * 22
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.mode & 0x1) << 5) | ((self.frequency_bottom << 6) & 0xC0)
		bytes[4] = ((self.frequency_bottom >> 2) & 0x1F) | ((self.frequency_top << 5) & 0xE0)
		bytes[5] = ((self.frequency_top >> 3) & 0xF) | ((self.resonance_bottom << 4) & 0xF0)
		bytes[6] = ((self.resonance_bottom >> 4) & 0x7) | ((self.resonance_top << 3) & 0xF8)
		bytes[7] = ((self.resonance_top >> 5) & 0x3) | ((self.shape << 2) & 0xFC)
		bytes[8] = ((self.shape >> 6) & 0x3) | ((self.invert & 0x1) << 2) | ((self.sweep_mode & 0x3) << 3) | ((self.manual << 5) & 0xE0)
		bytes[9] = ((self.manual >> 3) & 0xF) | ((self.sweep_mod_source << 4) & 0xF0)
		bytes[10] = ((self.sweep_mod_source >> 4) & 0x3) | ((self.response << 2) & 0xFC)
		bytes[11] = ((self.response >> 6) & 0x1) | ((self.envelope_sens & 0x7F) << 1)
		bytes[12] = (self.lfo_type & 0x3) | ((self.common_lfo_phase_offset & 0x3F) << 2)
		bytes[13] = self.lfo_frequency & 0xFF
		bytes[14] = (self.lfo_frequency_mod_source & 0x3F) | ((self.lfo_frequency_mod_intensity << 6) & 0xC0)
		bytes[15] = ((self.lfo_frequency_mod_intensity >> 2) & 0x3F) | ((self.lfo_bpm_midi_sync & 0x1) << 6) | ((self.lfo_bpm << 7) & 0x80)
		bytes[16] = (self.lfo_bpm >> 1) & 0xFF
		bytes[17] = ((self.lfo_bpm >> 63) & 0x9) | ((self.lfo_bpm_base_note << 6) & 0xC0)
		bytes[18] = ((self.lfo_bpm_base_note >> 2) & 0x3) | ((self.lfo_bpm_times & 0x3F) << 2)
		bytes[19] = (self.output_level & 0x7F) | ((self.output_level_mod_source << 7) & 0x80)
		bytes[20] = ((self.output_level_mod_source >> 1) & 0x1F) | ((self.output_level_mod_intensity << 5) & 0xE0)
		bytes[21] = (self.output_level_mod_intensity >> 3) & 0x1F
		return struct.pack('<22B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX012 Stereo Vintage/Custom Wah object'''
		json = []
		json.append(('name','FX012'))
		json.append(('description','Stereo Vintage/Custom Wah'))
		json.append(['mode',self.mode])
		json.append(['frequency_bottom',self.frequency_bottom])
		json.append(['frequency_top',self.frequency_top])
		json.append(['resonance_bottom',self.resonance_bottom])
		json.append(['resonance_top',self.resonance_top])
		json.append(['shape',self.shape])
		json.append(['invert',self.invert])
		json.append(['sweep_mode',self.sweep_mode])
		json.append(['manual',self.manual])
		json.append(['sweep_mod_source',self.sweep_mod_source])
		json.append(['response',self.response])
		json.append(['envelope_sens',self.envelope_sens])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_frequency_mod_source',self.lfo_frequency_mod_source])
		json.append(['lfo_frequency_mod_intensity',self.lfo_frequency_mod_intensity])
		json.append(['lfo_frequency_mod_intensity_display',self.lfo_frequency_mod_intensity_display])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		json.append(['output_level',self.output_level])
		json.append(['output_level_mod_source',self.output_level_mod_source])
		json.append(['output_level_mod_intensity',self.output_level_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mode(self):
		'''Mode'''
		return self._mode

	@mode.setter
	def mode(self,value):
		if 0 <= value <= 1:
			self._mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def frequency_bottom(self):
		'''Frequency Bottom'''
		return self._frequency_bottom

	@frequency_bottom.setter
	def frequency_bottom(self,value):
		if 0 <= value <= 100:
			self._frequency_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def frequency_top(self):
		'''Frequency Top'''
		return self._frequency_top

	@frequency_top.setter
	def frequency_top(self,value):
		if 0 <= value <= 100:
			self._frequency_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def resonance_bottom(self):
		'''Resonance Bottom'''
		return self._resonance_bottom

	@resonance_bottom.setter
	def resonance_bottom(self,value):
		if 0 <= value <= 100:
			self._resonance_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def resonance_top(self):
		'''Resonance Top'''
		return self._resonance_top

	@resonance_top.setter
	def resonance_top(self,value):
		if 0 <= value <= 100:
			self._resonance_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def shape(self):
		'''Shape'''
		return self._shape

	@shape.setter
	def shape(self,value):
		if -100 <= value <= 100:
			self._shape = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def invert(self):
		'''Invert'''
		return self._invert

	@invert.setter
	def invert(self,value):
		if 0 <= value <= 1:
			self._invert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def sweep_mode(self):
		'''Sweep Mode'''
		return self._sweep_mode

	@sweep_mode.setter
	def sweep_mode(self,value):
		if 0 <= value <= 2:
			self._sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def manual(self):
		'''Manual'''
		return self._manual

	@manual.setter
	def manual(self,value):
		if 0 <= value <= 100:
			self._manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def sweep_mod_source(self):
		'''Sweep Mod.Source'''
		return self._sweep_mod_source

	@sweep_mod_source.setter
	def sweep_mod_source(self,value):
		if 0 <= value <= 37:
			self._sweep_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def response(self):
		'''Response'''
		return self._response

	@response.setter
	def response(self,value):
		if 0 <= value <= 100:
			self._response = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_sens(self):
		'''Envelope Sens'''
		return self._envelope_sens

	@envelope_sens.setter
	def envelope_sens(self,value):
		if 0 <= value <= 100:
			self._envelope_sens = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_source(self):
		'''LFO Frequency Mod.Source'''
		return self._lfo_frequency_mod_source

	@lfo_frequency_mod_source.setter
	def lfo_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._lfo_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_intensity(self):
		'''LFO Frequency Mod.Intensity'''
		return self._lfo_frequency_mod_intensity

	@lfo_frequency_mod_intensity.setter
	def lfo_frequency_mod_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def lfo_frequency_mod_intensity_display(self):
		'''LFO Frequency Mod.Intensity display value'''
		return rng_8d_73_m20_p20_decode(self._lfo_frequency_mod_intensity)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level(self):
		'''Output Level'''
		return self._output_level

	@output_level.setter
	def output_level(self,value):
		if 0 <= value <= 100:
			self._output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level_mod_source(self):
		'''Output Level Mod.Source'''
		return self._output_level_mod_source

	@output_level_mod_source.setter
	def output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level_mod_intensity(self):
		'''Output Level Mod.Intensity'''
		return self._output_level_mod_intensity

	@output_level_mod_intensity.setter
	def output_level_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._output_level_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

