#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx013.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX013(object):
	category = '''Eq/Filter'''
	description = '''VOX Wah'''
	short_description = '''VOX Wah'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX013 VOX Wah object'''
		bytes = struct.unpack('<10B',raw[:10])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.type = (bytes[3] >> 5) & 0x1
		self.open = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		self.close = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.mode = (bytes[5] >> 4) & 0x1
		self.pedal_source = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x7) << 3)
		self.pedal_manual = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		self.auto_sensitivity = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x1) << 6)
		self.auto_polarity = (bytes[8] >> 1) & 0x1
		self.auto_attack = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x1) << 6)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX013 VOX Wah object into raw data'''
		bytes = [0] * 10
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.type & 0x1) << 5) | ((self.open << 6) & 0xC0)
		bytes[4] = ((self.open >> 2) & 0x1F) | ((self.close << 5) & 0xE0)
		bytes[5] = ((self.close >> 3) & 0xF) | ((self.mode & 0x1) << 4) | ((self.pedal_source << 5) & 0xE0)
		bytes[6] = ((self.pedal_source >> 3) & 0x7) | ((self.pedal_manual << 3) & 0xF8)
		bytes[7] = ((self.pedal_manual >> 5) & 0x3) | ((self.auto_sensitivity << 2) & 0xFC)
		bytes[8] = ((self.auto_sensitivity >> 6) & 0x1) | ((self.auto_polarity & 0x1) << 1) | ((self.auto_attack << 2) & 0xFC)
		bytes[9] = (self.auto_attack >> 6) & 0x1
		return struct.pack('<10B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX013 VOX Wah object'''
		json = []
		json.append(('name','FX013'))
		json.append(('description','VOX Wah'))
		json.append(['type',self.type])
		json.append(['open',self.open])
		json.append(['close',self.close])
		json.append(['mode',self.mode])
		json.append(['pedal_source',self.pedal_source])
		json.append(['pedal_manual',self.pedal_manual])
		json.append(['auto_sensitivity',self.auto_sensitivity])
		json.append(['auto_polarity',self.auto_polarity])
		json.append(['auto_attack',self.auto_attack])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def type(self):
		'''Type'''
		return self._type

	@type.setter
	def type(self,value):
		if 0 <= value <= 1:
			self._type = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def open(self):
		'''Open'''
		return self._open

	@open.setter
	def open(self,value):
		if 0 <= value <= 99:
			self._open = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def close(self):
		'''Close'''
		return self._close

	@close.setter
	def close(self,value):
		if 0 <= value <= 99:
			self._close = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def mode(self):
		'''Mode'''
		return self._mode

	@mode.setter
	def mode(self,value):
		if 0 <= value <= 1:
			self._mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def pedal_source(self):
		'''Pedal Source'''
		return self._pedal_source

	@pedal_source.setter
	def pedal_source(self,value):
		if 0 <= value <= 37:
			self._pedal_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def pedal_manual(self):
		'''Pedal Manual'''
		return self._pedal_manual

	@pedal_manual.setter
	def pedal_manual(self,value):
		if 0 <= value <= 99:
			self._pedal_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def auto_sensitivity(self):
		'''Auto Sensitivity'''
		return self._auto_sensitivity

	@auto_sensitivity.setter
	def auto_sensitivity(self,value):
		if 0 <= value <= 100:
			self._auto_sensitivity = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def auto_polarity(self):
		'''Auto Polarity'''
		return self._auto_polarity

	@auto_polarity.setter
	def auto_polarity(self,value):
		if 0 <= value <= 1:
			self._auto_polarity = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def auto_attack(self):
		'''Auto Attack'''
		return self._auto_attack

	@auto_attack.setter
	def auto_attack(self,value):
		if 0 <= value <= 99:
			self._auto_attack = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

