#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx015.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX015(object):
	category = '''Eq/Filter'''
	description = '''Stereo Multi Mode Filter'''
	short_description = '''St. MultiModeFiltr'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX015 Stereo Multi Mode Filter object'''
		bytes = struct.unpack('<23B',raw[:23])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.trim = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.filter_type = (bytes[4] >> 4) & 0x3
		self.cutoff = ((bytes[4] & 0xC0) >> 6) | ((bytes[5] & 0x1F) << 2)
		self.cutoff_mod_source = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x7) << 3)
		data = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.cutoff_mod_intensity = data
		self.resonance = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x3) << 5)
		self.resonance_mod_source = (bytes[8] >> 2) & 0x3F
		data = bytes[9] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.resonance_mod_intensity = data
		self.lfo_type = bytes[10] & 0x3
		data = (bytes[10] >> 2) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_waveform = bytes[11] & 0x1
		data = (bytes[12] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.lfo_phase = data
		self.lfo_frequency = ((bytes[12] & 0x80) >> 7) | ((bytes[13] & 0x7F) << 1)
		self.lfo_frequency_mod_source = ((bytes[13] & 0x80) >> 7) | ((bytes[14] & 0x1F) << 1)
		data = ((bytes[14] & 0xE0) >> 5) | ((bytes[15] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.lfo_frequency_mod_intensity = data
		self.lfo_bpm_midi_sync = (bytes[15] >> 5) & 0x1
		self.lfo_bpm = ((bytes[15] & 0xC0) >> 6) | (bytes[16] << 2) | ((bytes[17] & 0x1F) << 10)
		self.lfo_bpm_base_note = ((bytes[17] & 0xE0) >> 5) | ((bytes[18] & 0x1) << 3)
		self.lfo_bpm_times = (bytes[18] >> 1) & 0x3F
		self.lfo_depth = ((bytes[18] & 0x80) >> 7) | ((bytes[19] & 0x3F) << 1)
		self.drive_gain = ((bytes[19] & 0xC0) >> 6) | ((bytes[20] & 0x1F) << 2)
		self.drive_low_boost = ((bytes[20] & 0xE0) >> 5) | ((bytes[21] & 0xF) << 3)
		self.drive_switch = (bytes[21] >> 4) & 0x1
		self.output_level = ((bytes[21] & 0xE0) >> 5) | ((bytes[22] & 0xF) << 3)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX015 Stereo Multi Mode Filter object into raw data'''
		bytes = [0] * 23
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.trim << 5) & 0xE0
		bytes[4] = ((self.trim >> 3) & 0xF) | ((self.filter_type & 0x3) << 4) | ((self.cutoff << 6) & 0xC0)
		bytes[5] = ((self.cutoff >> 2) & 0x1F) | ((self.cutoff_mod_source << 5) & 0xE0)
		bytes[6] = ((self.cutoff_mod_source >> 3) & 0x7) | ((self.cutoff_mod_intensity << 3) & 0xF8)
		bytes[7] = ((self.cutoff_mod_intensity >> 5) & 0x7) | ((self.resonance << 3) & 0xF8)
		bytes[8] = ((self.resonance >> 5) & 0x3) | ((self.resonance_mod_source & 0x3F) << 2)
		bytes[9] = self.resonance_mod_intensity & 0xFF
		bytes[10] = (self.lfo_type & 0x3) | ((self.common_lfo_phase_offset & 0x3F) << 2)
		bytes[11] = self.lfo_waveform & 0x1
		bytes[12] = ((self.lfo_phase & 0x3F) << 1) | ((self.lfo_frequency << 7) & 0x80)
		bytes[13] = ((self.lfo_frequency >> 1) & 0x7F) | ((self.lfo_frequency_mod_source << 7) & 0x80)
		bytes[14] = ((self.lfo_frequency_mod_source >> 1) & 0x1F) | ((self.lfo_frequency_mod_intensity << 5) & 0xE0)
		bytes[15] = ((self.lfo_frequency_mod_intensity >> 3) & 0x1F) | ((self.lfo_bpm_midi_sync & 0x1) << 5) | ((self.lfo_bpm << 6) & 0xC0)
		bytes[16] = (self.lfo_bpm >> 2) & 0xFF
		bytes[17] = ((self.lfo_bpm >> 31) & 0xA) | ((self.lfo_bpm_base_note << 5) & 0xE0)
		bytes[18] = ((self.lfo_bpm_base_note >> 3) & 0x1) | ((self.lfo_bpm_times & 0x3F) << 1) | ((self.lfo_depth << 7) & 0x80)
		bytes[19] = ((self.lfo_depth >> 1) & 0x3F) | ((self.drive_gain << 6) & 0xC0)
		bytes[20] = ((self.drive_gain >> 2) & 0x1F) | ((self.drive_low_boost << 5) & 0xE0)
		bytes[21] = ((self.drive_low_boost >> 3) & 0xF) | ((self.drive_switch & 0x1) << 4) | ((self.output_level << 5) & 0xE0)
		bytes[22] = (self.output_level >> 3) & 0xF
		return struct.pack('<23B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX015 Stereo Multi Mode Filter object'''
		json = []
		json.append(('name','FX015'))
		json.append(('description','Stereo Multi Mode Filter'))
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['trim',self.trim])
		json.append(['filter_type',self.filter_type])
		json.append(['cutoff',self.cutoff])
		json.append(['cutoff_mod_source',self.cutoff_mod_source])
		json.append(['cutoff_mod_intensity',self.cutoff_mod_intensity])
		json.append(['resonance',self.resonance])
		json.append(['resonance_mod_source',self.resonance_mod_source])
		json.append(['resonance_mod_intensity',self.resonance_mod_intensity])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		json.append(['lfo_waveform',self.lfo_waveform])
		json.append(['lfo_phase',self.lfo_phase])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_frequency_mod_source',self.lfo_frequency_mod_source])
		json.append(['lfo_frequency_mod_intensity',self.lfo_frequency_mod_intensity])
		json.append(['lfo_frequency_mod_intensity_display',self.lfo_frequency_mod_intensity_display])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		json.append(['lfo_depth',self.lfo_depth])
		json.append(['drive_gain',self.drive_gain])
		json.append(['drive_low_boost',self.drive_low_boost])
		json.append(['drive_switch',self.drive_switch])
		json.append(['output_level',self.output_level])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def trim(self):
		'''Trim'''
		return self._trim

	@trim.setter
	def trim(self,value):
		if 0 <= value <= 100:
			self._trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def filter_type(self):
		'''Filter Type'''
		return self._filter_type

	@filter_type.setter
	def filter_type(self,value):
		if 0 <= value <= 3:
			self._filter_type = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def cutoff(self):
		'''Cutoff'''
		return self._cutoff

	@cutoff.setter
	def cutoff(self,value):
		if 0 <= value <= 100:
			self._cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def cutoff_mod_source(self):
		'''Cutoff Mod.Source'''
		return self._cutoff_mod_source

	@cutoff_mod_source.setter
	def cutoff_mod_source(self,value):
		if 0 <= value <= 37:
			self._cutoff_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def cutoff_mod_intensity(self):
		'''Cutoff Mod.Intensity'''
		return self._cutoff_mod_intensity

	@cutoff_mod_intensity.setter
	def cutoff_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._cutoff_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def resonance(self):
		'''Resonance'''
		return self._resonance

	@resonance.setter
	def resonance(self,value):
		if 0 <= value <= 100:
			self._resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def resonance_mod_source(self):
		'''Resonance Mod.Source'''
		return self._resonance_mod_source

	@resonance_mod_source.setter
	def resonance_mod_source(self,value):
		if 0 <= value <= 37:
			self._resonance_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def resonance_mod_intensity(self):
		'''Resonance Mod.Intensity'''
		return self._resonance_mod_intensity

	@resonance_mod_intensity.setter
	def resonance_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._resonance_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_waveform(self):
		'''LFO Waveform'''
		return self._lfo_waveform

	@lfo_waveform.setter
	def lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_phase(self):
		'''LFO Phase'''
		return self._lfo_phase

	@lfo_phase.setter
	def lfo_phase(self,value):
		if -18 <= value <= 18:
			self._lfo_phase = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_source(self):
		'''LFO Frequency Mod.Source'''
		return self._lfo_frequency_mod_source

	@lfo_frequency_mod_source.setter
	def lfo_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._lfo_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_intensity(self):
		'''LFO Frequency Mod.Intensity'''
		return self._lfo_frequency_mod_intensity

	@lfo_frequency_mod_intensity.setter
	def lfo_frequency_mod_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def lfo_frequency_mod_intensity_display(self):
		'''LFO Frequency Mod.Intensity display value'''
		return rng_8d_73_m20_p20_decode(self._lfo_frequency_mod_intensity)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_depth(self):
		'''LFO Depth'''
		return self._lfo_depth

	@lfo_depth.setter
	def lfo_depth(self,value):
		if 0 <= value <= 100:
			self._lfo_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def drive_gain(self):
		'''Drive Gain'''
		return self._drive_gain

	@drive_gain.setter
	def drive_gain(self,value):
		if 0 <= value <= 100:
			self._drive_gain = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def drive_low_boost(self):
		'''Drive Low Boost'''
		return self._drive_low_boost

	@drive_low_boost.setter
	def drive_low_boost(self,value):
		if 0 <= value <= 100:
			self._drive_low_boost = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def drive_switch(self):
		'''Drive Switch'''
		return self._drive_switch

	@drive_switch.setter
	def drive_switch(self,value):
		if 0 <= value <= 1:
			self._drive_switch = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level(self):
		'''Output Level'''
		return self._output_level

	@output_level.setter
	def output_level(self,value):
		if 0 <= value <= 100:
			self._output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

