#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx016.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX016(object):
	category = '''Eq/Filter'''
	description = '''Stereo Sub Oscillator'''
	short_description = '''St. Sub Oscillator'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX016 Stereo Sub Oscillator object'''
		bytes = struct.unpack('<12B',raw[:12])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.osc_mode = (bytes[3] >> 5) & 0x1
		data = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		if data & 0x40:
			data -= 0x80
		self.note_interval = data
		data = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.note_fine = data
		self.fixed_frequency = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x1F) << 3)
		self.fixed_frequency_mod_source = ((bytes[6] & 0xE0) >> 5) | ((bytes[7] & 0x7) << 3)
		data = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.fixed_frequency_mod_intensity = data
		self.envelope_pre_lpf = ((bytes[8] & 0xF8) >> 3) | ((bytes[9] & 0x3) << 5)
		self.envelope_sens = ((bytes[9] & 0xFC) >> 2) | ((bytes[10] & 0x1) << 6)
		data = ((bytes[10] & 0xFE) >> 1) | ((bytes[11] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.envelope_shape = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX016 Stereo Sub Oscillator object into raw data'''
		bytes = [0] * 12
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.osc_mode & 0x1) << 5) | ((self.note_interval << 6) & 0xC0)
		bytes[4] = ((self.note_interval >> 2) & 0x1F) | ((self.note_fine << 5) & 0xE0)
		bytes[5] = ((self.note_fine >> 3) & 0x1F) | ((self.fixed_frequency << 5) & 0xE0)
		bytes[6] = ((self.fixed_frequency >> 3) & 0x1F) | ((self.fixed_frequency_mod_source << 5) & 0xE0)
		bytes[7] = ((self.fixed_frequency_mod_source >> 3) & 0x7) | ((self.fixed_frequency_mod_intensity << 3) & 0xF8)
		bytes[8] = ((self.fixed_frequency_mod_intensity >> 5) & 0x7) | ((self.envelope_pre_lpf << 3) & 0xF8)
		bytes[9] = ((self.envelope_pre_lpf >> 5) & 0x3) | ((self.envelope_sens << 2) & 0xFC)
		bytes[10] = ((self.envelope_sens >> 6) & 0x1) | ((self.envelope_shape << 1) & 0xFE)
		bytes[11] = (self.envelope_shape >> 7) & 0x1
		return struct.pack('<12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX016 Stereo Sub Oscillator object'''
		json = []
		json.append(('name','FX016'))
		json.append(('description','Stereo Sub Oscillator'))
		json.append(['osc_mode',self.osc_mode])
		json.append(['note_interval',self.note_interval])
		json.append(['note_fine',self.note_fine])
		json.append(['fixed_frequency',self.fixed_frequency])
		json.append(['fixed_frequency_mod_source',self.fixed_frequency_mod_source])
		json.append(['fixed_frequency_mod_intensity',self.fixed_frequency_mod_intensity])
		json.append(['envelope_pre_lpf',self.envelope_pre_lpf])
		json.append(['envelope_sens',self.envelope_sens])
		json.append(['envelope_shape',self.envelope_shape])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def osc_mode(self):
		'''Osc Mode'''
		return self._osc_mode

	@osc_mode.setter
	def osc_mode(self,value):
		if 0 <= value <= 1:
			self._osc_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def note_interval(self):
		'''Note Interval'''
		return self._note_interval

	@note_interval.setter
	def note_interval(self,value):
		if -48 <= value <= 0:
			self._note_interval = value
		else:
			raise ValueError('%s not -48 <= value <= 0' % value)

#-------------------------------------------------------------------------------
	@property
	def note_fine(self):
		'''Note Fine'''
		return self._note_fine

	@note_fine.setter
	def note_fine(self,value):
		if -100 <= value <= 100:
			self._note_fine = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def fixed_frequency(self):
		'''Fixed Frequency'''
		return self._fixed_frequency

	@fixed_frequency.setter
	def fixed_frequency(self,value):
		if 20 <= value <= 160:
			self._fixed_frequency = value
		else:
			raise ValueError('%s not 20 <= value <= 160' % value)

#-------------------------------------------------------------------------------
	@property
	def fixed_frequency_mod_source(self):
		'''Fixed Frequency Mod.Source'''
		return self._fixed_frequency_mod_source

	@fixed_frequency_mod_source.setter
	def fixed_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._fixed_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def fixed_frequency_mod_intensity(self):
		'''Fixed Frequency Mod.Intensity'''
		return self._fixed_frequency_mod_intensity

	@fixed_frequency_mod_intensity.setter
	def fixed_frequency_mod_intensity(self,value):
		if -80 <= value <= 80:
			self._fixed_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -80 <= value <= 80' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_pre_lpf(self):
		'''Envelope Pre LPF'''
		return self._envelope_pre_lpf

	@envelope_pre_lpf.setter
	def envelope_pre_lpf(self,value):
		if 1 <= value <= 100:
			self._envelope_pre_lpf = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_sens(self):
		'''Envelope Sens'''
		return self._envelope_sens

	@envelope_sens.setter
	def envelope_sens(self,value):
		if 0 <= value <= 100:
			self._envelope_sens = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_shape(self):
		'''Envelope Shape'''
		return self._envelope_shape

	@envelope_shape.setter
	def envelope_shape(self,value):
		if -100 <= value <= 100:
			self._envelope_shape = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

