#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx017.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX017(object):
	category = '''Eq/Filter'''
	description = '''Talking Modulator'''
	short_description = '''Talking Modulator'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX017 Talking Modulator object'''
		bytes = struct.unpack('<16B',raw[:16])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.sweep_mode = (bytes[3] >> 5) & 0x1
		self.manual_voice_control = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		self.sweep_mod_source = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x7) << 3)
		self.lfo_type = (bytes[5] >> 3) & 0x3
		data = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x7) << 3)
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_frequency = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x7) << 5)
		self.lfo_frequency_mod_source = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x1) << 5)
		data = ((bytes[8] & 0xFE) >> 1) | ((bytes[9] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.lfo_frequency_mod_intensity = data
		self.lfo_bpm_midi_sync = (bytes[9] >> 1) & 0x1
		self.lfo_bpm = ((bytes[9] & 0xFC) >> 2) | (bytes[10] << 6) | ((bytes[11] & 0x1) << 14)
		self.lfo_bpm_base_note = (bytes[11] >> 1) & 0xF
		self.lfo_bpm_times = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0x7) << 3)
		self.voice_top = (bytes[12] >> 3) & 0x7
		self.voice_center = ((bytes[12] & 0xC0) >> 6) | ((bytes[13] & 0x1) << 2)
		self.voice_bottom = (bytes[13] >> 1) & 0x7
		data = ((bytes[13] & 0xF0) >> 4) | ((bytes[14] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.formant_shift = data
		self.resonance = ((bytes[14] & 0xF0) >> 4) | ((bytes[15] & 0x7) << 4)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX017 Talking Modulator object into raw data'''
		bytes = [0] * 16
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.sweep_mode & 0x1) << 5) | ((self.manual_voice_control << 6) & 0xC0)
		bytes[4] = ((self.manual_voice_control >> 2) & 0x1F) | ((self.sweep_mod_source << 5) & 0xE0)
		bytes[5] = ((self.sweep_mod_source >> 3) & 0x7) | ((self.lfo_type & 0x3) << 3) | ((self.common_lfo_phase_offset << 5) & 0xE0)
		bytes[6] = ((self.common_lfo_phase_offset >> 3) & 0x7) | ((self.lfo_frequency << 3) & 0xF8)
		bytes[7] = ((self.lfo_frequency >> 5) & 0x7) | ((self.lfo_frequency_mod_source << 3) & 0xF8)
		bytes[8] = ((self.lfo_frequency_mod_source >> 5) & 0x1) | ((self.lfo_frequency_mod_intensity << 1) & 0xFE)
		bytes[9] = ((self.lfo_frequency_mod_intensity >> 7) & 0x1) | ((self.lfo_bpm_midi_sync & 0x1) << 1) | ((self.lfo_bpm << 2) & 0xFC)
		bytes[10] = (self.lfo_bpm >> 6) & 0xFF
		bytes[11] = ((self.lfo_bpm >> 1) & 0xE) | ((self.lfo_bpm_base_note & 0xF) << 1) | ((self.lfo_bpm_times << 5) & 0xE0)
		bytes[12] = ((self.lfo_bpm_times >> 3) & 0x7) | ((self.voice_top & 0x7) << 3) | ((self.voice_center << 6) & 0xC0)
		bytes[13] = ((self.voice_center >> 2) & 0x1) | ((self.voice_bottom & 0x7) << 1) | ((self.formant_shift << 4) & 0xF0)
		bytes[14] = ((self.formant_shift >> 4) & 0xF) | ((self.resonance << 4) & 0xF0)
		bytes[15] = (self.resonance >> 4) & 0x7
		return struct.pack('<16B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX017 Talking Modulator object'''
		json = []
		json.append(('name','FX017'))
		json.append(('description','Talking Modulator'))
		json.append(['sweep_mode',self.sweep_mode])
		json.append(['manual_voice_control',self.manual_voice_control])
		json.append(['sweep_mod_source',self.sweep_mod_source])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_frequency_mod_source',self.lfo_frequency_mod_source])
		json.append(['lfo_frequency_mod_intensity',self.lfo_frequency_mod_intensity])
		json.append(['lfo_frequency_mod_intensity_display',self.lfo_frequency_mod_intensity_display])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		json.append(['voice_top',self.voice_top])
		json.append(['voice_center',self.voice_center])
		json.append(['voice_bottom',self.voice_bottom])
		json.append(['formant_shift',self.formant_shift])
		json.append(['resonance',self.resonance])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def sweep_mode(self):
		'''Sweep Mode'''
		return self._sweep_mode

	@sweep_mode.setter
	def sweep_mode(self,value):
		if 0 <= value <= 1:
			self._sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def manual_voice_control(self):
		'''Manual Voice Control'''
		return self._manual_voice_control

	@manual_voice_control.setter
	def manual_voice_control(self,value):
		if 0 <= value <= 100:
			self._manual_voice_control = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def sweep_mod_source(self):
		'''Sweep Mod.Source'''
		return self._sweep_mod_source

	@sweep_mod_source.setter
	def sweep_mod_source(self,value):
		if 0 <= value <= 37:
			self._sweep_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_source(self):
		'''LFO Frequency Mod.Source'''
		return self._lfo_frequency_mod_source

	@lfo_frequency_mod_source.setter
	def lfo_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._lfo_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_intensity(self):
		'''LFO Frequency Mod.Intensity'''
		return self._lfo_frequency_mod_intensity

	@lfo_frequency_mod_intensity.setter
	def lfo_frequency_mod_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def lfo_frequency_mod_intensity_display(self):
		'''LFO Frequency Mod.Intensity display value'''
		return rng_8d_73_m20_p20_decode(self._lfo_frequency_mod_intensity)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def voice_top(self):
		'''Voice Top'''
		return self._voice_top

	@voice_top.setter
	def voice_top(self,value):
		if 0 <= value <= 4:
			self._voice_top = value
		else:
			raise ValueError('%s not 0 <= value <= 4' % value)

#-------------------------------------------------------------------------------
	@property
	def voice_center(self):
		'''Voice Center'''
		return self._voice_center

	@voice_center.setter
	def voice_center(self,value):
		if 0 <= value <= 4:
			self._voice_center = value
		else:
			raise ValueError('%s not 0 <= value <= 4' % value)

#-------------------------------------------------------------------------------
	@property
	def voice_bottom(self):
		'''Voice Bottom'''
		return self._voice_bottom

	@voice_bottom.setter
	def voice_bottom(self,value):
		if 0 <= value <= 4:
			self._voice_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 4' % value)

#-------------------------------------------------------------------------------
	@property
	def formant_shift(self):
		'''Formant Shift'''
		return self._formant_shift

	@formant_shift.setter
	def formant_shift(self,value):
		if -100 <= value <= 100:
			self._formant_shift = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def resonance(self):
		'''Resonance'''
		return self._resonance

	@resonance.setter
	def resonance(self,value):
		if 0 <= value <= 100:
			self._resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

