#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx018.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

off_on_dict = {0: 'Off', 1: 'On'}

def rng_01_e6_p0v02_p20v0_decode(v):
	return v*0.02 if v<=0x64 else 2.0+(v-0x64)*0.05 if v<=0xA0 else 5.0+(v-0xA0)*0.1 if v<=0xD2 else 10.0+(v-0xD2)*0.5

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX018(object):
	category = '''Eq/Filter'''
	description = '''Stereo Decimator'''
	short_description = '''Stereo Decimator'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX018 Stereo Decimator object'''
		bytes = struct.unpack('<21B',raw[:21])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.pre_lpf = (bytes[3] >> 5) & 0x1
		self.sampling_frequency = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x7F) << 2)
		self.sampling_frequency_mod_source = ((bytes[4] & 0x80) >> 7) | ((bytes[5] & 0x1F) << 1)
		data = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x3F) << 3)
		if data & 0x100:
			data -= 0x200
		self.sampling_frequency_mod_intensity = data
		self.high_damp = ((bytes[6] & 0xC0) >> 6) | ((bytes[7] & 0x1F) << 2)
		self.resolution = ((bytes[7] & 0xE0) >> 5) | ((bytes[8] & 0x3) << 3)
		self.output_level = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x1) << 6)
		self.output_level_mod_source = (bytes[9] >> 1) & 0x3F
		data = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.output_level_mod_intensity = data
		self.lfo_type = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0x1) << 1)
		data = (bytes[11] >> 1) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_frequency = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x7F) << 1)
		self.lfo_frequency_mod_source = ((bytes[12] & 0x80) >> 7) | ((bytes[13] & 0x1F) << 1)
		data = ((bytes[13] & 0xE0) >> 5) | ((bytes[14] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.lfo_frequency_mod_intensity = data
		self.lfo_bpm_midi_sync = (bytes[14] >> 5) & 0x1
		self.lfo_bpm = ((bytes[14] & 0xC0) >> 6) | (bytes[15] << 2) | ((bytes[16] & 0x1F) << 10)
		self.lfo_bpm_base_note = ((bytes[16] & 0xE0) >> 5) | ((bytes[17] & 0x1) << 3)
		self.lfo_bpm_times = (bytes[17] >> 1) & 0x3F
		self.depth = ((bytes[17] & 0x80) >> 7) | ((bytes[18] & 0x3F) << 1)
		self.depth_mod_source = ((bytes[18] & 0xC0) >> 6) | ((bytes[19] & 0xF) << 2)
		data = ((bytes[19] & 0xF0) >> 4) | ((bytes[20] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX018 Stereo Decimator object into raw data'''
		bytes = [0] * 21
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.pre_lpf & 0x1) << 5) | ((self.sampling_frequency << 6) & 0xC0)
		bytes[4] = ((self.sampling_frequency >> 2) & 0x7F) | ((self.sampling_frequency_mod_source << 7) & 0x80)
		bytes[5] = ((self.sampling_frequency_mod_source >> 1) & 0x1F) | ((self.sampling_frequency_mod_intensity << 5) & 0xE0)
		bytes[6] = ((self.sampling_frequency_mod_intensity >> 3) & 0x3F) | ((self.high_damp << 6) & 0xC0)
		bytes[7] = ((self.high_damp >> 2) & 0x1F) | ((self.resolution << 5) & 0xE0)
		bytes[8] = ((self.resolution >> 3) & 0x3) | ((self.output_level << 2) & 0xFC)
		bytes[9] = ((self.output_level >> 6) & 0x1) | ((self.output_level_mod_source & 0x3F) << 1) | ((self.output_level_mod_intensity << 7) & 0x80)
		bytes[10] = ((self.output_level_mod_intensity >> 1) & 0x7F) | ((self.lfo_type << 7) & 0x80)
		bytes[11] = ((self.lfo_type >> 1) & 0x1) | ((self.common_lfo_phase_offset & 0x3F) << 1) | ((self.lfo_frequency << 7) & 0x80)
		bytes[12] = ((self.lfo_frequency >> 1) & 0x7F) | ((self.lfo_frequency_mod_source << 7) & 0x80)
		bytes[13] = ((self.lfo_frequency_mod_source >> 1) & 0x1F) | ((self.lfo_frequency_mod_intensity << 5) & 0xE0)
		bytes[14] = ((self.lfo_frequency_mod_intensity >> 3) & 0x1F) | ((self.lfo_bpm_midi_sync & 0x1) << 5) | ((self.lfo_bpm << 6) & 0xC0)
		bytes[15] = (self.lfo_bpm >> 2) & 0xFF
		bytes[16] = ((self.lfo_bpm >> 31) & 0xA) | ((self.lfo_bpm_base_note << 5) & 0xE0)
		bytes[17] = ((self.lfo_bpm_base_note >> 3) & 0x1) | ((self.lfo_bpm_times & 0x3F) << 1) | ((self.depth << 7) & 0x80)
		bytes[18] = ((self.depth >> 1) & 0x3F) | ((self.depth_mod_source << 6) & 0xC0)
		bytes[19] = ((self.depth_mod_source >> 2) & 0xF) | ((self.depth_mod_intensity << 4) & 0xF0)
		bytes[20] = (self.depth_mod_intensity >> 4) & 0xF
		return struct.pack('<21B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX018 Stereo Decimator object'''
		json = []
		json.append(('name','FX018'))
		json.append(('description','Stereo Decimator'))
		json.append(['pre_lpf',self.pre_lpf])
		json.append(['pre_lpf_display',self.pre_lpf_display])
		json.append(['sampling_frequency',self.sampling_frequency])
		json.append(['sampling_frequency_display',self.sampling_frequency_display])
		json.append(['sampling_frequency_mod_source',self.sampling_frequency_mod_source])
		json.append(['sampling_frequency_mod_source_display',self.sampling_frequency_mod_source_display])
		json.append(['sampling_frequency_mod_intensity',self.sampling_frequency_mod_intensity])
		json.append(['sampling_frequency_mod_intensity_display',self.sampling_frequency_mod_intensity_display])
		json.append(['high_damp',self.high_damp])
		json.append(['resolution',self.resolution])
		json.append(['output_level',self.output_level])
		json.append(['output_level_mod_source',self.output_level_mod_source])
		json.append(['output_level_mod_source_display',self.output_level_mod_source_display])
		json.append(['output_level_mod_intensity',self.output_level_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_source_display',self.wet_dry_mod_source_display])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_frequency_display',self.lfo_frequency_display])
		json.append(['lfo_frequency_mod_source',self.lfo_frequency_mod_source])
		json.append(['lfo_frequency_mod_source_display',self.lfo_frequency_mod_source_display])
		json.append(['lfo_frequency_mod_intensity',self.lfo_frequency_mod_intensity])
		json.append(['lfo_frequency_mod_intensity_display',self.lfo_frequency_mod_intensity_display])
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_source_display',self.depth_mod_source_display])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['lfo_type',self.lfo_type])
		json.append(['lfo_type_display',self.lfo_type_display])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		json.append(['common_lfo_phase_offset_display',self.common_lfo_phase_offset_display])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm_midi_sync_display',self.lfo_bpm_midi_sync_display])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

	@property
	def wet_dry_mod_source_display(self):
		'''Wet/Dry Mod.Source display value'''
		return module_sources[self._wet_dry_mod_source].short_desc

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_lpf(self):
		'''Pre LPF'''
		return self._pre_lpf

	@pre_lpf.setter
	def pre_lpf(self,value):
		if 0 <= value <= 1:
			self._pre_lpf = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def pre_lpf_display(self):
		'''Pre LPF display value'''
		return off_on_dict[self._pre_lpf]

#-------------------------------------------------------------------------------
	@property
	def sampling_frequency(self):
		'''Sampling Frequency'''
		return self._sampling_frequency

	@sampling_frequency.setter
	def sampling_frequency(self,value):
		if 10 <= value <= 480:
			self._sampling_frequency = value
		else:
			raise ValueError('%s not 10 <= value <= 480' % value)

	@property
	def sampling_frequency_display(self):
		'''Sampling Frequency display value'''
		dv = self._sampling_frequency * 0.1
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def sampling_frequency_mod_source(self):
		'''Sampling Frequency Mod.Source'''
		return self._sampling_frequency_mod_source

	@sampling_frequency_mod_source.setter
	def sampling_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._sampling_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

	@property
	def sampling_frequency_mod_source_display(self):
		'''Sampling Frequency Mod.Source display value'''
		return module_sources[self._sampling_frequency_mod_source].short_desc

#-------------------------------------------------------------------------------
	@property
	def sampling_frequency_mod_intensity(self):
		'''Sampling Frequency Mod.Intensity'''
		return self._sampling_frequency_mod_intensity

	@sampling_frequency_mod_intensity.setter
	def sampling_frequency_mod_intensity(self,value):
		if -240 <= value <= 240:
			self._sampling_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -240 <= value <= 240' % value)

	@property
	def sampling_frequency_mod_intensity_display(self):
		'''Sampling Frequency Mod.Intensity display value'''
		dv = self._sampling_frequency_mod_intensity * 200
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def resolution(self):
		'''Resolution'''
		return self._resolution

	@resolution.setter
	def resolution(self,value):
		if 4 <= value <= 24:
			self._resolution = value
		else:
			raise ValueError('%s not 4 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level(self):
		'''Output Level'''
		return self._output_level

	@output_level.setter
	def output_level(self,value):
		if 0 <= value <= 100:
			self._output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level_mod_source(self):
		'''Output Level Mod.Source'''
		return self._output_level_mod_source

	@output_level_mod_source.setter
	def output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

	@property
	def output_level_mod_source_display(self):
		'''Output Level Mod.Source display value'''
		return module_sources[self._output_level_mod_source].short_desc

#-------------------------------------------------------------------------------
	@property
	def output_level_mod_intensity(self):
		'''Output Level Mod.Intensity'''
		return self._output_level_mod_intensity

	@output_level_mod_intensity.setter
	def output_level_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._output_level_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

	@property
	def lfo_type_display(self):
		'''LFO Type display value'''
		return {0: 'Individual', 1: 'Common 1', 2: 'Common 2'}[self._lfo_type]

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

	@property
	def common_lfo_phase_offset_display(self):
		'''Common LFO Phase Offset display value'''
		dv = self._common_lfo_phase_offset * 10
		if dv >= 1000:
			return '%0.2fk' % (dv/1000.0)
		return '%s' % dv

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

	@property
	def lfo_frequency_display(self):
		'''LFO Frequency display value'''
		return rng_01_e6_p0v02_p20v0_decode(self._lfo_frequency)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_source(self):
		'''LFO Frequency Mod.Source'''
		return self._lfo_frequency_mod_source

	@lfo_frequency_mod_source.setter
	def lfo_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._lfo_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

	@property
	def lfo_frequency_mod_source_display(self):
		'''LFO Frequency Mod.Source display value'''
		return module_sources[self._lfo_frequency_mod_source].short_desc

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_intensity(self):
		'''LFO Frequency Mod.Intensity'''
		return self._lfo_frequency_mod_intensity

	@lfo_frequency_mod_intensity.setter
	def lfo_frequency_mod_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def lfo_frequency_mod_intensity_display(self):
		'''LFO Frequency Mod.Intensity display value'''
		return rng_8d_73_m20_p20_decode(self._lfo_frequency_mod_intensity)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def lfo_bpm_midi_sync_display(self):
		'''LFO BPM/MIDI Sync display value'''
		return {0: 'Off', 1: 'On'}[self._lfo_bpm_midi_sync]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

	@property
	def depth_mod_source_display(self):
		'''Depth Mod.Source display value'''
		return module_sources[self._depth_mod_source].short_desc

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

if __name__ == '__main__':
	test_data = (
		130,  0,  0, 57,188,199,  3,128,145,  1,  0,  0,  5,  0,224,231,
		 35,  9,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0)
	fx = FX018(''.join([chr(d) for d in test_data]))
	for v in fx.toJSON():
		print v
		pass


