#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx021.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX021(object):
	category = '''OD Amp Mic'''
	description = '''Stereo Guitar Cabinet'''
	short_description = '''St. Guitar Cabinet'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX021 Stereo Guitar Cabinet object'''
		bytes = struct.unpack('<5B',raw[:5])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.cabinet_type = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1) << 3)
		self.air = (bytes[4] >> 1) & 0x7F

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX021 Stereo Guitar Cabinet object into raw data'''
		bytes = [0] * 5
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.cabinet_type << 5) & 0xE0)
		bytes[4] = ((self.cabinet_type >> 3) & 0x1) | ((self.air & 0x7F) << 1)
		return struct.pack('<5B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX021 Stereo Guitar Cabinet object'''
		json = []
		json.append(('name','FX021'))
		json.append(('description','Stereo Guitar Cabinet'))
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['input_trim',self.input_trim])
		json.append(['cabinet_type',self.cabinet_type])
		json.append(['air',self.air])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def cabinet_type(self):
		'''Cabinet Type'''
		return self._cabinet_type

	@cabinet_type.setter
	def cabinet_type(self,value):
		if 0 <= value <= 9:
			self._cabinet_type = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

#-------------------------------------------------------------------------------
	@property
	def air(self):
		'''Air'''
		return self._air

	@air.setter
	def air(self,value):
		if 0 <= value <= 100:
			self._air = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

