#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx023.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX023(object):
	category = '''OD Amp Mic'''
	description = '''Bass Amp Model'''
	short_description = '''Bass Amp Model'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX023 Bass Amp Model object'''
		bytes = struct.unpack('<11B',raw[:11])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.amp_type = (bytes[3] >> 5) & 0x7
		self.volume = bytes[4] & 0x7F
		self.volume_mod_source = ((bytes[4] & 0x80) >> 7) | ((bytes[5] & 0x1F) << 1)
		data = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.volume_mod_intensity = data
		self.bass = ((bytes[6] & 0xE0) >> 5) | ((bytes[7] & 0xF) << 3)
		self.middle = ((bytes[7] & 0xF0) >> 4) | ((bytes[8] & 0x7) << 4)
		self.mid_range = (bytes[8] >> 3) & 0x7
		self.treble = ((bytes[8] & 0xC0) >> 6) | ((bytes[9] & 0x1F) << 2)
		self.presence = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0xF) << 3)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX023 Bass Amp Model object into raw data'''
		bytes = [0] * 11
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.amp_type & 0x7) << 5
		bytes[4] = (self.volume & 0x7F) | ((self.volume_mod_source << 7) & 0x80)
		bytes[5] = ((self.volume_mod_source >> 1) & 0x1F) | ((self.volume_mod_intensity << 5) & 0xE0)
		bytes[6] = ((self.volume_mod_intensity >> 3) & 0x1F) | ((self.bass << 5) & 0xE0)
		bytes[7] = ((self.bass >> 3) & 0xF) | ((self.middle << 4) & 0xF0)
		bytes[8] = ((self.middle >> 4) & 0x7) | ((self.mid_range & 0x7) << 3) | ((self.treble << 6) & 0xC0)
		bytes[9] = ((self.treble >> 2) & 0x1F) | ((self.presence << 5) & 0xE0)
		bytes[10] = (self.presence >> 3) & 0xF
		return struct.pack('<11B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX023 Bass Amp Model object'''
		json = []
		json.append(('name','FX023'))
		json.append(('description','Bass Amp Model'))
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['amp_type',self.amp_type])
		json.append(['volume',self.volume])
		json.append(['volume_mod_source',self.volume_mod_source])
		json.append(['volume_mod_intensity',self.volume_mod_intensity])
		json.append(['bass',self.bass])
		json.append(['middle',self.middle])
		json.append(['mid_range',self.mid_range])
		json.append(['treble',self.treble])
		json.append(['presence',self.presence])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def amp_type(self):
		'''Amp Type'''
		return self._amp_type

	@amp_type.setter
	def amp_type(self,value):
		if 0 <= value <= 6:
			self._amp_type = value
		else:
			raise ValueError('%s not 0 <= value <= 6' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 100:
			self._volume = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def volume_mod_source(self):
		'''Volume Mod.Source'''
		return self._volume_mod_source

	@volume_mod_source.setter
	def volume_mod_source(self,value):
		if 0 <= value <= 37:
			self._volume_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def volume_mod_intensity(self):
		'''Volume Mod.Intensity'''
		return self._volume_mod_intensity

	@volume_mod_intensity.setter
	def volume_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._volume_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass(self):
		'''Bass'''
		return self._bass

	@bass.setter
	def bass(self,value):
		if 0 <= value <= 100:
			self._bass = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def middle(self):
		'''Middle'''
		return self._middle

	@middle.setter
	def middle(self,value):
		if 0 <= value <= 100:
			self._middle = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_range(self):
		'''Mid Range'''
		return self._mid_range

	@mid_range.setter
	def mid_range(self,value):
		if 0 <= value <= 4:
			self._mid_range = value
		else:
			raise ValueError('%s not 0 <= value <= 4' % value)

#-------------------------------------------------------------------------------
	@property
	def treble(self):
		'''Treble'''
		return self._treble

	@treble.setter
	def treble(self,value):
		if 0 <= value <= 100:
			self._treble = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def presence(self):
		'''Presence'''
		return self._presence

	@presence.setter
	def presence(self,value):
		if 0 <= value <= 100:
			self._presence = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

