#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx024.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX024(object):
	category = '''OD Amp Mic'''
	description = '''Bass Amp Model + Cabinet'''
	short_description = '''Bass Amp+Cabinet'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX024 Bass Amp Model + Cabinet object'''
		bytes = struct.unpack('<12B',raw[:12])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.bass_amp_model_amp_type = (bytes[3] >> 5) & 0x7
		self.bass_amp_model_volume = bytes[4] & 0x7F
		self.bass_amp_model_volume_mod_source = ((bytes[4] & 0x80) >> 7) | ((bytes[5] & 0x1F) << 1)
		data = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.bass_amp_model_volume_mod_intensity = data
		self.bass_amp_model_bass = ((bytes[6] & 0xE0) >> 5) | ((bytes[7] & 0xF) << 3)
		self.bass_amp_model_middle = ((bytes[7] & 0xF0) >> 4) | ((bytes[8] & 0x7) << 4)
		self.bass_amp_model_mid_range = (bytes[8] >> 3) & 0x7
		self.bass_amp_model_treble = ((bytes[8] & 0xC0) >> 6) | ((bytes[9] & 0x1F) << 2)
		self.bass_amp_model_presence = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0xF) << 3)
		self.cabinet_cabinet_on = (bytes[10] >> 4) & 0x1
		self.cabinet_cabinet_type = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x1) << 3)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX024 Bass Amp Model + Cabinet object into raw data'''
		bytes = [0] * 12
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.bass_amp_model_amp_type & 0x7) << 5
		bytes[4] = (self.bass_amp_model_volume & 0x7F) | ((self.bass_amp_model_volume_mod_source << 7) & 0x80)
		bytes[5] = ((self.bass_amp_model_volume_mod_source >> 1) & 0x1F) | ((self.bass_amp_model_volume_mod_intensity << 5) & 0xE0)
		bytes[6] = ((self.bass_amp_model_volume_mod_intensity >> 3) & 0x1F) | ((self.bass_amp_model_bass << 5) & 0xE0)
		bytes[7] = ((self.bass_amp_model_bass >> 3) & 0xF) | ((self.bass_amp_model_middle << 4) & 0xF0)
		bytes[8] = ((self.bass_amp_model_middle >> 4) & 0x7) | ((self.bass_amp_model_mid_range & 0x7) << 3) | ((self.bass_amp_model_treble << 6) & 0xC0)
		bytes[9] = ((self.bass_amp_model_treble >> 2) & 0x1F) | ((self.bass_amp_model_presence << 5) & 0xE0)
		bytes[10] = ((self.bass_amp_model_presence >> 3) & 0xF) | ((self.cabinet_cabinet_on & 0x1) << 4) | ((self.cabinet_cabinet_type << 5) & 0xE0)
		bytes[11] = (self.cabinet_cabinet_type >> 3) & 0x1
		return struct.pack('<12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX024 Bass Amp Model + Cabinet object'''
		json = []
		json.append(('name','FX024'))
		json.append(('description','Bass Amp Model + Cabinet'))
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['bass_amp_model_amp_type',self.bass_amp_model_amp_type])
		json.append(['bass_amp_model_volume',self.bass_amp_model_volume])
		json.append(['bass_amp_model_volume_mod_source',self.bass_amp_model_volume_mod_source])
		json.append(['bass_amp_model_volume_mod_intensity',self.bass_amp_model_volume_mod_intensity])
		json.append(['bass_amp_model_bass',self.bass_amp_model_bass])
		json.append(['bass_amp_model_middle',self.bass_amp_model_middle])
		json.append(['bass_amp_model_mid_range',self.bass_amp_model_mid_range])
		json.append(['bass_amp_model_treble',self.bass_amp_model_treble])
		json.append(['bass_amp_model_presence',self.bass_amp_model_presence])
		json.append(['cabinet_cabinet_on',self.cabinet_cabinet_on])
		json.append(['cabinet_cabinet_type',self.cabinet_cabinet_type])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_amp_type(self):
		'''(Bass Amp Model) Amp Type'''
		return self._bass_amp_model_amp_type

	@bass_amp_model_amp_type.setter
	def bass_amp_model_amp_type(self,value):
		if 0 <= value <= 6:
			self._bass_amp_model_amp_type = value
		else:
			raise ValueError('%s not 0 <= value <= 6' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_volume(self):
		'''(Bass Amp Model) Volume'''
		return self._bass_amp_model_volume

	@bass_amp_model_volume.setter
	def bass_amp_model_volume(self,value):
		if 0 <= value <= 100:
			self._bass_amp_model_volume = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_volume_mod_source(self):
		'''(Bass Amp Model) Volume Mod.Source'''
		return self._bass_amp_model_volume_mod_source

	@bass_amp_model_volume_mod_source.setter
	def bass_amp_model_volume_mod_source(self,value):
		if 0 <= value <= 37:
			self._bass_amp_model_volume_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_volume_mod_intensity(self):
		'''(Bass Amp Model) Volume Mod.Intensity'''
		return self._bass_amp_model_volume_mod_intensity

	@bass_amp_model_volume_mod_intensity.setter
	def bass_amp_model_volume_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._bass_amp_model_volume_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_bass(self):
		'''(Bass Amp Model) Bass'''
		return self._bass_amp_model_bass

	@bass_amp_model_bass.setter
	def bass_amp_model_bass(self,value):
		if 0 <= value <= 100:
			self._bass_amp_model_bass = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_middle(self):
		'''(Bass Amp Model) Middle'''
		return self._bass_amp_model_middle

	@bass_amp_model_middle.setter
	def bass_amp_model_middle(self,value):
		if 0 <= value <= 100:
			self._bass_amp_model_middle = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_mid_range(self):
		'''(Bass Amp Model) Mid Range'''
		return self._bass_amp_model_mid_range

	@bass_amp_model_mid_range.setter
	def bass_amp_model_mid_range(self,value):
		if 0 <= value <= 4:
			self._bass_amp_model_mid_range = value
		else:
			raise ValueError('%s not 0 <= value <= 4' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_treble(self):
		'''(Bass Amp Model) Treble'''
		return self._bass_amp_model_treble

	@bass_amp_model_treble.setter
	def bass_amp_model_treble(self,value):
		if 0 <= value <= 100:
			self._bass_amp_model_treble = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_presence(self):
		'''(Bass Amp Model) Presence'''
		return self._bass_amp_model_presence

	@bass_amp_model_presence.setter
	def bass_amp_model_presence(self,value):
		if 0 <= value <= 100:
			self._bass_amp_model_presence = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def cabinet_cabinet_on(self):
		'''(Cabinet) Cabinet On'''
		return self._cabinet_cabinet_on

	@cabinet_cabinet_on.setter
	def cabinet_cabinet_on(self,value):
		if 0 <= value <= 1:
			self._cabinet_cabinet_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def cabinet_cabinet_type(self):
		'''(Cabinet) Cabinet Type'''
		return self._cabinet_cabinet_type

	@cabinet_cabinet_type.setter
	def cabinet_cabinet_type(self,value):
		if 0 <= value <= 11:
			self._cabinet_cabinet_type = value
		else:
			raise ValueError('%s not 0 <= value <= 11' % value)

