#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx026.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX026(object):
	category = '''OD Amp Mic'''
	description = '''Tube PreAmp Modeling'''
	short_description = '''Tube PreAmp Model'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX026 Tube PreAmp Modeling object'''
		bytes = struct.unpack('<14B',raw[:14])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.tube1_low_cut = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.tube1_high_cut = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		data = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		if data & 0x40:
			data -= 0x80
		self.tube1_gain = data
		self.tube1_saturation = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x1) << 6)
		self.tube1_bias = (bytes[7] >> 1) & 0x7F
		self.phase = bytes[8] & 0x1
		self.tube2_low_cut = (bytes[8] >> 1) & 0x7F
		self.tube2_high_cut = bytes[9] & 0x7F
		data = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x3F) << 1)
		if data & 0x40:
			data -= 0x80
		self.tube2_gain = data
		self.tube2_saturation = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0x1F) << 2)
		self.tube2_bias = ((bytes[11] & 0xE0) >> 5) | ((bytes[12] & 0xF) << 3)
		data = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.output_level = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX026 Tube PreAmp Modeling object into raw data'''
		bytes = [0] * 14
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.tube1_low_cut << 5) & 0xE0
		bytes[4] = ((self.tube1_low_cut >> 3) & 0xF) | ((self.tube1_high_cut << 4) & 0xF0)
		bytes[5] = ((self.tube1_high_cut >> 4) & 0x7) | ((self.tube1_gain << 3) & 0xF8)
		bytes[6] = ((self.tube1_gain >> 5) & 0x3) | ((self.tube1_saturation << 2) & 0xFC)
		bytes[7] = ((self.tube1_saturation >> 6) & 0x1) | ((self.tube1_bias & 0x7F) << 1)
		bytes[8] = (self.phase & 0x1) | ((self.tube2_low_cut & 0x7F) << 1)
		bytes[9] = (self.tube2_high_cut & 0x7F) | ((self.tube2_gain << 7) & 0x80)
		bytes[10] = ((self.tube2_gain >> 1) & 0x3F) | ((self.tube2_saturation << 6) & 0xC0)
		bytes[11] = ((self.tube2_saturation >> 2) & 0x1F) | ((self.tube2_bias << 5) & 0xE0)
		bytes[12] = ((self.tube2_bias >> 3) & 0xF) | ((self.output_level << 4) & 0xF0)
		bytes[13] = (self.output_level >> 4) & 0xF
		return struct.pack('<14B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX026 Tube PreAmp Modeling object'''
		json = []
		json.append(('name','FX026'))
		json.append(('description','Tube PreAmp Modeling'))
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['tube1_low_cut',self.tube1_low_cut])
		json.append(['tube1_high_cut',self.tube1_high_cut])
		json.append(['tube1_gain',self.tube1_gain])
		json.append(['tube1_saturation',self.tube1_saturation])
		json.append(['tube1_bias',self.tube1_bias])
		json.append(['phase',self.phase])
		json.append(['tube2_low_cut',self.tube2_low_cut])
		json.append(['tube2_high_cut',self.tube2_high_cut])
		json.append(['tube2_gain',self.tube2_gain])
		json.append(['tube2_saturation',self.tube2_saturation])
		json.append(['tube2_bias',self.tube2_bias])
		json.append(['output_level',self.output_level])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tube1_low_cut(self):
		'''Tube1 Low Cut'''
		return self._tube1_low_cut

	@tube1_low_cut.setter
	def tube1_low_cut(self,value):
		if 0 <= value <= 104:
			self._tube1_low_cut = value
		else:
			raise ValueError('%s not 0 <= value <= 104' % value)

#-------------------------------------------------------------------------------
	@property
	def tube1_high_cut(self):
		'''Tube1 High Cut'''
		return self._tube1_high_cut

	@tube1_high_cut.setter
	def tube1_high_cut(self,value):
		if 0 <= value <= 101:
			self._tube1_high_cut = value
		else:
			raise ValueError('%s not 0 <= value <= 101' % value)

#-------------------------------------------------------------------------------
	@property
	def tube1_gain(self):
		'''Tube1 Gain'''
		return self._tube1_gain

	@tube1_gain.setter
	def tube1_gain(self,value):
		if -48 <= value <= 48:
			self._tube1_gain = value
		else:
			raise ValueError('%s not -48 <= value <= 48' % value)

#-------------------------------------------------------------------------------
	@property
	def tube1_saturation(self):
		'''Tube1 Saturation'''
		return self._tube1_saturation

	@tube1_saturation.setter
	def tube1_saturation(self,value):
		if 0 <= value <= 100:
			self._tube1_saturation = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tube1_bias(self):
		'''Tube1 Bias'''
		return self._tube1_bias

	@tube1_bias.setter
	def tube1_bias(self,value):
		if 0 <= value <= 100:
			self._tube1_bias = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phase(self):
		'''Phase'''
		return self._phase

	@phase.setter
	def phase(self,value):
		if 0 <= value <= 1:
			self._phase = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def tube2_low_cut(self):
		'''Tube2 Low Cut'''
		return self._tube2_low_cut

	@tube2_low_cut.setter
	def tube2_low_cut(self,value):
		if 0 <= value <= 104:
			self._tube2_low_cut = value
		else:
			raise ValueError('%s not 0 <= value <= 104' % value)

#-------------------------------------------------------------------------------
	@property
	def tube2_high_cut(self):
		'''Tube2 High Cut'''
		return self._tube2_high_cut

	@tube2_high_cut.setter
	def tube2_high_cut(self,value):
		if 0 <= value <= 101:
			self._tube2_high_cut = value
		else:
			raise ValueError('%s not 0 <= value <= 101' % value)

#-------------------------------------------------------------------------------
	@property
	def tube2_gain(self):
		'''Tube2 Gain'''
		return self._tube2_gain

	@tube2_gain.setter
	def tube2_gain(self,value):
		if -48 <= value <= 48:
			self._tube2_gain = value
		else:
			raise ValueError('%s not -48 <= value <= 48' % value)

#-------------------------------------------------------------------------------
	@property
	def tube2_saturation(self):
		'''Tube2 Saturation'''
		return self._tube2_saturation

	@tube2_saturation.setter
	def tube2_saturation(self,value):
		if 0 <= value <= 100:
			self._tube2_saturation = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tube2_bias(self):
		'''Tube2 Bias'''
		return self._tube2_bias

	@tube2_bias.setter
	def tube2_bias(self,value):
		if 0 <= value <= 100:
			self._tube2_bias = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level(self):
		'''Output Level'''
		return self._output_level

	@output_level.setter
	def output_level(self,value):
		if -96 <= value <= 0:
			self._output_level = value
		else:
			raise ValueError('%s not -96 <= value <= 0' % value)

