#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx028.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX028(object):
	category = '''OD Amp Mic'''
	description = '''Mic Modeling + PreAmp'''
	short_description = '''Mic Model+PreAmp'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX028 Mic Modeling + PreAmp object'''
		bytes = struct.unpack('<10B',raw[:10])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.mic_type = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1) << 3)
		self.mic_position = (bytes[4] >> 1) & 0x3
		self.low_cut = ((bytes[4] & 0xF8) >> 3) | ((bytes[5] & 0x3) << 5)
		self.high_cut = ((bytes[5] & 0xFC) >> 2) | ((bytes[6] & 0x1) << 6)
		data = (bytes[6] >> 1) & 0x7F
		if data & 0x40:
			data -= 0x80
		self.tube_pre_amp_gain = data
		self.tube_saturation = bytes[7] & 0x7F
		self.tube_bias = ((bytes[7] & 0x80) >> 7) | ((bytes[8] & 0x3F) << 1)
		data = ((bytes[8] & 0xC0) >> 6) | ((bytes[9] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.output_level = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX028 Mic Modeling + PreAmp object into raw data'''
		bytes = [0] * 10
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.mic_type << 5) & 0xE0
		bytes[4] = ((self.mic_type >> 3) & 0x1) | ((self.mic_position & 0x3) << 1) | ((self.low_cut << 3) & 0xF8)
		bytes[5] = ((self.low_cut >> 5) & 0x3) | ((self.high_cut << 2) & 0xFC)
		bytes[6] = ((self.high_cut >> 6) & 0x1) | ((self.tube_pre_amp_gain & 0x7F) << 1)
		bytes[7] = (self.tube_saturation & 0x7F) | ((self.tube_bias << 7) & 0x80)
		bytes[8] = ((self.tube_bias >> 1) & 0x3F) | ((self.output_level << 6) & 0xC0)
		bytes[9] = (self.output_level >> 2) & 0x3F
		return struct.pack('<10B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX028 Mic Modeling + PreAmp object'''
		json = []
		json.append(('name','FX028'))
		json.append(('description','Mic Modeling + PreAmp'))
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['mic_type',self.mic_type])
		json.append(['mic_position',self.mic_position])
		json.append(['low_cut',self.low_cut])
		json.append(['high_cut',self.high_cut])
		json.append(['tube_pre_amp_gain',self.tube_pre_amp_gain])
		json.append(['tube_saturation',self.tube_saturation])
		json.append(['tube_bias',self.tube_bias])
		json.append(['output_level',self.output_level])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mic_type(self):
		'''Mic Type'''
		return self._mic_type

	@mic_type.setter
	def mic_type(self,value):
		if 0 <= value <= 8:
			self._mic_type = value
		else:
			raise ValueError('%s not 0 <= value <= 8' % value)

#-------------------------------------------------------------------------------
	@property
	def mic_position(self):
		'''Mic Position'''
		return self._mic_position

	@mic_position.setter
	def mic_position(self,value):
		if 0 <= value <= 3:
			self._mic_position = value
		else:
			raise ValueError('%s not 0 <= value <= 3' % value)

#-------------------------------------------------------------------------------
	@property
	def low_cut(self):
		'''Low Cut'''
		return self._low_cut

	@low_cut.setter
	def low_cut(self,value):
		if 0 <= value <= 104:
			self._low_cut = value
		else:
			raise ValueError('%s not 0 <= value <= 104' % value)

#-------------------------------------------------------------------------------
	@property
	def high_cut(self):
		'''High Cut'''
		return self._high_cut

	@high_cut.setter
	def high_cut(self,value):
		if 0 <= value <= 101:
			self._high_cut = value
		else:
			raise ValueError('%s not 0 <= value <= 101' % value)

#-------------------------------------------------------------------------------
	@property
	def tube_pre_amp_gain(self):
		'''Tube Pre Amp Gain'''
		return self._tube_pre_amp_gain

	@tube_pre_amp_gain.setter
	def tube_pre_amp_gain(self,value):
		if -48 <= value <= 48:
			self._tube_pre_amp_gain = value
		else:
			raise ValueError('%s not -48 <= value <= 48' % value)

#-------------------------------------------------------------------------------
	@property
	def tube_saturation(self):
		'''Tube Saturation'''
		return self._tube_saturation

	@tube_saturation.setter
	def tube_saturation(self,value):
		if 0 <= value <= 100:
			self._tube_saturation = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tube_bias(self):
		'''Tube Bias'''
		return self._tube_bias

	@tube_bias.setter
	def tube_bias(self,value):
		if 0 <= value <= 100:
			self._tube_bias = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def output_level(self):
		'''Output Level'''
		return self._output_level

	@output_level.setter
	def output_level(self,value):
		if -96 <= value <= 0:
			self._output_level = value
		else:
			raise ValueError('%s not -96 <= value <= 0' % value)

