#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx030.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def wet_dry_decode(v):
	return 'Dry' if v==0 else 'Wet' if v==100 else '%d: %d' % (v,100-v)

#===============================================================================
class FX030(object):
	category = '''Cho/Fln/Phaser'''
	description = '''Vintage Chorus'''
	short_description = '''Vintage Chorus'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX030 Vintage Chorus object'''
		bytes = struct.unpack('<10B',raw[:10])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.speed = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		self.depth = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.manual = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		self.output_mode = (bytes[6] >> 3) & 0x1
		self.speed_mod_source = ((bytes[6] & 0xF0) >> 4) | ((bytes[7] & 0x3) << 4)
		data = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.speed_mod_intensity = data
		self.depth_mod_source = (bytes[8] >> 2) & 0x3F
		data = bytes[9] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX030 Vintage Chorus object into raw data'''
		bytes = [0] * 10
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.speed << 5) & 0xE0
		bytes[4] = ((self.speed >> 3) & 0x1F) | ((self.depth << 5) & 0xE0)
		bytes[5] = ((self.depth >> 3) & 0xF) | ((self.manual << 4) & 0xF0)
		bytes[6] = ((self.manual >> 4) & 0x7) | ((self.output_mode & 0x1) << 3) | ((self.speed_mod_source << 4) & 0xF0)
		bytes[7] = ((self.speed_mod_source >> 4) & 0x3) | ((self.speed_mod_intensity << 2) & 0xFC)
		bytes[8] = ((self.speed_mod_intensity >> 6) & 0x3) | ((self.depth_mod_source & 0x3F) << 2)
		bytes[9] = self.depth_mod_intensity & 0xFF
		return struct.pack('<10B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX030 Vintage Chorus object'''
		json = []
		json.append(('name','FX030'))
		json.append(('description','Vintage Chorus'))
		json.append(['speed',self.speed])
		json.append(['speed_mod_source',self.speed_mod_source])
		json.append(['speed_mod_intensity',self.speed_mod_intensity])
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['manual',self.manual])
		json.append(['output_mode',self.output_mode])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_display',self.wet_dry_display])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

	@property
	def wet_dry_display(self):
		'''Wet/Dry display value'''
		return wet_dry_decode(self._wet_dry)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def speed(self):
		'''Speed'''
		return self._speed

	@speed.setter
	def speed(self,value):
		if 5 <= value <= 210:
			self._speed = value
		else:
			raise ValueError('%s not 5 <= value <= 210' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def manual(self):
		'''Manual'''
		return self._manual

	@manual.setter
	def manual(self,value):
		if 0 <= value <= 99:
			self._manual = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def output_mode(self):
		'''Output Mode'''
		return self._output_mode

	@output_mode.setter
	def output_mode(self,value):
		if 0 <= value <= 1:
			self._output_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_source(self):
		'''Speed Mod.Source'''
		return self._speed_mod_source

	@speed_mod_source.setter
	def speed_mod_source(self,value):
		if 0 <= value <= 37:
			self._speed_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_intensity(self):
		'''Speed Mod.Intensity'''
		return self._speed_mod_intensity

	@speed_mod_intensity.setter
	def speed_mod_intensity(self,value):
		if -105 <= value <= 105:
			self._speed_mod_intensity = value
		else:
			raise ValueError('%s not -105 <= value <= 105' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

