#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx031.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX031(object):
	category = '''Cho/Fln/Phaser'''
	description = '''Black Chorus'''
	short_description = '''Black Chorus'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX031 Black Chorus object'''
		bytes = struct.unpack('<10B',raw[:10])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.speed = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		self.intensity = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.mode = (bytes[5] >> 4) & 0x3
		self.width = ((bytes[5] & 0xC0) >> 6) | ((bytes[6] & 0x1F) << 2)
		self.input_gain = ((bytes[6] & 0xE0) >> 5) | ((bytes[7] & 0xF) << 3)
		self.output_mode = (bytes[7] >> 4) & 0x1
		self.speed_mod_source = ((bytes[7] & 0xE0) >> 5) | ((bytes[8] & 0x7) << 3)
		data = ((bytes[8] & 0xF8) >> 3) | ((bytes[9] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.speed_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX031 Black Chorus object into raw data'''
		bytes = [0] * 10
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.speed << 5) & 0xE0
		bytes[4] = ((self.speed >> 3) & 0x1F) | ((self.intensity << 5) & 0xE0)
		bytes[5] = ((self.intensity >> 3) & 0xF) | ((self.mode & 0x3) << 4) | ((self.width << 6) & 0xC0)
		bytes[6] = ((self.width >> 2) & 0x1F) | ((self.input_gain << 5) & 0xE0)
		bytes[7] = ((self.input_gain >> 3) & 0xF) | ((self.output_mode & 0x1) << 4) | ((self.speed_mod_source << 5) & 0xE0)
		bytes[8] = ((self.speed_mod_source >> 3) & 0x7) | ((self.speed_mod_intensity << 3) & 0xF8)
		bytes[9] = (self.speed_mod_intensity >> 5) & 0x7
		return struct.pack('<10B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX031 Black Chorus object'''
		json = []
		json.append(('name','FX031'))
		json.append(('description','Black Chorus'))
		json.append(['speed',self.speed])
		json.append(['speed_mod_source',self.speed_mod_source])
		json.append(['speed_mod_intensity',self.speed_mod_intensity])
		json.append(['intensity',self.intensity])
		json.append(['mode',self.mode])
		json.append(['width',self.width])
		json.append(['input_gain',self.input_gain])
		json.append(['output_mode',self.output_mode])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def speed(self):
		'''Speed'''
		return self._speed

	@speed.setter
	def speed(self,value):
		if 5 <= value <= 210:
			self._speed = value
		else:
			raise ValueError('%s not 5 <= value <= 210' % value)

#-------------------------------------------------------------------------------
	@property
	def intensity(self):
		'''Intensity'''
		return self._intensity

	@intensity.setter
	def intensity(self,value):
		if 0 <= value <= 99:
			self._intensity = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def mode(self):
		'''Mode'''
		return self._mode

	@mode.setter
	def mode(self,value):
		if 0 <= value <= 2:
			self._mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def width(self):
		'''Width'''
		return self._width

	@width.setter
	def width(self,value):
		if 0 <= value <= 100:
			self._width = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_gain(self):
		'''Input Gain'''
		return self._input_gain

	@input_gain.setter
	def input_gain(self,value):
		if 0 <= value <= 99:
			self._input_gain = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def output_mode(self):
		'''Output Mode'''
		return self._output_mode

	@output_mode.setter
	def output_mode(self,value):
		if 0 <= value <= 1:
			self._output_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_source(self):
		'''Speed Mod.Source'''
		return self._speed_mod_source

	@speed_mod_source.setter
	def speed_mod_source(self,value):
		if 0 <= value <= 37:
			self._speed_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_intensity(self):
		'''Speed Mod.Intensity'''
		return self._speed_mod_intensity

	@speed_mod_intensity.setter
	def speed_mod_intensity(self,value):
		if -105 <= value <= 105:
			self._speed_mod_intensity = value
		else:
			raise ValueError('%s not -105 <= value <= 105' % value)

