#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx033.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX033(object):
	category = '''Cho/Fln/Phaser'''
	description = '''Stereo Harmonic Chorus'''
	short_description = '''St.HarmonicChorus'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX033 Stereo Harmonic Chorus object'''
		bytes = struct.unpack('<20B',raw[:20])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.lfo_type = (bytes[3] >> 5) & 0x3
		self.lfo_waveform = (bytes[3] >> 7) & 0x1
		data = bytes[4] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		data = ((bytes[4] & 0xC0) >> 6) | ((bytes[5] & 0xF) << 2)
		if data & 0x20:
			data -= 0x40
		self.right_lfo_phase = data
		self.lfo_frequency = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0xF) << 4)
		self.lfo_frequency_mod_source = ((bytes[6] & 0xF0) >> 4) | ((bytes[7] & 0x3) << 4)
		data = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.lfo_frequency_mod_intensity = data
		self.lfo_bpm_midi_sync = (bytes[8] >> 2) & 0x1
		self.lfo_bpm = ((bytes[8] & 0xF8) >> 3) | (bytes[9] << 5) | ((bytes[10] & 0x3) << 13)
		self.lfo_bpm_base_note = (bytes[10] >> 2) & 0xF
		self.lfo_bpm_times = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0xF) << 2)
		self.pre_delay = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0xF) << 4)
		self.depth = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x7) << 4)
		self.depth_mod_source = ((bytes[13] & 0xF8) >> 3) | ((bytes[14] & 0x1) << 5)
		data = ((bytes[14] & 0xFE) >> 1) | ((bytes[15] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data
		self.high_low_split_point = (bytes[15] >> 1) & 0x7F
		data = bytes[16] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.high_damp = bytes[17] & 0x7F
		self.low_level = ((bytes[17] & 0x80) >> 7) | ((bytes[18] & 0x3F) << 1)
		self.high_level = ((bytes[18] & 0xC0) >> 6) | ((bytes[19] & 0x1F) << 2)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX033 Stereo Harmonic Chorus object into raw data'''
		bytes = [0] * 20
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.lfo_type & 0x3) << 5) | ((self.lfo_waveform & 0x1) << 7)
		bytes[4] = (self.common_lfo_phase_offset & 0x3F) | ((self.right_lfo_phase << 6) & 0xC0)
		bytes[5] = ((self.right_lfo_phase >> 2) & 0xF) | ((self.lfo_frequency << 4) & 0xF0)
		bytes[6] = ((self.lfo_frequency >> 4) & 0xF) | ((self.lfo_frequency_mod_source << 4) & 0xF0)
		bytes[7] = ((self.lfo_frequency_mod_source >> 4) & 0x3) | ((self.lfo_frequency_mod_intensity << 2) & 0xFC)
		bytes[8] = ((self.lfo_frequency_mod_intensity >> 6) & 0x3) | ((self.lfo_bpm_midi_sync & 0x1) << 2) | ((self.lfo_bpm << 3) & 0xF8)
		bytes[9] = (self.lfo_bpm >> 5) & 0xFF
		bytes[10] = ((self.lfo_bpm >> 3) & 0xD) | ((self.lfo_bpm_base_note & 0xF) << 2) | ((self.lfo_bpm_times << 6) & 0xC0)
		bytes[11] = ((self.lfo_bpm_times >> 2) & 0xF) | ((self.pre_delay << 4) & 0xF0)
		bytes[12] = ((self.pre_delay >> 4) & 0xF) | ((self.depth << 4) & 0xF0)
		bytes[13] = ((self.depth >> 4) & 0x7) | ((self.depth_mod_source << 3) & 0xF8)
		bytes[14] = ((self.depth_mod_source >> 5) & 0x1) | ((self.depth_mod_intensity << 1) & 0xFE)
		bytes[15] = ((self.depth_mod_intensity >> 7) & 0x1) | ((self.high_low_split_point & 0x7F) << 1)
		bytes[16] = self.feedback & 0xFF
		bytes[17] = (self.high_damp & 0x7F) | ((self.low_level << 7) & 0x80)
		bytes[18] = ((self.low_level >> 1) & 0x3F) | ((self.high_level << 6) & 0xC0)
		bytes[19] = (self.high_level >> 2) & 0x1F
		return struct.pack('<20B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX033 Stereo Harmonic Chorus object'''
		json = []
		json.append(('name','FX033'))
		json.append(('description','Stereo Harmonic Chorus'))
		json.append(['lfo_waveform',self.lfo_waveform])
		json.append(['right_lfo_phase',self.right_lfo_phase])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_frequency_mod_source',self.lfo_frequency_mod_source])
		json.append(['lfo_frequency_mod_intensity',self.lfo_frequency_mod_intensity])
		json.append(['lfo_frequency_mod_intensity_display',self.lfo_frequency_mod_intensity_display])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		json.append(['pre_delay',self.pre_delay])
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['high_low_split_point',self.high_low_split_point])
		json.append(['feedback',self.feedback])
		json.append(['high_damp',self.high_damp])
		json.append(['low_level',self.low_level])
		json.append(['high_level',self.high_level])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_waveform(self):
		'''LFO Waveform'''
		return self._lfo_waveform

	@lfo_waveform.setter
	def lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def right_lfo_phase(self):
		'''Right LFO Phase'''
		return self._right_lfo_phase

	@right_lfo_phase.setter
	def right_lfo_phase(self,value):
		if -18 <= value <= 18:
			self._right_lfo_phase = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_source(self):
		'''LFO Frequency Mod.Source'''
		return self._lfo_frequency_mod_source

	@lfo_frequency_mod_source.setter
	def lfo_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._lfo_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_intensity(self):
		'''LFO Frequency Mod.Intensity'''
		return self._lfo_frequency_mod_intensity

	@lfo_frequency_mod_intensity.setter
	def lfo_frequency_mod_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def lfo_frequency_mod_intensity_display(self):
		'''LFO Frequency Mod.Intensity display value'''
		return rng_8d_73_m20_p20_decode(self._lfo_frequency_mod_intensity)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_delay(self):
		'''Pre Delay'''
		return self._pre_delay

	@pre_delay.setter
	def pre_delay(self,value):
		if 0 <= value <= 135:
			self._pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 135' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_low_split_point(self):
		'''High/Low Split Point'''
		return self._high_low_split_point

	@high_low_split_point.setter
	def high_low_split_point(self,value):
		if 1 <= value <= 100:
			self._high_low_split_point = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_level(self):
		'''Low Level'''
		return self._low_level

	@low_level.setter
	def low_level(self,value):
		if 0 <= value <= 100:
			self._low_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_level(self):
		'''High Level'''
		return self._high_level

	@high_level.setter
	def high_level(self,value):
		if 0 <= value <= 100:
			self._high_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

