#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx036.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX036(object):
	category = '''Cho/Fln/Phaser'''
	description = '''Ensemble'''
	short_description = '''Ensemble'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX036 Ensemble object'''
		bytes = struct.unpack('<10B',raw[:10])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.speed = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.speed_mod_source = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x3) << 4)
		data = ((bytes[5] & 0xFC) >> 2) | ((bytes[6] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.speed_mod_intensity = data
		self.depth = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x1) << 6)
		self.depth_mod_source = (bytes[7] >> 1) & 0x3F
		data = ((bytes[7] & 0x80) >> 7) | ((bytes[8] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data
		self.shimmer = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x3F) << 1)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX036 Ensemble object into raw data'''
		bytes = [0] * 10
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.speed << 5) & 0xE0
		bytes[4] = ((self.speed >> 3) & 0xF) | ((self.speed_mod_source << 4) & 0xF0)
		bytes[5] = ((self.speed_mod_source >> 4) & 0x3) | ((self.speed_mod_intensity << 2) & 0xFC)
		bytes[6] = ((self.speed_mod_intensity >> 6) & 0x3) | ((self.depth << 2) & 0xFC)
		bytes[7] = ((self.depth >> 6) & 0x1) | ((self.depth_mod_source & 0x3F) << 1) | ((self.depth_mod_intensity << 7) & 0x80)
		bytes[8] = ((self.depth_mod_intensity >> 1) & 0x7F) | ((self.shimmer << 7) & 0x80)
		bytes[9] = (self.shimmer >> 1) & 0x3F
		return struct.pack('<10B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX036 Ensemble object'''
		json = []
		json.append(('name','FX036'))
		json.append(('description','Ensemble'))
		json.append(['speed',self.speed])
		json.append(['speed_mod_source',self.speed_mod_source])
		json.append(['speed_mod_intensity',self.speed_mod_intensity])
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['shimmer',self.shimmer])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def speed(self):
		'''Speed'''
		return self._speed

	@speed.setter
	def speed(self,value):
		if 1 <= value <= 100:
			self._speed = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_source(self):
		'''Speed Mod.Source'''
		return self._speed_mod_source

	@speed_mod_source.setter
	def speed_mod_source(self,value):
		if 0 <= value <= 37:
			self._speed_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_intensity(self):
		'''Speed Mod.Intensity'''
		return self._speed_mod_intensity

	@speed_mod_intensity.setter
	def speed_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._speed_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def shimmer(self):
		'''Shimmer'''
		return self._shimmer

	@shimmer.setter
	def shimmer(self,value):
		if 0 <= value <= 100:
			self._shimmer = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

