#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx041.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX041(object):
	category = '''Cho/Fln/Phaser'''
	description = '''Stereo Envelope Flanger'''
	short_description = '''St. Env. Flanger'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX041 Stereo Envelope Flanger object'''
		bytes = struct.unpack('<13B',raw[:13])
		data = ((bytes[0] & 0xFE) >> 1) | ((bytes[1] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.wet_dry = data
		self.wet_dry_mod_source = (bytes[1] >> 1) & 0x3F
		data = ((bytes[1] & 0x80) >> 7) | ((bytes[2] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.l_delay_bottom = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x3F) << 2)
		self.l_delay_top = ((bytes[4] & 0xC0) >> 6) | ((bytes[5] & 0x3F) << 2)
		self.r_delay_bottom = ((bytes[5] & 0xC0) >> 6) | ((bytes[6] & 0x3F) << 2)
		self.r_delay_top = ((bytes[6] & 0xC0) >> 6) | ((bytes[7] & 0x3F) << 2)
		self.sweep_mode = (bytes[7] >> 6) & 0x1
		self.sweep_mode_mod_source = ((bytes[7] & 0x80) >> 7) | ((bytes[8] & 0x1F) << 1)
		self.eg_attack = ((bytes[8] & 0xE0) >> 5) | ((bytes[9] & 0xF) << 3)
		self.eg_decay = ((bytes[9] & 0xF0) >> 4) | ((bytes[10] & 0x7) << 4)
		data = ((bytes[10] & 0xF8) >> 3) | ((bytes[11] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.high_damp = ((bytes[11] & 0xF8) >> 3) | ((bytes[12] & 0x3) << 5)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX041 Stereo Envelope Flanger object into raw data'''
		bytes = [0] * 13
		bytes[0] = (self.wet_dry << 1) & 0xFE
		bytes[1] = ((self.wet_dry >> 7) & 0x1) | ((self.wet_dry_mod_source & 0x3F) << 1) | ((self.wet_dry_mod_intensity << 7) & 0x80)
		bytes[2] = (self.wet_dry_mod_intensity >> 1) & 0x7F
		bytes[3] = (self.l_delay_bottom << 6) & 0xC0
		bytes[4] = ((self.l_delay_bottom >> 2) & 0x3F) | ((self.l_delay_top << 6) & 0xC0)
		bytes[5] = ((self.l_delay_top >> 2) & 0x3F) | ((self.r_delay_bottom << 6) & 0xC0)
		bytes[6] = ((self.r_delay_bottom >> 2) & 0x3F) | ((self.r_delay_top << 6) & 0xC0)
		bytes[7] = ((self.r_delay_top >> 2) & 0x3F) | ((self.sweep_mode & 0x1) << 6) | ((self.sweep_mode_mod_source << 7) & 0x80)
		bytes[8] = ((self.sweep_mode_mod_source >> 1) & 0x1F) | ((self.eg_attack << 5) & 0xE0)
		bytes[9] = ((self.eg_attack >> 3) & 0xF) | ((self.eg_decay << 4) & 0xF0)
		bytes[10] = ((self.eg_decay >> 4) & 0x7) | ((self.feedback << 3) & 0xF8)
		bytes[11] = ((self.feedback >> 5) & 0x7) | ((self.high_damp << 3) & 0xF8)
		bytes[12] = (self.high_damp >> 5) & 0x3
		return struct.pack('<13B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX041 Stereo Envelope Flanger object'''
		json = []
		json.append(('name','FX041'))
		json.append(('description','Stereo Envelope Flanger'))
		json.append(['l_delay_bottom',self.l_delay_bottom])
		json.append(['l_delay_top',self.l_delay_top])
		json.append(['r_delay_bottom',self.r_delay_bottom])
		json.append(['r_delay_top',self.r_delay_top])
		json.append(['sweep_mode',self.sweep_mode])
		json.append(['sweep_mode_mod_source',self.sweep_mode_mod_source])
		json.append(['eg_attack',self.eg_attack])
		json.append(['eg_decay',self.eg_decay])
		json.append(['feedback',self.feedback])
		json.append(['high_damp',self.high_damp])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if -100 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def l_delay_bottom(self):
		'''L Delay Bottom'''
		return self._l_delay_bottom

	@l_delay_bottom.setter
	def l_delay_bottom(self,value):
		if 0 <= value <= 140:
			self._l_delay_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 140' % value)

#-------------------------------------------------------------------------------
	@property
	def l_delay_top(self):
		'''L Delay Top'''
		return self._l_delay_top

	@l_delay_top.setter
	def l_delay_top(self,value):
		if 0 <= value <= 140:
			self._l_delay_top = value
		else:
			raise ValueError('%s not 0 <= value <= 140' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_bottom(self):
		'''R Delay Bottom'''
		return self._r_delay_bottom

	@r_delay_bottom.setter
	def r_delay_bottom(self,value):
		if 0 <= value <= 140:
			self._r_delay_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 140' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_top(self):
		'''R Delay Top'''
		return self._r_delay_top

	@r_delay_top.setter
	def r_delay_top(self,value):
		if 0 <= value <= 140:
			self._r_delay_top = value
		else:
			raise ValueError('%s not 0 <= value <= 140' % value)

#-------------------------------------------------------------------------------
	@property
	def sweep_mode(self):
		'''Sweep Mode'''
		return self._sweep_mode

	@sweep_mode.setter
	def sweep_mode(self,value):
		if 0 <= value <= 1:
			self._sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def sweep_mode_mod_source(self):
		'''Sweep Mode Mod.Source'''
		return self._sweep_mode_mod_source

	@sweep_mode_mod_source.setter
	def sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def eg_attack(self):
		'''EG Attack'''
		return self._eg_attack

	@eg_attack.setter
	def eg_attack(self,value):
		if 0 <= value <= 99:
			self._eg_attack = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def eg_decay(self):
		'''EG Decay'''
		return self._eg_decay

	@eg_decay.setter
	def eg_decay(self,value):
		if 0 <= value <= 99:
			self._eg_decay = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

