#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx045.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX045(object):
	category = '''Cho/Fln/Phaser'''
	description = '''Black Phaser'''
	short_description = '''Black Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX045 Black Phaser object'''
		bytes = struct.unpack('<11B',raw[:11])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.speed = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		self.depth = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.resonance = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		self.manual = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		self.speed_mod_source = (bytes[7] >> 2) & 0x3F
		data = bytes[8] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.speed_mod_intensity = data
		self.depth_mod_source = bytes[9] & 0x3F
		data = ((bytes[9] & 0xC0) >> 6) | ((bytes[10] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX045 Black Phaser object into raw data'''
		bytes = [0] * 11
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.speed << 5) & 0xE0
		bytes[4] = ((self.speed >> 3) & 0x1F) | ((self.depth << 5) & 0xE0)
		bytes[5] = ((self.depth >> 3) & 0xF) | ((self.resonance << 4) & 0xF0)
		bytes[6] = ((self.resonance >> 4) & 0x7) | ((self.manual << 3) & 0xF8)
		bytes[7] = ((self.manual >> 5) & 0x3) | ((self.speed_mod_source & 0x3F) << 2)
		bytes[8] = self.speed_mod_intensity & 0xFF
		bytes[9] = (self.depth_mod_source & 0x3F) | ((self.depth_mod_intensity << 6) & 0xC0)
		bytes[10] = (self.depth_mod_intensity >> 2) & 0x3F
		return struct.pack('<11B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX045 Black Phaser object'''
		json = []
		json.append(('name','FX045'))
		json.append(('description','Black Phaser'))
		json.append(['speed',self.speed])
		json.append(['speed_mod_source',self.speed_mod_source])
		json.append(['speed_mod_intensity',self.speed_mod_intensity])
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['resonance',self.resonance])
		json.append(['manual',self.manual])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def speed(self):
		'''Speed'''
		return self._speed

	@speed.setter
	def speed(self,value):
		if 5 <= value <= 210:
			self._speed = value
		else:
			raise ValueError('%s not 5 <= value <= 210' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def resonance(self):
		'''Resonance'''
		return self._resonance

	@resonance.setter
	def resonance(self,value):
		if 0 <= value <= 100:
			self._resonance = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def manual(self):
		'''Manual'''
		return self._manual

	@manual.setter
	def manual(self,value):
		if 0 <= value <= 99:
			self._manual = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_source(self):
		'''Speed Mod.Source'''
		return self._speed_mod_source

	@speed_mod_source.setter
	def speed_mod_source(self,value):
		if 0 <= value <= 37:
			self._speed_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def speed_mod_intensity(self):
		'''Speed Mod.Intensity'''
		return self._speed_mod_intensity

	@speed_mod_intensity.setter
	def speed_mod_intensity(self,value):
		if -105 <= value <= 105:
			self._speed_mod_intensity = value
		else:
			raise ValueError('%s not -105 <= value <= 105' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

