#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx048.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX048(object):
	category = '''Cho/Fln/Phaser'''
	description = '''Stereo Envelope Phaser'''
	short_description = '''St. Env. Phaser'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX048 Stereo Envelope Phaser object'''
		bytes = struct.unpack('<12B',raw[:12])
		data = ((bytes[0] & 0xFE) >> 1) | ((bytes[1] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.wet_dry = data
		self.wet_dry_mod_source = (bytes[1] >> 1) & 0x3F
		data = ((bytes[1] & 0x80) >> 7) | ((bytes[2] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.l_manual_bottom = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x1F) << 2)
		self.l_manual_top = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0xF) << 3)
		self.r_manual_bottom = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x7) << 4)
		self.r_manual_top = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		self.sweep_mode = (bytes[7] >> 2) & 0x1
		self.sweep_mode_mod_source = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x1) << 5)
		self.eg_attack = (bytes[8] >> 1) & 0x7F
		self.eg_decay = bytes[9] & 0x7F
		data = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.resonance = data
		self.high_damp = ((bytes[10] & 0x80) >> 7) | ((bytes[11] & 0x3F) << 1)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX048 Stereo Envelope Phaser object into raw data'''
		bytes = [0] * 12
		bytes[0] = (self.wet_dry << 1) & 0xFE
		bytes[1] = ((self.wet_dry >> 7) & 0x1) | ((self.wet_dry_mod_source & 0x3F) << 1) | ((self.wet_dry_mod_intensity << 7) & 0x80)
		bytes[2] = (self.wet_dry_mod_intensity >> 1) & 0x7F
		bytes[3] = (self.l_manual_bottom << 6) & 0xC0
		bytes[4] = ((self.l_manual_bottom >> 2) & 0x1F) | ((self.l_manual_top << 5) & 0xE0)
		bytes[5] = ((self.l_manual_top >> 3) & 0xF) | ((self.r_manual_bottom << 4) & 0xF0)
		bytes[6] = ((self.r_manual_bottom >> 4) & 0x7) | ((self.r_manual_top << 3) & 0xF8)
		bytes[7] = ((self.r_manual_top >> 5) & 0x3) | ((self.sweep_mode & 0x1) << 2) | ((self.sweep_mode_mod_source << 3) & 0xF8)
		bytes[8] = ((self.sweep_mode_mod_source >> 5) & 0x1) | ((self.eg_attack & 0x7F) << 1)
		bytes[9] = (self.eg_decay & 0x7F) | ((self.resonance << 7) & 0x80)
		bytes[10] = ((self.resonance >> 1) & 0x7F) | ((self.high_damp << 7) & 0x80)
		bytes[11] = (self.high_damp >> 1) & 0x3F
		return struct.pack('<12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX048 Stereo Envelope Phaser object'''
		json = []
		json.append(('name','FX048'))
		json.append(('description','Stereo Envelope Phaser'))
		json.append(['l_manual_bottom',self.l_manual_bottom])
		json.append(['l_manual_top',self.l_manual_top])
		json.append(['r_manual_bottom',self.r_manual_bottom])
		json.append(['r_manual_top',self.r_manual_top])
		json.append(['sweep_mode',self.sweep_mode])
		json.append(['sweep_mode_mod_source',self.sweep_mode_mod_source])
		json.append(['eg_attack',self.eg_attack])
		json.append(['eg_decay',self.eg_decay])
		json.append(['resonance',self.resonance])
		json.append(['high_damp',self.high_damp])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if -100 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def l_manual_bottom(self):
		'''L Manual Bottom'''
		return self._l_manual_bottom

	@l_manual_bottom.setter
	def l_manual_bottom(self,value):
		if 0 <= value <= 100:
			self._l_manual_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def l_manual_top(self):
		'''L Manual Top'''
		return self._l_manual_top

	@l_manual_top.setter
	def l_manual_top(self,value):
		if 0 <= value <= 100:
			self._l_manual_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def r_manual_bottom(self):
		'''R Manual Bottom'''
		return self._r_manual_bottom

	@r_manual_bottom.setter
	def r_manual_bottom(self,value):
		if 0 <= value <= 100:
			self._r_manual_bottom = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def r_manual_top(self):
		'''R Manual Top'''
		return self._r_manual_top

	@r_manual_top.setter
	def r_manual_top(self,value):
		if 0 <= value <= 100:
			self._r_manual_top = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def sweep_mode(self):
		'''Sweep Mode'''
		return self._sweep_mode

	@sweep_mode.setter
	def sweep_mode(self,value):
		if 0 <= value <= 1:
			self._sweep_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def sweep_mode_mod_source(self):
		'''Sweep Mode Mod.Source'''
		return self._sweep_mode_mod_source

	@sweep_mode_mod_source.setter
	def sweep_mode_mod_source(self,value):
		if 0 <= value <= 37:
			self._sweep_mode_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def eg_attack(self):
		'''EG Attack'''
		return self._eg_attack

	@eg_attack.setter
	def eg_attack(self,value):
		if 0 <= value <= 99:
			self._eg_attack = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def eg_decay(self):
		'''EG Decay'''
		return self._eg_decay

	@eg_decay.setter
	def eg_decay(self,value):
		if 0 <= value <= 99:
			self._eg_decay = value
		else:
			raise ValueError('%s not 0 <= value <= 99' % value)

#-------------------------------------------------------------------------------
	@property
	def resonance(self):
		'''Resonance'''
		return self._resonance

	@resonance.setter
	def resonance(self,value):
		if -100 <= value <= 100:
			self._resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

