#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx051.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX051(object):
	category = '''Mod./P.Shift'''
	description = '''2Voice Resonator'''
	short_description = '''2Voice Resonator'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX051 2Voice Resonator object'''
		bytes = struct.unpack('<21B',raw[:21])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.control_mode = (bytes[3] >> 5) & 0x3
		self.lfo_mod_invert = (bytes[3] >> 7) & 0x1
		self.control_mod_source = bytes[4] & 0x3F
		self.lfo_type = (bytes[4] >> 6) & 0x3
		data = bytes[5] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_frequency = ((bytes[5] & 0xC0) >> 6) | ((bytes[6] & 0x3F) << 2)
		self.lfo_bpm_midi_sync = (bytes[6] >> 6) & 0x1
		self.lfo_bpm = ((bytes[6] & 0x80) >> 7) | (bytes[7] << 1) | ((bytes[8] & 0x3F) << 9)
		self.lfo_bpm_base_note = ((bytes[8] & 0xC0) >> 6) | ((bytes[9] & 0x3) << 2)
		self.lfo_bpm_times = (bytes[9] >> 2) & 0x3F
		data = bytes[10] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.modulation_depth = data
		self.voice1_pitch = bytes[11] & 0x7F
		self.voice1_fine = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x7) << 1)
		data = ((bytes[12] & 0xF8) >> 3) | ((bytes[13] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.voice1_resonance = data
		self.voice1_high_damp = ((bytes[13] & 0xF8) >> 3) | ((bytes[14] & 0x3) << 5)
		self.voice1_level = ((bytes[14] & 0xFC) >> 2) | ((bytes[15] & 0x1) << 6)
		data = (bytes[15] >> 1) & 0xF
		if data & 0x8:
			data -= 0x10
		self.voice1_pan = data
		self.voice2_pitch = ((bytes[15] & 0xE0) >> 5) | ((bytes[16] & 0xF) << 3)
		self.voice2_fine = (bytes[16] >> 4) & 0xF
		data = bytes[17] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.voice2_resonance = data
		self.voice2_high_damp = bytes[18] & 0x7F
		self.voice2_level = ((bytes[18] & 0x80) >> 7) | ((bytes[19] & 0x3F) << 1)
		data = ((bytes[19] & 0xC0) >> 6) | ((bytes[20] & 0x3) << 2)
		if data & 0x8:
			data -= 0x10
		self.voice2_pan = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX051 2Voice Resonator object into raw data'''
		bytes = [0] * 21
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.control_mode & 0x3) << 5) | ((self.lfo_mod_invert & 0x1) << 7)
		bytes[4] = (self.control_mod_source & 0x3F) | ((self.lfo_type & 0x3) << 6)
		bytes[5] = (self.common_lfo_phase_offset & 0x3F) | ((self.lfo_frequency << 6) & 0xC0)
		bytes[6] = ((self.lfo_frequency >> 2) & 0x3F) | ((self.lfo_bpm_midi_sync & 0x1) << 6) | ((self.lfo_bpm << 7) & 0x80)
		bytes[7] = (self.lfo_bpm >> 1) & 0xFF
		bytes[8] = ((self.lfo_bpm >> 63) & 0x9) | ((self.lfo_bpm_base_note << 6) & 0xC0)
		bytes[9] = ((self.lfo_bpm_base_note >> 2) & 0x3) | ((self.lfo_bpm_times & 0x3F) << 2)
		bytes[10] = self.modulation_depth & 0xFF
		bytes[11] = (self.voice1_pitch & 0x7F) | ((self.voice1_fine << 7) & 0x80)
		bytes[12] = ((self.voice1_fine >> 1) & 0x7) | ((self.voice1_resonance << 3) & 0xF8)
		bytes[13] = ((self.voice1_resonance >> 5) & 0x7) | ((self.voice1_high_damp << 3) & 0xF8)
		bytes[14] = ((self.voice1_high_damp >> 5) & 0x3) | ((self.voice1_level << 2) & 0xFC)
		bytes[15] = ((self.voice1_level >> 6) & 0x1) | ((self.voice1_pan & 0xF) << 1) | ((self.voice2_pitch << 5) & 0xE0)
		bytes[16] = ((self.voice2_pitch >> 3) & 0xF) | ((self.voice2_fine & 0xF) << 4)
		bytes[17] = self.voice2_resonance & 0xFF
		bytes[18] = (self.voice2_high_damp & 0x7F) | ((self.voice2_level << 7) & 0x80)
		bytes[19] = ((self.voice2_level >> 1) & 0x3F) | ((self.voice2_pan << 6) & 0xC0)
		bytes[20] = (self.voice2_pan >> 2) & 0x3
		return struct.pack('<21B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX051 2Voice Resonator object'''
		json = []
		json.append(('name','FX051'))
		json.append(('description','2Voice Resonator'))
		json.append(['control_mode',self.control_mode])
		json.append(['lfo_mod_invert',self.lfo_mod_invert])
		json.append(['control_mod_source',self.control_mod_source])
		json.append(['input_trim',self.input_trim])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['modulation_depth',self.modulation_depth])
		json.append(['voice1_pitch',self.voice1_pitch])
		json.append(['voice2_pitch',self.voice2_pitch])
		json.append(['voice1_resonance',self.voice1_resonance])
		json.append(['voice2_resonance',self.voice2_resonance])
		json.append(['voice1_high_damp',self.voice1_high_damp])
		json.append(['voice2_high_damp',self.voice2_high_damp])
		json.append(['voice1_level',self.voice1_level])
		json.append(['voice2_level',self.voice2_level])
		json.append(['voice1_pan',self.voice1_pan])
		json.append(['voice2_pan',self.voice2_pan])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['voice1_fine',self.voice1_fine])
		json.append(['voice2_fine',self.voice2_fine])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def control_mode(self):
		'''Control Mode'''
		return self._control_mode

	@control_mode.setter
	def control_mode(self,value):
		if 0 <= value <= 2:
			self._control_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_mod_invert(self):
		'''LFO/Mod. Invert'''
		return self._lfo_mod_invert

	@lfo_mod_invert.setter
	def lfo_mod_invert(self,value):
		if 0 <= value <= 1:
			self._lfo_mod_invert = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def control_mod_source(self):
		'''Control Mod.Source'''
		return self._control_mod_source

	@control_mod_source.setter
	def control_mod_source(self,value):
		if 0 <= value <= 37:
			self._control_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def modulation_depth(self):
		'''Modulation Depth'''
		return self._modulation_depth

	@modulation_depth.setter
	def modulation_depth(self,value):
		if -100 <= value <= 100:
			self._modulation_depth = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def voice1_pitch(self):
		'''Voice1 Pitch'''
		return self._voice1_pitch

	@voice1_pitch.setter
	def voice1_pitch(self,value):
		if 0 <= value <= 107:
			self._voice1_pitch = value
		else:
			raise ValueError('%s not 0 <= value <= 107' % value)

#-------------------------------------------------------------------------------
	@property
	def voice1_fine(self):
		'''Voice1 Fine'''
		return self._voice1_fine

	@voice1_fine.setter
	def voice1_fine(self,value):
		if 0 <= value <= 10:
			self._voice1_fine = value
		else:
			raise ValueError('%s not 0 <= value <= 10' % value)

#-------------------------------------------------------------------------------
	@property
	def voice1_resonance(self):
		'''Voice1 Resonance'''
		return self._voice1_resonance

	@voice1_resonance.setter
	def voice1_resonance(self,value):
		if -100 <= value <= 100:
			self._voice1_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def voice1_high_damp(self):
		'''Voice1 High Damp'''
		return self._voice1_high_damp

	@voice1_high_damp.setter
	def voice1_high_damp(self,value):
		if 0 <= value <= 100:
			self._voice1_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def voice1_level(self):
		'''Voice1 Level'''
		return self._voice1_level

	@voice1_level.setter
	def voice1_level(self,value):
		if 0 <= value <= 100:
			self._voice1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def voice1_pan(self):
		'''Voice1 Pan'''
		return self._voice1_pan

	@voice1_pan.setter
	def voice1_pan(self,value):
		if -6 <= value <= 6:
			self._voice1_pan = value
		else:
			raise ValueError('%s not -6 <= value <= 6' % value)

#-------------------------------------------------------------------------------
	@property
	def voice2_pitch(self):
		'''Voice2 Pitch'''
		return self._voice2_pitch

	@voice2_pitch.setter
	def voice2_pitch(self,value):
		if 0 <= value <= 107:
			self._voice2_pitch = value
		else:
			raise ValueError('%s not 0 <= value <= 107' % value)

#-------------------------------------------------------------------------------
	@property
	def voice2_fine(self):
		'''Voice2 Fine'''
		return self._voice2_fine

	@voice2_fine.setter
	def voice2_fine(self,value):
		if 0 <= value <= 10:
			self._voice2_fine = value
		else:
			raise ValueError('%s not 0 <= value <= 10' % value)

#-------------------------------------------------------------------------------
	@property
	def voice2_resonance(self):
		'''Voice2 Resonance'''
		return self._voice2_resonance

	@voice2_resonance.setter
	def voice2_resonance(self,value):
		if -100 <= value <= 100:
			self._voice2_resonance = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def voice2_high_damp(self):
		'''Voice2 High Damp'''
		return self._voice2_high_damp

	@voice2_high_damp.setter
	def voice2_high_damp(self,value):
		if 0 <= value <= 100:
			self._voice2_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def voice2_level(self):
		'''Voice2 Level'''
		return self._voice2_level

	@voice2_level.setter
	def voice2_level(self,value):
		if 0 <= value <= 100:
			self._voice2_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def voice2_pan(self):
		'''Voice2 Pan'''
		return self._voice2_pan

	@voice2_pan.setter
	def voice2_pan(self,value):
		if -6 <= value <= 6:
			self._voice2_pan = value
		else:
			raise ValueError('%s not -6 <= value <= 6' % value)

