#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx057.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

#===============================================================================
class FX057(object):
	category = '''Mod./P.Shift'''
	description = '''Stereo Envelope Tremolo'''
	short_description = '''St. Env. Tremolo'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX057 Stereo Envelope Tremolo object'''
		bytes = struct.unpack('<12B',raw[:12])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.envelope_sens = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		data = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.envelope_shape = data
		self.lfo_waveform = (bytes[5] >> 4) & 0x3
		data = ((bytes[5] & 0xC0) >> 6) | ((bytes[6] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.lfo_shape = data
		data = ((bytes[6] & 0xC0) >> 6) | ((bytes[7] & 0xF) << 2)
		if data & 0x20:
			data -= 0x40
		self.lfo_phase = data
		self.lfo_frequency = ((bytes[7] & 0xF0) >> 4) | ((bytes[8] & 0xF) << 4)
		data = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.lfo_frequency_envelope_amount = data
		self.depth = ((bytes[9] & 0xF0) >> 4) | ((bytes[10] & 0x7) << 4)
		data = ((bytes[10] & 0xF8) >> 3) | ((bytes[11] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.depth_envelope_amount = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX057 Stereo Envelope Tremolo object into raw data'''
		bytes = [0] * 12
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.envelope_sens << 5) & 0xE0
		bytes[4] = ((self.envelope_sens >> 3) & 0xF) | ((self.envelope_shape << 4) & 0xF0)
		bytes[5] = ((self.envelope_shape >> 4) & 0xF) | ((self.lfo_waveform & 0x3) << 4) | ((self.lfo_shape << 6) & 0xC0)
		bytes[6] = ((self.lfo_shape >> 2) & 0x3F) | ((self.lfo_phase << 6) & 0xC0)
		bytes[7] = ((self.lfo_phase >> 2) & 0xF) | ((self.lfo_frequency << 4) & 0xF0)
		bytes[8] = ((self.lfo_frequency >> 4) & 0xF) | ((self.lfo_frequency_envelope_amount << 4) & 0xF0)
		bytes[9] = ((self.lfo_frequency_envelope_amount >> 4) & 0xF) | ((self.depth << 4) & 0xF0)
		bytes[10] = ((self.depth >> 4) & 0x7) | ((self.depth_envelope_amount << 3) & 0xF8)
		bytes[11] = (self.depth_envelope_amount >> 5) & 0x7
		return struct.pack('<12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX057 Stereo Envelope Tremolo object'''
		json = []
		json.append(('name','FX057'))
		json.append(('description','Stereo Envelope Tremolo'))
		json.append(['envelope_sens',self.envelope_sens])
		json.append(['envelope_shape',self.envelope_shape])
		json.append(['lfo_waveform',self.lfo_waveform])
		json.append(['lfo_shape',self.lfo_shape])
		json.append(['lfo_phase',self.lfo_phase])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_frequency_envelope_amount',self.lfo_frequency_envelope_amount])
		json.append(['lfo_frequency_envelope_amount_display',self.lfo_frequency_envelope_amount_display])
		json.append(['depth',self.depth])
		json.append(['depth_envelope_amount',self.depth_envelope_amount])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_sens(self):
		'''Envelope Sens'''
		return self._envelope_sens

	@envelope_sens.setter
	def envelope_sens(self,value):
		if 0 <= value <= 100:
			self._envelope_sens = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def envelope_shape(self):
		'''Envelope Shape'''
		return self._envelope_shape

	@envelope_shape.setter
	def envelope_shape(self,value):
		if -100 <= value <= 100:
			self._envelope_shape = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_waveform(self):
		'''LFO Waveform'''
		return self._lfo_waveform

	@lfo_waveform.setter
	def lfo_waveform(self,value):
		if 0 <= value <= 2:
			self._lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_shape(self):
		'''LFO Shape'''
		return self._lfo_shape

	@lfo_shape.setter
	def lfo_shape(self,value):
		if -100 <= value <= 100:
			self._lfo_shape = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_phase(self):
		'''LFO Phase'''
		return self._lfo_phase

	@lfo_phase.setter
	def lfo_phase(self,value):
		if -18 <= value <= 18:
			self._lfo_phase = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_envelope_amount(self):
		'''LFO Frequency Envelope Amount'''
		return self._lfo_frequency_envelope_amount

	@lfo_frequency_envelope_amount.setter
	def lfo_frequency_envelope_amount(self,value):
		if -115 <= value <= 115:
			self._lfo_frequency_envelope_amount = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def lfo_frequency_envelope_amount_display(self):
		'''LFO Frequency Envelope Amount display value'''
		return rng_8d_73_m20_p20_decode(self._lfo_frequency_envelope_amount)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_envelope_amount(self):
		'''Depth Envelope Amount'''
		return self._depth_envelope_amount

	@depth_envelope_amount.setter
	def depth_envelope_amount(self,value):
		if -100 <= value <= 100:
			self._depth_envelope_amount = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

