#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx059.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX059(object):
	category = '''Mod./P.Shift'''
	description = '''Stereo Phaser + Tremolo'''
	short_description = '''St. Phaser + Trml'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX059 Stereo Phaser + Tremolo object'''
		bytes = struct.unpack('<20B',raw[:20])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.lfo_type = (bytes[3] >> 5) & 0x3
		data = ((bytes[3] & 0x80) >> 7) | ((bytes[4] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.type = (bytes[4] >> 5) & 0x7
		data = bytes[5] & 0x7
		if data & 0x4:
			data -= 0x8
		self.lfo_phase = data
		self.lfo_frequency = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x7) << 5)
		self.lfo_frequency_mod_source = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x1) << 5)
		data = ((bytes[7] & 0xFE) >> 1) | ((bytes[8] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.lfo_frequency_mod_intensity = data
		self.lfo_bpm_midi_sync = (bytes[8] >> 1) & 0x1
		self.lfo_bpm = ((bytes[8] & 0xFC) >> 2) | (bytes[9] << 6) | ((bytes[10] & 0x1) << 14)
		self.lfo_bpm_base_note = (bytes[10] >> 1) & 0xF
		self.lfo_bpm_times = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x7) << 3)
		self.phaser_manual = ((bytes[11] & 0xF8) >> 3) | ((bytes[12] & 0x3) << 5)
		self.phaser_depth = (bytes[12] >> 2) & 0x3F
		self.phaser_depth_mod_source = bytes[13] & 0x3F
		data = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0x1F) << 2)
		if data & 0x40:
			data -= 0x80
		self.phaser_depth_mod_intensity = data
		data = ((bytes[14] & 0xE0) >> 5) | ((bytes[15] & 0xF) << 3)
		if data & 0x40:
			data -= 0x80
		self.phaser_resonance = data
		data = ((bytes[15] & 0xF0) >> 4) | ((bytes[16] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.phaser_wet_dry = data
		data = ((bytes[16] & 0xF8) >> 3) | ((bytes[17] & 0x3) << 5)
		if data & 0x40:
			data -= 0x80
		self.tremolo_shape = data
		self.tremolo_depth = (bytes[17] >> 2) & 0x3F
		self.tremolo_depth_mod_source = bytes[18] & 0x3F
		data = ((bytes[18] & 0xC0) >> 6) | ((bytes[19] & 0x1F) << 2)
		if data & 0x40:
			data -= 0x80
		self.tremolo_depth_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX059 Stereo Phaser + Tremolo object into raw data'''
		bytes = [0] * 20
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.lfo_type & 0x3) << 5) | ((self.common_lfo_phase_offset << 7) & 0x80)
		bytes[4] = ((self.common_lfo_phase_offset >> 1) & 0x1F) | ((self.type & 0x7) << 5)
		bytes[5] = (self.lfo_phase & 0x7) | ((self.lfo_frequency << 3) & 0xF8)
		bytes[6] = ((self.lfo_frequency >> 5) & 0x7) | ((self.lfo_frequency_mod_source << 3) & 0xF8)
		bytes[7] = ((self.lfo_frequency_mod_source >> 5) & 0x1) | ((self.lfo_frequency_mod_intensity << 1) & 0xFE)
		bytes[8] = ((self.lfo_frequency_mod_intensity >> 7) & 0x1) | ((self.lfo_bpm_midi_sync & 0x1) << 1) | ((self.lfo_bpm << 2) & 0xFC)
		bytes[9] = (self.lfo_bpm >> 6) & 0xFF
		bytes[10] = ((self.lfo_bpm >> 1) & 0xE) | ((self.lfo_bpm_base_note & 0xF) << 1) | ((self.lfo_bpm_times << 5) & 0xE0)
		bytes[11] = ((self.lfo_bpm_times >> 3) & 0x7) | ((self.phaser_manual << 3) & 0xF8)
		bytes[12] = ((self.phaser_manual >> 5) & 0x3) | ((self.phaser_depth & 0x3F) << 2)
		bytes[13] = (self.phaser_depth_mod_source & 0x3F) | ((self.phaser_depth_mod_intensity << 6) & 0xC0)
		bytes[14] = ((self.phaser_depth_mod_intensity >> 2) & 0x1F) | ((self.phaser_resonance << 5) & 0xE0)
		bytes[15] = ((self.phaser_resonance >> 3) & 0xF) | ((self.phaser_wet_dry << 4) & 0xF0)
		bytes[16] = ((self.phaser_wet_dry >> 4) & 0x7) | ((self.tremolo_shape << 3) & 0xF8)
		bytes[17] = ((self.tremolo_shape >> 5) & 0x3) | ((self.tremolo_depth & 0x3F) << 2)
		bytes[18] = (self.tremolo_depth_mod_source & 0x3F) | ((self.tremolo_depth_mod_intensity << 6) & 0xC0)
		bytes[19] = (self.tremolo_depth_mod_intensity >> 2) & 0x1F
		return struct.pack('<20B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX059 Stereo Phaser + Tremolo object'''
		json = []
		json.append(('name','FX059'))
		json.append(('description','Stereo Phaser + Tremolo'))
		json.append(['type',self.type])
		json.append(['lfo_phase',self.lfo_phase])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_frequency_mod_source',self.lfo_frequency_mod_source])
		json.append(['lfo_frequency_mod_intensity',self.lfo_frequency_mod_intensity])
		json.append(['lfo_frequency_mod_intensity_display',self.lfo_frequency_mod_intensity_display])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		json.append(['phaser_manual',self.phaser_manual])
		json.append(['phaser_depth',self.phaser_depth])
		json.append(['phaser_depth_mod_source',self.phaser_depth_mod_source])
		json.append(['phaser_depth_mod_intensity',self.phaser_depth_mod_intensity])
		json.append(['phaser_resonance',self.phaser_resonance])
		json.append(['phaser_wet_dry',self.phaser_wet_dry])
		json.append(['tremolo_shape',self.tremolo_shape])
		json.append(['tremolo_depth',self.tremolo_depth])
		json.append(['tremolo_depth_mod_source',self.tremolo_depth_mod_source])
		json.append(['tremolo_depth_mod_intensity',self.tremolo_depth_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def type(self):
		'''Type'''
		return self._type

	@type.setter
	def type(self,value):
		if 0 <= value <= 5:
			self._type = value
		else:
			raise ValueError('%s not 0 <= value <= 5' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_phase(self):
		'''LFO Phase'''
		return self._lfo_phase

	@lfo_phase.setter
	def lfo_phase(self,value):
		if -2 <= value <= 2:
			self._lfo_phase = value
		else:
			raise ValueError('%s not -2 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_source(self):
		'''LFO Frequency Mod.Source'''
		return self._lfo_frequency_mod_source

	@lfo_frequency_mod_source.setter
	def lfo_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._lfo_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_intensity(self):
		'''LFO Frequency Mod.Intensity'''
		return self._lfo_frequency_mod_intensity

	@lfo_frequency_mod_intensity.setter
	def lfo_frequency_mod_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def lfo_frequency_mod_intensity_display(self):
		'''LFO Frequency Mod.Intensity display value'''
		return rng_8d_73_m20_p20_decode(self._lfo_frequency_mod_intensity)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_manual(self):
		'''Phaser Manual'''
		return self._phaser_manual

	@phaser_manual.setter
	def phaser_manual(self,value):
		if 0 <= value <= 100:
			self._phaser_manual = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth(self):
		'''Phaser Depth'''
		return self._phaser_depth

	@phaser_depth.setter
	def phaser_depth(self,value):
		if 0 <= value <= 50:
			self._phaser_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth_mod_source(self):
		'''Phaser Depth Mod.Source'''
		return self._phaser_depth_mod_source

	@phaser_depth_mod_source.setter
	def phaser_depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._phaser_depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_depth_mod_intensity(self):
		'''Phaser Depth Mod.Intensity'''
		return self._phaser_depth_mod_intensity

	@phaser_depth_mod_intensity.setter
	def phaser_depth_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._phaser_depth_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_resonance(self):
		'''Phaser Resonance'''
		return self._phaser_resonance

	@phaser_resonance.setter
	def phaser_resonance(self,value):
		if -50 <= value <= 50:
			self._phaser_resonance = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def phaser_wet_dry(self):
		'''Phaser Wet/Dry'''
		return self._phaser_wet_dry

	@phaser_wet_dry.setter
	def phaser_wet_dry(self,value):
		if -50 <= value <= 50:
			self._phaser_wet_dry = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def tremolo_shape(self):
		'''Tremolo Shape'''
		return self._tremolo_shape

	@tremolo_shape.setter
	def tremolo_shape(self,value):
		if -50 <= value <= 50:
			self._tremolo_shape = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def tremolo_depth(self):
		'''Tremolo Depth'''
		return self._tremolo_depth

	@tremolo_depth.setter
	def tremolo_depth(self,value):
		if 0 <= value <= 50:
			self._tremolo_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def tremolo_depth_mod_source(self):
		'''Tremolo Depth Mod.Source'''
		return self._tremolo_depth_mod_source

	@tremolo_depth_mod_source.setter
	def tremolo_depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._tremolo_depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def tremolo_depth_mod_intensity(self):
		'''Tremolo Depth Mod.Intensity'''
		return self._tremolo_depth_mod_intensity

	@tremolo_depth_mod_intensity.setter
	def tremolo_depth_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._tremolo_depth_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

