#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx060.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def rng_8d_73_m20_p20_decode(v):
	a1 = v*0.06 if -0x32 <= v <= 0x32 else math.copysign(3.0,v)
	a2 = v*0.1 if -0x50 <= v <= 0x50 else math.copysign(8.0,v)
	a3 = v*0.8 if -0x69 <= v <= 0x69 else math.copysign(84.0,v)
	return v*1.0-a1-a2-a3

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX060(object):
	category = '''Mod./P.Shift'''
	description = '''Stereo Ring Modulator'''
	short_description = '''St. Ring Modulator'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX060 Stereo Ring Modulator object'''
		bytes = struct.unpack('<21B',raw[:21])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.pre_lpf = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.osc_mode = (bytes[4] >> 4) & 0x1
		self.fixed_frequency = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x1F) << 3)
		self.fixed_frequency_mod_source = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x7) << 3)
		data = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.fixed_frequency_mod_intensity = data
		data = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x3) << 5)
		if data & 0x40:
			data -= 0x80
		self.note_offset = data
		data = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.note_fine = data
		self.lfo_type = (bytes[9] >> 2) & 0x3
		data = ((bytes[9] & 0xF0) >> 4) | ((bytes[10] & 0x3) << 4)
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_frequency = ((bytes[10] & 0xFC) >> 2) | ((bytes[11] & 0x3) << 6)
		self.lfo_frequency_mod_source = (bytes[11] >> 2) & 0x3F
		data = bytes[12] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.lfo_frequency_mod_intensity = data
		self.lfo_bpm_midi_sync = bytes[13] & 0x1
		self.lfo_bpm = ((bytes[13] & 0xFE) >> 1) | ((bytes[14] & 0xFF) << 7)
		self.lfo_bpm_base_note = bytes[15] & 0xF
		self.lfo_bpm_times = ((bytes[15] & 0xF0) >> 4) | ((bytes[16] & 0x3) << 4)
		self.depth = ((bytes[16] & 0xFC) >> 2) | ((bytes[17] & 0x1) << 6)
		self.depth_mod_source = (bytes[17] >> 1) & 0x3F
		data = ((bytes[17] & 0x80) >> 7) | ((bytes[18] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.depth_mod_intensity = data
		self.direct_mix = ((bytes[19] & 0xE0) >> 5) | ((bytes[20] & 0xF) << 3)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX060 Stereo Ring Modulator object into raw data'''
		bytes = [0] * 21
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.pre_lpf << 5) & 0xE0
		bytes[4] = ((self.pre_lpf >> 3) & 0xF) | ((self.osc_mode & 0x1) << 4) | ((self.fixed_frequency << 5) & 0xE0)
		bytes[5] = ((self.fixed_frequency >> 3) & 0x1F) | ((self.fixed_frequency_mod_source << 5) & 0xE0)
		bytes[6] = ((self.fixed_frequency_mod_source >> 3) & 0x7) | ((self.fixed_frequency_mod_intensity << 3) & 0xF8)
		bytes[7] = ((self.fixed_frequency_mod_intensity >> 5) & 0x7) | ((self.note_offset << 3) & 0xF8)
		bytes[8] = ((self.note_offset >> 5) & 0x3) | ((self.note_fine << 2) & 0xFC)
		bytes[9] = ((self.note_fine >> 6) & 0x3) | ((self.lfo_type & 0x3) << 2) | ((self.common_lfo_phase_offset << 4) & 0xF0)
		bytes[10] = ((self.common_lfo_phase_offset >> 4) & 0x3) | ((self.lfo_frequency << 2) & 0xFC)
		bytes[11] = ((self.lfo_frequency >> 6) & 0x3) | ((self.lfo_frequency_mod_source & 0x3F) << 2)
		bytes[12] = self.lfo_frequency_mod_intensity & 0xFF
		bytes[13] = (self.lfo_bpm_midi_sync & 0x1) | ((self.lfo_bpm << 1) & 0xFE)
		bytes[14] = (self.lfo_bpm >> 7) & 0xFF
		bytes[15] = (self.lfo_bpm_base_note & 0xF) | ((self.lfo_bpm_times << 4) & 0xF0)
		bytes[16] = ((self.lfo_bpm_times >> 4) & 0x3) | ((self.depth << 2) & 0xFC)
		bytes[17] = ((self.depth >> 6) & 0x1) | ((self.depth_mod_source & 0x3F) << 1) | ((self.depth_mod_intensity << 7) & 0x80)
		bytes[18] = (self.depth_mod_intensity >> 1) & 0x7F
		bytes[19] = (self.direct_mix << 5) & 0xE0
		bytes[20] = (self.direct_mix >> 3) & 0xF
		return struct.pack('<21B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX060 Stereo Ring Modulator object'''
		json = []
		json.append(('name','FX060'))
		json.append(('description','Stereo Ring Modulator'))
		json.append(['pre_lpf',self.pre_lpf])
		json.append(['osc_mode',self.osc_mode])
		json.append(['fixed_frequency',self.fixed_frequency])
		json.append(['fixed_frequency_mod_source',self.fixed_frequency_mod_source])
		json.append(['fixed_frequency_mod_intensity',self.fixed_frequency_mod_intensity])
		json.append(['note_offset',self.note_offset])
		json.append(['note_fine',self.note_fine])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_frequency_mod_source',self.lfo_frequency_mod_source])
		json.append(['lfo_frequency_mod_intensity',self.lfo_frequency_mod_intensity])
		json.append(['lfo_frequency_mod_intensity_display',self.lfo_frequency_mod_intensity_display])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		json.append(['depth',self.depth])
		json.append(['depth_mod_source',self.depth_mod_source])
		json.append(['depth_mod_intensity',self.depth_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		json.append(['direct_mix',self.direct_mix])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_lpf(self):
		'''Pre LPF'''
		return self._pre_lpf

	@pre_lpf.setter
	def pre_lpf(self,value):
		if 0 <= value <= 100:
			self._pre_lpf = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def osc_mode(self):
		'''Osc Mode'''
		return self._osc_mode

	@osc_mode.setter
	def osc_mode(self,value):
		if 0 <= value <= 1:
			self._osc_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def fixed_frequency(self):
		'''Fixed Frequency'''
		return self._fixed_frequency

	@fixed_frequency.setter
	def fixed_frequency(self,value):
		if 0 <= value <= 228:
			self._fixed_frequency = value
		else:
			raise ValueError('%s not 0 <= value <= 228' % value)

#-------------------------------------------------------------------------------
	@property
	def fixed_frequency_mod_source(self):
		'''Fixed Frequency Mod.Source'''
		return self._fixed_frequency_mod_source

	@fixed_frequency_mod_source.setter
	def fixed_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._fixed_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def fixed_frequency_mod_intensity(self):
		'''Fixed Frequency Mod.Intensity'''
		return self._fixed_frequency_mod_intensity

	@fixed_frequency_mod_intensity.setter
	def fixed_frequency_mod_intensity(self,value):
		if -114 <= value <= 114:
			self._fixed_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -114 <= value <= 114' % value)

#-------------------------------------------------------------------------------
	@property
	def note_offset(self):
		'''Note Offset'''
		return self._note_offset

	@note_offset.setter
	def note_offset(self,value):
		if -48 <= value <= 48:
			self._note_offset = value
		else:
			raise ValueError('%s not -48 <= value <= 48' % value)

#-------------------------------------------------------------------------------
	@property
	def note_fine(self):
		'''Note Fine'''
		return self._note_fine

	@note_fine.setter
	def note_fine(self,value):
		if -100 <= value <= 100:
			self._note_fine = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_source(self):
		'''LFO Frequency Mod.Source'''
		return self._lfo_frequency_mod_source

	@lfo_frequency_mod_source.setter
	def lfo_frequency_mod_source(self,value):
		if 0 <= value <= 37:
			self._lfo_frequency_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency_mod_intensity(self):
		'''LFO Frequency Mod.Intensity'''
		return self._lfo_frequency_mod_intensity

	@lfo_frequency_mod_intensity.setter
	def lfo_frequency_mod_intensity(self,value):
		if -115 <= value <= 115:
			self._lfo_frequency_mod_intensity = value
		else:
			raise ValueError('%s not -115 <= value <= 115' % value)

	@property
	def lfo_frequency_mod_intensity_display(self):
		'''LFO Frequency Mod.Intensity display value'''
		return rng_8d_73_m20_p20_decode(self._lfo_frequency_mod_intensity)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_source(self):
		'''Depth Mod.Source'''
		return self._depth_mod_source

	@depth_mod_source.setter
	def depth_mod_source(self,value):
		if 0 <= value <= 37:
			self._depth_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def depth_mod_intensity(self):
		'''Depth Mod.Intensity'''
		return self._depth_mod_intensity

	@depth_mod_intensity.setter
	def depth_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._depth_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def direct_mix(self):
		'''Direct Mix'''
		return self._direct_mix

	@direct_mix.setter
	def direct_mix(self,value):
		if 0 <= value <= 100:
			self._direct_mix = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

