#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx061.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX061(object):
	category = '''Mod./P.Shift'''
	description = '''Detune'''
	short_description = '''Detune'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX061 Detune object'''
		bytes = struct.unpack('<11B',raw[:11])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		data = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.pitch_shift = data
		self.pitch_shift_mod_source = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x7) << 3)
		data = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.pitch_shift_mod_intensity = data
		self.delay_time = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x7) << 5)
		data = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.high_damp = ((bytes[8] & 0xF8) >> 3) | ((bytes[9] & 0x3) << 5)
		self.input_trim_mod_source = (bytes[9] >> 2) & 0x3F
		data = bytes[10] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.input_trim_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX061 Detune object into raw data'''
		bytes = [0] * 11
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.pitch_shift << 5) & 0xE0
		bytes[4] = ((self.pitch_shift >> 3) & 0x1F) | ((self.pitch_shift_mod_source << 5) & 0xE0)
		bytes[5] = ((self.pitch_shift_mod_source >> 3) & 0x7) | ((self.pitch_shift_mod_intensity << 3) & 0xF8)
		bytes[6] = ((self.pitch_shift_mod_intensity >> 5) & 0x7) | ((self.delay_time << 3) & 0xF8)
		bytes[7] = ((self.delay_time >> 5) & 0x7) | ((self.feedback << 3) & 0xF8)
		bytes[8] = ((self.feedback >> 5) & 0x7) | ((self.high_damp << 3) & 0xF8)
		bytes[9] = ((self.high_damp >> 5) & 0x3) | ((self.input_trim_mod_source & 0x3F) << 2)
		bytes[10] = self.input_trim_mod_intensity & 0xFF
		return struct.pack('<11B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX061 Detune object'''
		json = []
		json.append(('name','FX061'))
		json.append(('description','Detune'))
		json.append(['pitch_shift',self.pitch_shift])
		json.append(['pitch_shift_mod_source',self.pitch_shift_mod_source])
		json.append(['pitch_shift_mod_intensity',self.pitch_shift_mod_intensity])
		json.append(['delay_time',self.delay_time])
		json.append(['feedback',self.feedback])
		json.append(['high_damp',self.high_damp])
		json.append(['input_trim_mod_source',self.input_trim_mod_source])
		json.append(['input_trim_mod_intensity',self.input_trim_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift(self):
		'''Pitch Shift'''
		return self._pitch_shift

	@pitch_shift.setter
	def pitch_shift(self,value):
		if -100 <= value <= 100:
			self._pitch_shift = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_mod_source(self):
		'''Pitch Shift Mod.Source'''
		return self._pitch_shift_mod_source

	@pitch_shift_mod_source.setter
	def pitch_shift_mod_source(self,value):
		if 0 <= value <= 37:
			self._pitch_shift_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_mod_intensity(self):
		'''Pitch Shift Mod.Intensity'''
		return self._pitch_shift_mod_intensity

	@pitch_shift_mod_intensity.setter
	def pitch_shift_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._pitch_shift_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time(self):
		'''Delay Time'''
		return self._delay_time

	@delay_time.setter
	def delay_time(self,value):
		if 0 <= value <= 77:
			self._delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 77' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_source(self):
		'''Input Trim Mod.Source'''
		return self._input_trim_mod_source

	@input_trim_mod_source.setter
	def input_trim_mod_source(self,value):
		if 0 <= value <= 37:
			self._input_trim_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_intensity(self):
		'''Input Trim Mod.Intensity'''
		return self._input_trim_mod_intensity

	@input_trim_mod_intensity.setter
	def input_trim_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._input_trim_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

