#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx063.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX063(object):
	category = '''Mod./P.Shift'''
	description = '''Pitch Shifter BPM'''
	short_description = '''Pitch Shifter BPM'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX063 Pitch Shifter BPM object'''
		bytes = struct.unpack('<15B',raw[:15])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.mode = (bytes[3] >> 5) & 0x3
		data = ((bytes[3] & 0x80) >> 7) | ((bytes[4] & 0x1F) << 1)
		if data & 0x20:
			data -= 0x40
		self.pitch_shift_coarse = data
		self.pitch_shift_mod_source = ((bytes[4] & 0xE0) >> 5) | ((bytes[5] & 0x7) << 3)
		data = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		if data & 0x20:
			data -= 0x40
		self.pitch_shift_coarse_mod_intensity = data
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.pitch_shift_fine = data
		data = ((bytes[7] & 0xFE) >> 1) | ((bytes[8] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.pitch_shift_fine_mod_intensity = data
		self.bpm = ((bytes[8] & 0xFE) >> 1) | ((bytes[9] & 0xFF) << 7)
		self.base_note = bytes[10] & 0xF
		self.times = ((bytes[10] & 0xF0) >> 4) | ((bytes[11] & 0x3) << 4)
		self.feedback_position = (bytes[11] >> 2) & 0x1
		data = ((bytes[11] & 0xF8) >> 3) | ((bytes[12] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.high_damp = ((bytes[12] & 0xF8) >> 3) | ((bytes[13] & 0x3) << 5)
		self.input_trim_mod_source = (bytes[13] >> 2) & 0x3F
		data = bytes[14] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.input_trim_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX063 Pitch Shifter BPM object into raw data'''
		bytes = [0] * 15
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.mode & 0x3) << 5) | ((self.pitch_shift_coarse << 7) & 0x80)
		bytes[4] = ((self.pitch_shift_coarse >> 1) & 0x1F) | ((self.pitch_shift_mod_source << 5) & 0xE0)
		bytes[5] = ((self.pitch_shift_mod_source >> 3) & 0x7) | ((self.pitch_shift_coarse_mod_intensity << 3) & 0xF8)
		bytes[6] = ((self.pitch_shift_coarse_mod_intensity >> 5) & 0x1) | ((self.pitch_shift_fine << 1) & 0xFE)
		bytes[7] = ((self.pitch_shift_fine >> 7) & 0x1) | ((self.pitch_shift_fine_mod_intensity << 1) & 0xFE)
		bytes[8] = ((self.pitch_shift_fine_mod_intensity >> 7) & 0x1) | ((self.bpm << 1) & 0xFE)
		bytes[9] = (self.bpm >> 7) & 0xFF
		bytes[10] = (self.base_note & 0xF) | ((self.times << 4) & 0xF0)
		bytes[11] = ((self.times >> 4) & 0x3) | ((self.feedback_position & 0x1) << 2) | ((self.feedback << 3) & 0xF8)
		bytes[12] = ((self.feedback >> 5) & 0x7) | ((self.high_damp << 3) & 0xF8)
		bytes[13] = ((self.high_damp >> 5) & 0x3) | ((self.input_trim_mod_source & 0x3F) << 2)
		bytes[14] = self.input_trim_mod_intensity & 0xFF
		return struct.pack('<15B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX063 Pitch Shifter BPM object'''
		json = []
		json.append(('name','FX063'))
		json.append(('description','Pitch Shifter BPM'))
		json.append(['mode',self.mode])
		json.append(['pitch_shift_coarse',self.pitch_shift_coarse])
		json.append(['pitch_shift_mod_source',self.pitch_shift_mod_source])
		json.append(['pitch_shift_coarse_mod_intensity',self.pitch_shift_coarse_mod_intensity])
		json.append(['pitch_shift_fine',self.pitch_shift_fine])
		json.append(['pitch_shift_fine_mod_intensity',self.pitch_shift_fine_mod_intensity])
		json.append(['feedback_position',self.feedback_position])
		json.append(['feedback',self.feedback])
		json.append(['high_damp',self.high_damp])
		json.append(['input_trim_mod_source',self.input_trim_mod_source])
		json.append(['input_trim_mod_intensity',self.input_trim_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['bpm',self.bpm])
		json.append(['bpm_display',self.bpm_display])
		json.append(['base_note',self.base_note])
		json.append(['base_note_display',self.base_note_display])
		json.append(['times',self.times])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mode(self):
		'''Mode'''
		return self._mode

	@mode.setter
	def mode(self,value):
		if 0 <= value <= 2:
			self._mode = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_coarse(self):
		'''Pitch Shift Coarse'''
		return self._pitch_shift_coarse

	@pitch_shift_coarse.setter
	def pitch_shift_coarse(self,value):
		if -24 <= value <= 24:
			self._pitch_shift_coarse = value
		else:
			raise ValueError('%s not -24 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_mod_source(self):
		'''Pitch Shift Mod.Source'''
		return self._pitch_shift_mod_source

	@pitch_shift_mod_source.setter
	def pitch_shift_mod_source(self,value):
		if 0 <= value <= 37:
			self._pitch_shift_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_coarse_mod_intensity(self):
		'''Pitch Shift Coarse Mod.Intensity'''
		return self._pitch_shift_coarse_mod_intensity

	@pitch_shift_coarse_mod_intensity.setter
	def pitch_shift_coarse_mod_intensity(self,value):
		if -24 <= value <= 24:
			self._pitch_shift_coarse_mod_intensity = value
		else:
			raise ValueError('%s not -24 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_fine(self):
		'''Pitch Shift Fine'''
		return self._pitch_shift_fine

	@pitch_shift_fine.setter
	def pitch_shift_fine(self,value):
		if -100 <= value <= 100:
			self._pitch_shift_fine = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pitch_shift_fine_mod_intensity(self):
		'''Pitch Shift Fine Mod.Intensity'''
		return self._pitch_shift_fine_mod_intensity

	@pitch_shift_fine_mod_intensity.setter
	def pitch_shift_fine_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._pitch_shift_fine_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bpm(self):
		'''BPM'''
		return self._bpm

	@bpm.setter
	def bpm(self,value):
		if 3999 <= value <= 30000:
			self._bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def bpm_display(self):
		'''BPM display value'''
		return bpm_decode(self._bpm)

#-------------------------------------------------------------------------------
	@property
	def base_note(self):
		'''Base Note'''
		return self._base_note

	@base_note.setter
	def base_note(self,value):
		if 0 <= value <= 9:
			self._base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def base_note_display(self):
		'''Base Note display value'''
		return base_note_dict[self._base_note]

#-------------------------------------------------------------------------------
	@property
	def times(self):
		'''Times'''
		return self._times

	@times.setter
	def times(self,value):
		if 1 <= value <= 32:
			self._times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_position(self):
		'''Feedback Position'''
		return self._feedback_position

	@feedback_position.setter
	def feedback_position(self,value):
		if 0 <= value <= 1:
			self._feedback_position = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_source(self):
		'''Input Trim Mod.Source'''
		return self._input_trim_mod_source

	@input_trim_mod_source.setter
	def input_trim_mod_source(self,value):
		if 0 <= value <= 37:
			self._input_trim_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_intensity(self):
		'''Input Trim Mod.Intensity'''
		return self._input_trim_mod_intensity

	@input_trim_mod_intensity.setter
	def input_trim_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._input_trim_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

