#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx069.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX069(object):
	category = '''Delay'''
	description = '''Stereo/Cross Delay'''
	short_description = '''Stereo/Cross Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX069 Stereo/Cross Delay object'''
		bytes = struct.unpack('<17B',raw[:17])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.stereo_cross = (bytes[3] >> 5) & 0x1
		self.left_delay_time = ((bytes[3] & 0xC0) >> 6) | (bytes[4] << 2) | ((bytes[5] & 0xF) << 10)
		self.right_delay_time = ((bytes[5] & 0xF0) >> 4) | (bytes[6] << 4) | ((bytes[7] & 0x3) << 12)
		data = ((bytes[7] & 0xFC) >> 2) | ((bytes[8] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.left_feedback = data
		self.feedback_mod_source = (bytes[8] >> 2) & 0x3F
		data = bytes[9] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.left_feedback_mod_intensity = data
		data = bytes[10] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.right_feedback = data
		data = bytes[11] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.right_feedback_mod_intensity = data
		self.high_damp = bytes[12] & 0x7F
		self.low_damp = ((bytes[12] & 0x80) >> 7) | ((bytes[13] & 0x3F) << 1)
		self.input_trim_mod_source = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0xF) << 2)
		data = ((bytes[14] & 0xF0) >> 4) | ((bytes[15] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.input_trim_mod_intensity = data
		data = ((bytes[15] & 0xF0) >> 4) | ((bytes[16] & 0x7) << 4)
		if data & 0x40:
			data -= 0x80
		self.spread = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX069 Stereo/Cross Delay object into raw data'''
		bytes = [0] * 17
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.stereo_cross & 0x1) << 5) | ((self.left_delay_time << 6) & 0xC0)
		bytes[4] = (self.left_delay_time >> 2) & 0xFF
		bytes[5] = ((self.left_delay_time >> 15) & 0xA) | ((self.right_delay_time << 4) & 0xF0)
		bytes[6] = (self.right_delay_time >> 4) & 0xFF
		bytes[7] = ((self.right_delay_time >> 3) & 0xC) | ((self.left_feedback << 2) & 0xFC)
		bytes[8] = ((self.left_feedback >> 6) & 0x3) | ((self.feedback_mod_source & 0x3F) << 2)
		bytes[9] = self.left_feedback_mod_intensity & 0xFF
		bytes[10] = self.right_feedback & 0xFF
		bytes[11] = self.right_feedback_mod_intensity & 0xFF
		bytes[12] = (self.high_damp & 0x7F) | ((self.low_damp << 7) & 0x80)
		bytes[13] = ((self.low_damp >> 1) & 0x3F) | ((self.input_trim_mod_source << 6) & 0xC0)
		bytes[14] = ((self.input_trim_mod_source >> 2) & 0xF) | ((self.input_trim_mod_intensity << 4) & 0xF0)
		bytes[15] = ((self.input_trim_mod_intensity >> 4) & 0xF) | ((self.spread << 4) & 0xF0)
		bytes[16] = (self.spread >> 4) & 0x7
		return struct.pack('<17B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX069 Stereo/Cross Delay object'''
		json = []
		json.append(('name','FX069'))
		json.append(('description','Stereo/Cross Delay'))
		json.append(['left_delay_time',self.left_delay_time])
		json.append(['right_delay_time',self.right_delay_time])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['left_feedback',self.left_feedback])
		json.append(['right_feedback',self.right_feedback])
		json.append(['feedback_mod_source',self.feedback_mod_source])
		json.append(['left_feedback_mod_intensity',self.left_feedback_mod_intensity])
		json.append(['right_feedback_mod_intensity',self.right_feedback_mod_intensity])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['input_trim_mod_source',self.input_trim_mod_source])
		json.append(['input_trim_mod_intensity',self.input_trim_mod_intensity])
		json.append(['spread',self.spread])
		json.append(['stereo_cross',self.stereo_cross])
		json.append(['stereo_cross_display',self.stereo_cross_display])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def stereo_cross(self):
		'''Stereo/Cross'''
		return self._stereo_cross

	@stereo_cross.setter
	def stereo_cross(self,value):
		if 0 <= value <= 1:
			self._stereo_cross = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

	@property
	def stereo_cross_display(self):
		'''Stereo/Cross display value'''
		return {0: 'Stereo', 1: 'Cross'}[self._stereo_cross]

#-------------------------------------------------------------------------------
	@property
	def left_delay_time(self):
		'''Left Delay Time'''
		return self._left_delay_time

	@left_delay_time.setter
	def left_delay_time(self,value):
		if 0 <= value <= 13600:
			self._left_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 13600' % value)

#-------------------------------------------------------------------------------
	@property
	def right_delay_time(self):
		'''Right Delay Time'''
		return self._right_delay_time

	@right_delay_time.setter
	def right_delay_time(self,value):
		if 0 <= value <= 13600:
			self._right_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 13600' % value)

#-------------------------------------------------------------------------------
	@property
	def left_feedback(self):
		'''Left Feedback'''
		return self._left_feedback

	@left_feedback.setter
	def left_feedback(self,value):
		if -100 <= value <= 100:
			self._left_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_source(self):
		'''Feedback Mod.Source'''
		return self._feedback_mod_source

	@feedback_mod_source.setter
	def feedback_mod_source(self,value):
		if 0 <= value <= 37:
			self._feedback_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def left_feedback_mod_intensity(self):
		'''Left Feedback Mod.Intensity'''
		return self._left_feedback_mod_intensity

	@left_feedback_mod_intensity.setter
	def left_feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._left_feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def right_feedback(self):
		'''Right Feedback'''
		return self._right_feedback

	@right_feedback.setter
	def right_feedback(self,value):
		if -100 <= value <= 100:
			self._right_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def right_feedback_mod_intensity(self):
		'''Right Feedback Mod.Intensity'''
		return self._right_feedback_mod_intensity

	@right_feedback_mod_intensity.setter
	def right_feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._right_feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_source(self):
		'''Input Trim Mod.Source'''
		return self._input_trim_mod_source

	@input_trim_mod_source.setter
	def input_trim_mod_source(self,value):
		if 0 <= value <= 37:
			self._input_trim_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_intensity(self):
		'''Input Trim Mod.Intensity'''
		return self._input_trim_mod_intensity

	@input_trim_mod_intensity.setter
	def input_trim_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._input_trim_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def spread(self):
		'''Spread'''
		return self._spread

	@spread.setter
	def spread(self,value):
		if -50 <= value <= 50:
			self._spread = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

