#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx071.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX071(object):
	category = '''Delay'''
	description = '''Stereo Modulation Delay'''
	short_description = '''St. Mod Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX071 Stereo Modulation Delay object'''
		bytes = struct.unpack('<22B',raw[:22])
		data = ((bytes[0] & 0xFE) >> 1) | ((bytes[1] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.wet_dry = data
		self.wet_dry_mod_source = (bytes[1] >> 1) & 0x3F
		data = ((bytes[1] & 0x80) >> 7) | ((bytes[2] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.modulation_mode = (bytes[3] >> 6) & 0x1
		self.mod_l_r_polarity = (bytes[3] >> 7) & 0x1
		self.mod_source = bytes[4] & 0x3F
		self.mod_response = ((bytes[4] & 0xC0) >> 6) | ((bytes[5] & 0x7) << 2)
		self.lfo_waveform = (bytes[5] >> 3) & 0x1
		data = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.lfo_shape = data
		self.lfo_type = (bytes[6] >> 4) & 0x3
		data = ((bytes[6] & 0xC0) >> 6) | ((bytes[7] & 0xF) << 2)
		if data & 0x20:
			data -= 0x40
		self.common_lfo_phase_offset = data
		self.lfo_frequency = ((bytes[7] & 0xF0) >> 4) | ((bytes[8] & 0xF) << 4)
		self.lfo_bpm_midi_sync = (bytes[8] >> 4) & 0x1
		self.lfo_bpm = ((bytes[8] & 0xE0) >> 5) | (bytes[9] << 3) | ((bytes[10] & 0xF) << 11)
		self.lfo_bpm_base_note = (bytes[10] >> 4) & 0xF
		self.lfo_bpm_times = bytes[11] & 0x3F
		self.lfo_sync = (bytes[11] >> 6) & 0x1
		self.lfo_sync_source = ((bytes[11] & 0x80) >> 7) | ((bytes[12] & 0x1F) << 1)
		data = ((bytes[12] & 0xE0) >> 5) | ((bytes[13] & 0x7) << 3)
		if data & 0x20:
			data -= 0x40
		self.lfo_l_phase = data
		data = ((bytes[13] & 0xF8) >> 3) | ((bytes[14] & 0x1) << 5)
		if data & 0x20:
			data -= 0x40
		self.lfo_r_phase = data
		self.l_depth = ((bytes[14] & 0xFE) >> 1) | ((bytes[15] & 0x1) << 7)
		self.r_depth = ((bytes[15] & 0xFE) >> 1) | ((bytes[16] & 0x1) << 7)
		self.l_delay_time = ((bytes[16] & 0xFE) >> 1) | ((bytes[17] & 0x7F) << 7)
		self.r_delay_time = ((bytes[17] & 0x80) >> 7) | (bytes[18] << 1) | ((bytes[19] & 0x1F) << 9)
		data = ((bytes[19] & 0xE0) >> 5) | ((bytes[20] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.l_feedback = data
		data = ((bytes[20] & 0xE0) >> 5) | ((bytes[21] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.r_feedback = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX071 Stereo Modulation Delay object into raw data'''
		bytes = [0] * 22
		bytes[0] = (self.wet_dry << 1) & 0xFE
		bytes[1] = ((self.wet_dry >> 7) & 0x1) | ((self.wet_dry_mod_source & 0x3F) << 1) | ((self.wet_dry_mod_intensity << 7) & 0x80)
		bytes[2] = (self.wet_dry_mod_intensity >> 1) & 0x7F
		bytes[3] = ((self.modulation_mode & 0x1) << 6) | ((self.mod_l_r_polarity & 0x1) << 7)
		bytes[4] = (self.mod_source & 0x3F) | ((self.mod_response << 6) & 0xC0)
		bytes[5] = ((self.mod_response >> 2) & 0x7) | ((self.lfo_waveform & 0x1) << 3) | ((self.lfo_shape << 4) & 0xF0)
		bytes[6] = ((self.lfo_shape >> 4) & 0xF) | ((self.lfo_type & 0x3) << 4) | ((self.common_lfo_phase_offset << 6) & 0xC0)
		bytes[7] = ((self.common_lfo_phase_offset >> 2) & 0xF) | ((self.lfo_frequency << 4) & 0xF0)
		bytes[8] = ((self.lfo_frequency >> 4) & 0xF) | ((self.lfo_bpm_midi_sync & 0x1) << 4) | ((self.lfo_bpm << 5) & 0xE0)
		bytes[9] = (self.lfo_bpm >> 3) & 0xFF
		bytes[10] = ((self.lfo_bpm >> 15) & 0xB) | ((self.lfo_bpm_base_note & 0xF) << 4)
		bytes[11] = (self.lfo_bpm_times & 0x3F) | ((self.lfo_sync & 0x1) << 6) | ((self.lfo_sync_source << 7) & 0x80)
		bytes[12] = ((self.lfo_sync_source >> 1) & 0x1F) | ((self.lfo_l_phase << 5) & 0xE0)
		bytes[13] = ((self.lfo_l_phase >> 3) & 0x7) | ((self.lfo_r_phase << 3) & 0xF8)
		bytes[14] = ((self.lfo_r_phase >> 5) & 0x1) | ((self.l_depth << 1) & 0xFE)
		bytes[15] = ((self.l_depth >> 7) & 0x1) | ((self.r_depth << 1) & 0xFE)
		bytes[16] = ((self.r_depth >> 7) & 0x1) | ((self.l_delay_time << 1) & 0xFE)
		bytes[17] = ((self.l_delay_time >> 7) & 0x7F) | ((self.r_delay_time << 7) & 0x80)
		bytes[18] = (self.r_delay_time >> 1) & 0xFF
		bytes[19] = ((self.r_delay_time >> 31) & 0x9) | ((self.l_feedback << 5) & 0xE0)
		bytes[20] = ((self.l_feedback >> 3) & 0x1F) | ((self.r_feedback << 5) & 0xE0)
		bytes[21] = (self.r_feedback >> 3) & 0x1F
		return struct.pack('<22B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX071 Stereo Modulation Delay object'''
		json = []
		json.append(('name','FX071'))
		json.append(('description','Stereo Modulation Delay'))
		json.append(['modulation_mode',self.modulation_mode])
		json.append(['mod_source',self.mod_source])
		json.append(['mod_response',self.mod_response])
		json.append(['lfo_waveform',self.lfo_waveform])
		json.append(['lfo_shape',self.lfo_shape])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_sync',self.lfo_sync])
		json.append(['lfo_sync_source',self.lfo_sync_source])
		json.append(['lfo_l_phase',self.lfo_l_phase])
		json.append(['lfo_r_phase',self.lfo_r_phase])
		json.append(['l_depth',self.l_depth])
		json.append(['r_depth',self.r_depth])
		json.append(['l_delay_time',self.l_delay_time])
		json.append(['r_delay_time',self.r_delay_time])
		json.append(['l_feedback',self.l_feedback])
		json.append(['r_feedback',self.r_feedback])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['mod_l_r_polarity',self.mod_l_r_polarity])
		json.append(['lfo_type',self.lfo_type])
		json.append(['common_lfo_phase_offset',self.common_lfo_phase_offset])
		json.append(['lfo_bpm_midi_sync',self.lfo_bpm_midi_sync])
		json.append(['lfo_bpm',self.lfo_bpm])
		json.append(['lfo_bpm_display',self.lfo_bpm_display])
		json.append(['lfo_bpm_base_note',self.lfo_bpm_base_note])
		json.append(['lfo_bpm_base_note_display',self.lfo_bpm_base_note_display])
		json.append(['lfo_bpm_times',self.lfo_bpm_times])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if -100 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def modulation_mode(self):
		'''Modulation Mode'''
		return self._modulation_mode

	@modulation_mode.setter
	def modulation_mode(self,value):
		if 0 <= value <= 1:
			self._modulation_mode = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def mod_l_r_polarity(self):
		'''Mod. L/R Polarity'''
		return self._mod_l_r_polarity

	@mod_l_r_polarity.setter
	def mod_l_r_polarity(self,value):
		if 0 <= value <= 1:
			self._mod_l_r_polarity = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def mod_source(self):
		'''Mod.Source'''
		return self._mod_source

	@mod_source.setter
	def mod_source(self,value):
		if 0 <= value <= 37:
			self._mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def mod_response(self):
		'''Mod.Response'''
		return self._mod_response

	@mod_response.setter
	def mod_response(self,value):
		if 0 <= value <= 30:
			self._mod_response = value
		else:
			raise ValueError('%s not 0 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_waveform(self):
		'''LFO Waveform'''
		return self._lfo_waveform

	@lfo_waveform.setter
	def lfo_waveform(self,value):
		if 0 <= value <= 1:
			self._lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_shape(self):
		'''LFO Shape'''
		return self._lfo_shape

	@lfo_shape.setter
	def lfo_shape(self,value):
		if -100 <= value <= 100:
			self._lfo_shape = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_type(self):
		'''LFO Type'''
		return self._lfo_type

	@lfo_type.setter
	def lfo_type(self,value):
		if 0 <= value <= 2:
			self._lfo_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def common_lfo_phase_offset(self):
		'''Common LFO Phase Offset'''
		return self._common_lfo_phase_offset

	@common_lfo_phase_offset.setter
	def common_lfo_phase_offset(self,value):
		if -18 <= value <= 18:
			self._common_lfo_phase_offset = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_midi_sync(self):
		'''LFO BPM/MIDI Sync'''
		return self._lfo_bpm_midi_sync

	@lfo_bpm_midi_sync.setter
	def lfo_bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm(self):
		'''LFO BPM'''
		return self._lfo_bpm

	@lfo_bpm.setter
	def lfo_bpm(self,value):
		if 3999 <= value <= 30000:
			self._lfo_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def lfo_bpm_display(self):
		'''LFO BPM display value'''
		return bpm_decode(self._lfo_bpm)

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_base_note(self):
		'''LFO BPM Base Note'''
		return self._lfo_bpm_base_note

	@lfo_bpm_base_note.setter
	def lfo_bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._lfo_bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def lfo_bpm_base_note_display(self):
		'''LFO BPM Base Note display value'''
		return base_note_dict[self._lfo_bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def lfo_bpm_times(self):
		'''LFO BPM Times'''
		return self._lfo_bpm_times

	@lfo_bpm_times.setter
	def lfo_bpm_times(self,value):
		if 1 <= value <= 32:
			self._lfo_bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_sync(self):
		'''LFO Sync'''
		return self._lfo_sync

	@lfo_sync.setter
	def lfo_sync(self,value):
		if 0 <= value <= 1:
			self._lfo_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_sync_source(self):
		'''LFO Sync Source'''
		return self._lfo_sync_source

	@lfo_sync_source.setter
	def lfo_sync_source(self,value):
		if 0 <= value <= 37:
			self._lfo_sync_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_l_phase(self):
		'''LFO L Phase'''
		return self._lfo_l_phase

	@lfo_l_phase.setter
	def lfo_l_phase(self,value):
		if -18 <= value <= 18:
			self._lfo_l_phase = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_r_phase(self):
		'''LFO R Phase'''
		return self._lfo_r_phase

	@lfo_r_phase.setter
	def lfo_r_phase(self,value):
		if -18 <= value <= 18:
			self._lfo_r_phase = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def l_depth(self):
		'''L Depth'''
		return self._l_depth

	@l_depth.setter
	def l_depth(self,value):
		if 0 <= value <= 100:
			self._l_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def r_depth(self):
		'''R Depth'''
		return self._r_depth

	@r_depth.setter
	def r_depth(self,value):
		if 0 <= value <= 100:
			self._r_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def l_delay_time(self):
		'''L Delay Time'''
		return self._l_delay_time

	@l_delay_time.setter
	def l_delay_time(self,value):
		if 0 <= value <= 10000:
			self._l_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 10000' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_time(self):
		'''R Delay Time'''
		return self._r_delay_time

	@r_delay_time.setter
	def r_delay_time(self,value):
		if 0 <= value <= 10000:
			self._r_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 10000' % value)

#-------------------------------------------------------------------------------
	@property
	def l_feedback(self):
		'''L Feedback'''
		return self._l_feedback

	@l_feedback.setter
	def l_feedback(self,value):
		if -100 <= value <= 100:
			self._l_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def r_feedback(self):
		'''R Feedback'''
		return self._r_feedback

	@r_feedback.setter
	def r_feedback(self,value):
		if -100 <= value <= 100:
			self._r_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

