#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx072.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX072(object):
	category = '''Delay'''
	description = '''Stereo Dynamic Delay'''
	short_description = '''St. Dynamic Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX072 Stereo Dynamic Delay object'''
		bytes = struct.unpack('<15B',raw[:15])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.control_target = (bytes[3] >> 5) & 0x3
		self.polarity = (bytes[3] >> 7) & 0x1
		self.threshold = bytes[4] & 0x7F
		self.offset = ((bytes[4] & 0x80) >> 7) | ((bytes[5] & 0x3F) << 1)
		self.attack = ((bytes[5] & 0xC0) >> 6) | ((bytes[6] & 0x1F) << 2)
		self.release = ((bytes[6] & 0xE0) >> 5) | ((bytes[7] & 0xF) << 3)
		self.l_delay_time = ((bytes[7] & 0xF0) >> 4) | (bytes[8] << 4) | ((bytes[9] & 0x3) << 12)
		self.r_delay_time = ((bytes[9] & 0xFC) >> 2) | ((bytes[10] & 0xFF) << 6)
		data = bytes[11] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.high_damp = bytes[12] & 0x7F
		self.low_damp = ((bytes[12] & 0x80) >> 7) | ((bytes[13] & 0x3F) << 1)
		data = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.spread = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX072 Stereo Dynamic Delay object into raw data'''
		bytes = [0] * 15
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.control_target & 0x3) << 5) | ((self.polarity & 0x1) << 7)
		bytes[4] = (self.threshold & 0x7F) | ((self.offset << 7) & 0x80)
		bytes[5] = ((self.offset >> 1) & 0x3F) | ((self.attack << 6) & 0xC0)
		bytes[6] = ((self.attack >> 2) & 0x1F) | ((self.release << 5) & 0xE0)
		bytes[7] = ((self.release >> 3) & 0xF) | ((self.l_delay_time << 4) & 0xF0)
		bytes[8] = (self.l_delay_time >> 4) & 0xFF
		bytes[9] = ((self.l_delay_time >> 3) & 0xC) | ((self.r_delay_time << 2) & 0xFC)
		bytes[10] = (self.r_delay_time >> 6) & 0xFF
		bytes[11] = self.feedback & 0xFF
		bytes[12] = (self.high_damp & 0x7F) | ((self.low_damp << 7) & 0x80)
		bytes[13] = ((self.low_damp >> 1) & 0x3F) | ((self.spread << 6) & 0xC0)
		bytes[14] = (self.spread >> 2) & 0x3F
		return struct.pack('<15B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX072 Stereo Dynamic Delay object'''
		json = []
		json.append(('name','FX072'))
		json.append(('description','Stereo Dynamic Delay'))
		json.append(['control_target',self.control_target])
		json.append(['polarity',self.polarity])
		json.append(['threshold',self.threshold])
		json.append(['attack',self.attack])
		json.append(['release',self.release])
		json.append(['l_delay_time',self.l_delay_time])
		json.append(['r_delay_time',self.r_delay_time])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['feedback',self.feedback])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['spread',self.spread])
		json.append(['offset',self.offset])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def control_target(self):
		'''Control Target'''
		return self._control_target

	@control_target.setter
	def control_target(self,value):
		if 0 <= value <= 2:
			self._control_target = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def polarity(self):
		'''Polarity'''
		return self._polarity

	@polarity.setter
	def polarity(self,value):
		if 0 <= value <= 1:
			self._polarity = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def threshold(self):
		'''Threshold'''
		return self._threshold

	@threshold.setter
	def threshold(self,value):
		if 0 <= value <= 100:
			self._threshold = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def offset(self):
		'''Offset'''
		return self._offset

	@offset.setter
	def offset(self,value):
		if 0 <= value <= 100:
			self._offset = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def attack(self):
		'''Attack'''
		return self._attack

	@attack.setter
	def attack(self,value):
		if 1 <= value <= 100:
			self._attack = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def release(self):
		'''Release'''
		return self._release

	@release.setter
	def release(self,value):
		if 1 <= value <= 100:
			self._release = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def l_delay_time(self):
		'''L Delay Time'''
		return self._l_delay_time

	@l_delay_time.setter
	def l_delay_time(self,value):
		if 0 <= value <= 2740:
			self._l_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 2740' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_time(self):
		'''R Delay Time'''
		return self._r_delay_time

	@r_delay_time.setter
	def r_delay_time(self,value):
		if 0 <= value <= 2740:
			self._r_delay_time = value
		else:
			raise ValueError('%s not 0 <= value <= 2740' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def spread(self):
		'''Spread'''
		return self._spread

	@spread.setter
	def spread(self,value):
		if -100 <= value <= 100:
			self._spread = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

