#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx077.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

#===============================================================================
class FX077(object):
	category = '''Delay'''
	description = '''Sequence Beats per Minute Delay'''
	short_description = '''Sequence BPM Dly'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX077 Sequence Beats per Minute Delay object'''
		bytes = struct.unpack('<16B',raw[:16])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.bpm = ((bytes[3] & 0xE0) >> 5) | (bytes[4] << 3) | ((bytes[5] & 0xF) << 11)
		self.rhythm_pattern = ((bytes[5] & 0xF0) >> 4) | ((bytes[6] & 0x1) << 4)
		self.tap1_pan = (bytes[6] >> 1) & 0x7F
		self.tap2_pan = bytes[7] & 0x7F
		self.tap3_pan = ((bytes[7] & 0x80) >> 7) | ((bytes[8] & 0x3F) << 1)
		self.tap4_pan = ((bytes[8] & 0xC0) >> 6) | ((bytes[9] & 0x1F) << 2)
		data = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.feedback_mod_source = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x7) << 3)
		data = ((bytes[11] & 0xF8) >> 3) | ((bytes[12] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.feedback_mod_intensity = data
		self.high_damp = ((bytes[12] & 0xF8) >> 3) | ((bytes[13] & 0x3) << 5)
		self.low_damp = ((bytes[13] & 0xFC) >> 2) | ((bytes[14] & 0x1) << 6)
		self.input_trim_mod_source = (bytes[14] >> 1) & 0x3F
		data = ((bytes[14] & 0x80) >> 7) | ((bytes[15] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.input_trim_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX077 Sequence Beats per Minute Delay object into raw data'''
		bytes = [0] * 16
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.bpm << 5) & 0xE0
		bytes[4] = (self.bpm >> 3) & 0xFF
		bytes[5] = ((self.bpm >> 15) & 0xB) | ((self.rhythm_pattern << 4) & 0xF0)
		bytes[6] = ((self.rhythm_pattern >> 4) & 0x1) | ((self.tap1_pan & 0x7F) << 1)
		bytes[7] = (self.tap2_pan & 0x7F) | ((self.tap3_pan << 7) & 0x80)
		bytes[8] = ((self.tap3_pan >> 1) & 0x3F) | ((self.tap4_pan << 6) & 0xC0)
		bytes[9] = ((self.tap4_pan >> 2) & 0x1F) | ((self.feedback << 5) & 0xE0)
		bytes[10] = ((self.feedback >> 3) & 0x1F) | ((self.feedback_mod_source << 5) & 0xE0)
		bytes[11] = ((self.feedback_mod_source >> 3) & 0x7) | ((self.feedback_mod_intensity << 3) & 0xF8)
		bytes[12] = ((self.feedback_mod_intensity >> 5) & 0x7) | ((self.high_damp << 3) & 0xF8)
		bytes[13] = ((self.high_damp >> 5) & 0x3) | ((self.low_damp << 2) & 0xFC)
		bytes[14] = ((self.low_damp >> 6) & 0x1) | ((self.input_trim_mod_source & 0x3F) << 1) | ((self.input_trim_mod_intensity << 7) & 0x80)
		bytes[15] = (self.input_trim_mod_intensity >> 1) & 0x7F
		return struct.pack('<16B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX077 Sequence Beats per Minute Delay object'''
		json = []
		json.append(('name','FX077'))
		json.append(('description','Sequence Beats per Minute Delay'))
		json.append(['bpm',self.bpm])
		json.append(['bpm_display',self.bpm_display])
		json.append(['rhythm_pattern',self.rhythm_pattern])
		json.append(['feedback',self.feedback])
		json.append(['feedback_mod_source',self.feedback_mod_source])
		json.append(['feedback_mod_intensity',self.feedback_mod_intensity])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['input_trim_mod_source',self.input_trim_mod_source])
		json.append(['input_trim_mod_intensity',self.input_trim_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['tap1_pan',self.tap1_pan])
		json.append(['tap2_pan',self.tap2_pan])
		json.append(['tap3_pan',self.tap3_pan])
		json.append(['tap4_pan',self.tap4_pan])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bpm(self):
		'''BPM'''
		return self._bpm

	@bpm.setter
	def bpm(self,value):
		if 3999 <= value <= 30000:
			self._bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def bpm_display(self):
		'''BPM display value'''
		return bpm_decode(self._bpm)

#-------------------------------------------------------------------------------
	@property
	def rhythm_pattern(self):
		'''Rhythm Pattern'''
		return self._rhythm_pattern

	@rhythm_pattern.setter
	def rhythm_pattern(self,value):
		if 0 <= value <= 20:
			self._rhythm_pattern = value
		else:
			raise ValueError('%s not 0 <= value <= 20' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_pan(self):
		'''Tap1 Pan'''
		return self._tap1_pan

	@tap1_pan.setter
	def tap1_pan(self,value):
		if 0 <= value <= 100:
			self._tap1_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_pan(self):
		'''Tap2 Pan'''
		return self._tap2_pan

	@tap2_pan.setter
	def tap2_pan(self,value):
		if 0 <= value <= 100:
			self._tap2_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_pan(self):
		'''Tap3 Pan'''
		return self._tap3_pan

	@tap3_pan.setter
	def tap3_pan(self,value):
		if 0 <= value <= 100:
			self._tap3_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap4_pan(self):
		'''Tap4 Pan'''
		return self._tap4_pan

	@tap4_pan.setter
	def tap4_pan(self,value):
		if 0 <= value <= 100:
			self._tap4_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_source(self):
		'''Feedback Mod.Source'''
		return self._feedback_mod_source

	@feedback_mod_source.setter
	def feedback_mod_source(self,value):
		if 0 <= value <= 37:
			self._feedback_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_intensity(self):
		'''Feedback Mod.Intensity'''
		return self._feedback_mod_intensity

	@feedback_mod_intensity.setter
	def feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_source(self):
		'''Input Trim Mod.Source'''
		return self._input_trim_mod_source

	@input_trim_mod_source.setter
	def input_trim_mod_source(self,value):
		if 0 <= value <= 37:
			self._input_trim_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_intensity(self):
		'''Input Trim Mod.Intensity'''
		return self._input_trim_mod_intensity

	@input_trim_mod_intensity.setter
	def input_trim_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._input_trim_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

