#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx083.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX083(object):
	category = '''Delay'''
	description = '''Tape Echo Beats per Minute'''
	short_description = '''Tape Echo BPM'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX083 Tape Echo Beats per Minute object'''
		bytes = struct.unpack('<27B',raw[:27])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.tone = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.bpm = ((bytes[4] & 0xF0) >> 4) | (bytes[5] << 4) | ((bytes[6] & 0x7) << 12)
		self.delay_time_base_note = (bytes[6] >> 3) & 0xF
		self.delay_time_base_times = ((bytes[6] & 0x80) >> 7) | ((bytes[7] & 0x1F) << 1)
		self.delay_time_mod_source = ((bytes[7] & 0xE0) >> 5) | ((bytes[8] & 0x7) << 3)
		self.delay_time_mod_max_base_note = (bytes[8] >> 3) & 0xF
		self.delay_time_mod_max_base_times = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x1F) << 1)
		self.tap2_position = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0xF) << 3)
		self.tap3_position = ((bytes[10] & 0xF0) >> 4) | ((bytes[11] & 0x7) << 4)
		self.tap1_level = ((bytes[11] & 0xF8) >> 3) | ((bytes[12] & 0x3) << 5)
		self.tap1_pan = ((bytes[12] & 0xFC) >> 2) | ((bytes[13] & 0x1) << 6)
		data = ((bytes[13] & 0xFE) >> 1) | ((bytes[14] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.tap1_feedback = data
		self.tap2_level = (bytes[14] >> 1) & 0x7F
		self.tap2_pan = bytes[15] & 0x7F
		data = ((bytes[15] & 0x80) >> 7) | ((bytes[16] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.tap2_feedback = data
		self.tap3_level = ((bytes[16] & 0x80) >> 7) | ((bytes[17] & 0x3F) << 1)
		self.tap3_pan = ((bytes[17] & 0xC0) >> 6) | ((bytes[18] & 0x1F) << 2)
		data = ((bytes[18] & 0xE0) >> 5) | ((bytes[19] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.tap3_feedback = data
		self.feedback_level = ((bytes[19] & 0xE0) >> 5) | ((bytes[20] & 0xF) << 3)
		self.feedback_level_mod_source = ((bytes[20] & 0xF0) >> 4) | ((bytes[21] & 0x3) << 4)
		data = ((bytes[21] & 0xFC) >> 2) | ((bytes[22] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.feedback_level_mod_intensity = data
		self.high_damp = ((bytes[22] & 0xFC) >> 2) | ((bytes[23] & 0x1) << 6)
		self.low_damp = (bytes[23] >> 1) & 0x7F
		self.saturation = bytes[24] & 0x7F
		self.lfo_frequency = ((bytes[24] & 0x80) >> 7) | ((bytes[25] & 0x3F) << 1)
		self.lfo_depth = ((bytes[25] & 0xC0) >> 6) | ((bytes[26] & 0x1F) << 2)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX083 Tape Echo Beats per Minute object into raw data'''
		bytes = [0] * 27
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.tone << 5) & 0xE0)
		bytes[4] = ((self.tone >> 3) & 0xF) | ((self.bpm << 4) & 0xF0)
		bytes[5] = (self.bpm >> 4) & 0xFF
		bytes[6] = ((self.bpm >> 7) & 0xC) | ((self.delay_time_base_note & 0xF) << 3) | ((self.delay_time_base_times << 7) & 0x80)
		bytes[7] = ((self.delay_time_base_times >> 1) & 0x1F) | ((self.delay_time_mod_source << 5) & 0xE0)
		bytes[8] = ((self.delay_time_mod_source >> 3) & 0x7) | ((self.delay_time_mod_max_base_note & 0xF) << 3) | ((self.delay_time_mod_max_base_times << 7) & 0x80)
		bytes[9] = ((self.delay_time_mod_max_base_times >> 1) & 0x1F) | ((self.tap2_position << 5) & 0xE0)
		bytes[10] = ((self.tap2_position >> 3) & 0xF) | ((self.tap3_position << 4) & 0xF0)
		bytes[11] = ((self.tap3_position >> 4) & 0x7) | ((self.tap1_level << 3) & 0xF8)
		bytes[12] = ((self.tap1_level >> 5) & 0x3) | ((self.tap1_pan << 2) & 0xFC)
		bytes[13] = ((self.tap1_pan >> 6) & 0x1) | ((self.tap1_feedback << 1) & 0xFE)
		bytes[14] = ((self.tap1_feedback >> 7) & 0x1) | ((self.tap2_level & 0x7F) << 1)
		bytes[15] = (self.tap2_pan & 0x7F) | ((self.tap2_feedback << 7) & 0x80)
		bytes[16] = ((self.tap2_feedback >> 1) & 0x7F) | ((self.tap3_level << 7) & 0x80)
		bytes[17] = ((self.tap3_level >> 1) & 0x3F) | ((self.tap3_pan << 6) & 0xC0)
		bytes[18] = ((self.tap3_pan >> 2) & 0x1F) | ((self.tap3_feedback << 5) & 0xE0)
		bytes[19] = ((self.tap3_feedback >> 3) & 0x1F) | ((self.feedback_level << 5) & 0xE0)
		bytes[20] = ((self.feedback_level >> 3) & 0xF) | ((self.feedback_level_mod_source << 4) & 0xF0)
		bytes[21] = ((self.feedback_level_mod_source >> 4) & 0x3) | ((self.feedback_level_mod_intensity << 2) & 0xFC)
		bytes[22] = ((self.feedback_level_mod_intensity >> 6) & 0x3) | ((self.high_damp << 2) & 0xFC)
		bytes[23] = ((self.high_damp >> 6) & 0x1) | ((self.low_damp & 0x7F) << 1)
		bytes[24] = (self.saturation & 0x7F) | ((self.lfo_frequency << 7) & 0x80)
		bytes[25] = ((self.lfo_frequency >> 1) & 0x3F) | ((self.lfo_depth << 6) & 0xC0)
		bytes[26] = (self.lfo_depth >> 2) & 0x1F
		return struct.pack('<27B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX083 Tape Echo Beats per Minute object'''
		json = []
		json.append(('name','FX083'))
		json.append(('description','Tape Echo Beats per Minute'))
		json.append(['tone',self.tone])
		json.append(['bpm',self.bpm])
		json.append(['bpm_display',self.bpm_display])
		json.append(['delay_time_base_note',self.delay_time_base_note])
		json.append(['delay_time_base_note_display',self.delay_time_base_note_display])
		json.append(['delay_time_base_times',self.delay_time_base_times])
		json.append(['delay_time_mod_source',self.delay_time_mod_source])
		json.append(['delay_time_mod_max_base_note',self.delay_time_mod_max_base_note])
		json.append(['delay_time_mod_max_base_note_display',self.delay_time_mod_max_base_note_display])
		json.append(['delay_time_mod_max_base_times',self.delay_time_mod_max_base_times])
		json.append(['input_trim',self.input_trim])
		json.append(['tap2_position',self.tap2_position])
		json.append(['tap3_position',self.tap3_position])
		json.append(['tap1_level',self.tap1_level])
		json.append(['tap1_pan',self.tap1_pan])
		json.append(['tap1_feedback',self.tap1_feedback])
		json.append(['tap2_level',self.tap2_level])
		json.append(['tap2_pan',self.tap2_pan])
		json.append(['tap2_feedback',self.tap2_feedback])
		json.append(['tap3_level',self.tap3_level])
		json.append(['tap3_pan',self.tap3_pan])
		json.append(['tap3_feedback',self.tap3_feedback])
		json.append(['feedback_level',self.feedback_level])
		json.append(['feedback_level_mod_source',self.feedback_level_mod_source])
		json.append(['feedback_level_mod_intensity',self.feedback_level_mod_intensity])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['saturation',self.saturation])
		json.append(['lfo_frequency',self.lfo_frequency])
		json.append(['lfo_depth',self.lfo_depth])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tone(self):
		'''Tone'''
		return self._tone

	@tone.setter
	def tone(self,value):
		if 0 <= value <= 100:
			self._tone = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bpm(self):
		'''BPM'''
		return self._bpm

	@bpm.setter
	def bpm(self,value):
		if 3999 <= value <= 30000:
			self._bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def bpm_display(self):
		'''BPM display value'''
		return bpm_decode(self._bpm)

#-------------------------------------------------------------------------------
	@property
	def delay_time_base_note(self):
		'''Delay Time Base Note'''
		return self._delay_time_base_note

	@delay_time_base_note.setter
	def delay_time_base_note(self,value):
		if 0 <= value <= 9:
			self._delay_time_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_time_base_note_display(self):
		'''Delay Time Base Note display value'''
		return base_note_dict[self._delay_time_base_note]

#-------------------------------------------------------------------------------
	@property
	def delay_time_base_times(self):
		'''Delay Time Base Times'''
		return self._delay_time_base_times

	@delay_time_base_times.setter
	def delay_time_base_times(self,value):
		if 1 <= value <= 16:
			self._delay_time_base_times = value
		else:
			raise ValueError('%s not 1 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time_mod_source(self):
		'''Delay Time Mod.Source'''
		return self._delay_time_mod_source

	@delay_time_mod_source.setter
	def delay_time_mod_source(self,value):
		if 0 <= value <= 37:
			self._delay_time_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def delay_time_mod_max_base_note(self):
		'''Delay Time Mod.Max Base Note'''
		return self._delay_time_mod_max_base_note

	@delay_time_mod_max_base_note.setter
	def delay_time_mod_max_base_note(self,value):
		if 0 <= value <= 9:
			self._delay_time_mod_max_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def delay_time_mod_max_base_note_display(self):
		'''Delay Time Mod.Max Base Note display value'''
		return base_note_dict[self._delay_time_mod_max_base_note]

#-------------------------------------------------------------------------------
	@property
	def delay_time_mod_max_base_times(self):
		'''Delay Time Mod.Max Base Times'''
		return self._delay_time_mod_max_base_times

	@delay_time_mod_max_base_times.setter
	def delay_time_mod_max_base_times(self,value):
		if 1 <= value <= 16:
			self._delay_time_mod_max_base_times = value
		else:
			raise ValueError('%s not 1 <= value <= 16' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_position(self):
		'''Tap2 Position'''
		return self._tap2_position

	@tap2_position.setter
	def tap2_position(self,value):
		if 0 <= value <= 100:
			self._tap2_position = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_position(self):
		'''Tap3 Position'''
		return self._tap3_position

	@tap3_position.setter
	def tap3_position(self,value):
		if 0 <= value <= 100:
			self._tap3_position = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_level(self):
		'''Tap1 Level'''
		return self._tap1_level

	@tap1_level.setter
	def tap1_level(self,value):
		if 0 <= value <= 100:
			self._tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_pan(self):
		'''Tap1 Pan'''
		return self._tap1_pan

	@tap1_pan.setter
	def tap1_pan(self,value):
		if 0 <= value <= 100:
			self._tap1_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap1_feedback(self):
		'''Tap1 Feedback'''
		return self._tap1_feedback

	@tap1_feedback.setter
	def tap1_feedback(self,value):
		if -100 <= value <= 100:
			self._tap1_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_level(self):
		'''Tap2 Level'''
		return self._tap2_level

	@tap2_level.setter
	def tap2_level(self,value):
		if 0 <= value <= 100:
			self._tap2_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_pan(self):
		'''Tap2 Pan'''
		return self._tap2_pan

	@tap2_pan.setter
	def tap2_pan(self,value):
		if 0 <= value <= 100:
			self._tap2_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap2_feedback(self):
		'''Tap2 Feedback'''
		return self._tap2_feedback

	@tap2_feedback.setter
	def tap2_feedback(self,value):
		if -100 <= value <= 100:
			self._tap2_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_level(self):
		'''Tap3 Level'''
		return self._tap3_level

	@tap3_level.setter
	def tap3_level(self,value):
		if 0 <= value <= 100:
			self._tap3_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_pan(self):
		'''Tap3 Pan'''
		return self._tap3_pan

	@tap3_pan.setter
	def tap3_pan(self,value):
		if 0 <= value <= 100:
			self._tap3_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tap3_feedback(self):
		'''Tap3 Feedback'''
		return self._tap3_feedback

	@tap3_feedback.setter
	def tap3_feedback(self,value):
		if -100 <= value <= 100:
			self._tap3_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_level(self):
		'''Feedback Level'''
		return self._feedback_level

	@feedback_level.setter
	def feedback_level(self,value):
		if 0 <= value <= 100:
			self._feedback_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_level_mod_source(self):
		'''Feedback Level Mod.Source'''
		return self._feedback_level_mod_source

	@feedback_level_mod_source.setter
	def feedback_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._feedback_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_level_mod_intensity(self):
		'''Feedback Level Mod.Intensity'''
		return self._feedback_level_mod_intensity

	@feedback_level_mod_intensity.setter
	def feedback_level_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._feedback_level_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def saturation(self):
		'''Saturation'''
		return self._saturation

	@saturation.setter
	def saturation(self,value):
		if 0 <= value <= 100:
			self._saturation = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_frequency(self):
		'''LFO Frequency'''
		return self._lfo_frequency

	@lfo_frequency.setter
	def lfo_frequency(self,value):
		if 2 <= value <= 100:
			self._lfo_frequency = value
		else:
			raise ValueError('%s not 2 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def lfo_depth(self):
		'''LFO Depth'''
		return self._lfo_depth

	@lfo_depth.setter
	def lfo_depth(self,value):
		if 0 <= value <= 100:
			self._lfo_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

