#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx086.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX086(object):
	category = '''Reverb/ER'''
	description = '''Reverb Wet Plate'''
	short_description = '''Reverb Wet Plate'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX086 Reverb Wet Plate object'''
		bytes = struct.unpack('<9B',raw[:9])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.input_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.reverb_time = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.high_damp = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.pre_delay = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x7) << 5)
		self.pre_delay_thru = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		data = (bytes[7] >> 2) & 0x3F
		if data & 0x20:
			data -= 0x40
		self.pre_leq_gain = data
		data = bytes[8] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.pre_heq_gain = data
		self.pre_leq_fc = (bytes[8] >> 6) & 0x1
		self.pre_heq_fc = (bytes[8] >> 7) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX086 Reverb Wet Plate object into raw data'''
		bytes = [0] * 9
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.input_trim << 6) & 0xC0)
		bytes[3] = ((self.input_trim >> 2) & 0x1F) | ((self.reverb_time << 5) & 0xE0)
		bytes[4] = ((self.reverb_time >> 3) & 0xF) | ((self.high_damp << 4) & 0xF0)
		bytes[5] = ((self.high_damp >> 4) & 0x7) | ((self.pre_delay << 3) & 0xF8)
		bytes[6] = ((self.pre_delay >> 5) & 0x7) | ((self.pre_delay_thru << 3) & 0xF8)
		bytes[7] = ((self.pre_delay_thru >> 5) & 0x3) | ((self.pre_leq_gain & 0x3F) << 2)
		bytes[8] = (self.pre_heq_gain & 0x3F) | ((self.pre_leq_fc & 0x1) << 6) | ((self.pre_heq_fc & 0x1) << 7)
		return struct.pack('<9B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX086 Reverb Wet Plate object'''
		json = []
		json.append(('name','FX086'))
		json.append(('description','Reverb Wet Plate'))
		json.append(['reverb_time',self.reverb_time])
		json.append(['high_damp',self.high_damp])
		json.append(['pre_delay',self.pre_delay])
		json.append(['pre_delay_thru',self.pre_delay_thru])
		json.append(['input_trim',self.input_trim])
		json.append(['pre_leq_gain',self.pre_leq_gain])
		json.append(['pre_heq_gain',self.pre_heq_gain])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['pre_leq_fc',self.pre_leq_fc])
		json.append(['pre_heq_fc',self.pre_heq_fc])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim(self):
		'''Input Trim'''
		return self._input_trim

	@input_trim.setter
	def input_trim(self,value):
		if 0 <= value <= 100:
			self._input_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def reverb_time(self):
		'''Reverb Time'''
		return self._reverb_time

	@reverb_time.setter
	def reverb_time(self,value):
		if 1 <= value <= 100:
			self._reverb_time = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_delay(self):
		'''Pre Delay'''
		return self._pre_delay

	@pre_delay.setter
	def pre_delay(self,value):
		if 0 <= value <= 200:
			self._pre_delay = value
		else:
			raise ValueError('%s not 0 <= value <= 200' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_delay_thru(self):
		'''Pre Delay Thru'''
		return self._pre_delay_thru

	@pre_delay_thru.setter
	def pre_delay_thru(self,value):
		if 0 <= value <= 100:
			self._pre_delay_thru = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_leq_gain(self):
		'''Pre LEQ Gain'''
		return self._pre_leq_gain

	@pre_leq_gain.setter
	def pre_leq_gain(self,value):
		if -30 <= value <= 30:
			self._pre_leq_gain = value
		else:
			raise ValueError('%s not -30 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_heq_gain(self):
		'''Pre HEQ Gain'''
		return self._pre_heq_gain

	@pre_heq_gain.setter
	def pre_heq_gain(self,value):
		if -30 <= value <= 30:
			self._pre_heq_gain = value
		else:
			raise ValueError('%s not -30 <= value <= 30' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_leq_fc(self):
		'''Pre LEQ Fc'''
		return self._pre_leq_fc

	@pre_leq_fc.setter
	def pre_leq_fc(self,value):
		if 0 <= value <= 1:
			self._pre_leq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def pre_heq_fc(self):
		'''Pre HEQ Fc'''
		return self._pre_heq_fc

	@pre_heq_fc.setter
	def pre_heq_fc(self,value):
		if 0 <= value <= 1:
			self._pre_heq_fc = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

