#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx121.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX121(object):
	category = '''Mono-Mono'''
	description = '''Decimator - Amp Simulation'''
	short_description = '''Decimator - Amp'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX121 Decimator - Amp Simulation object'''
		bytes = struct.unpack('<8B',raw[:8])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.decimator_pre_lpf = (bytes[3] >> 5) & 0x1
		self.decimator_sampling_frequency = ((bytes[3] & 0xC0) >> 6) | ((bytes[4] & 0x7F) << 2)
		self.decimator_high_damp = ((bytes[4] & 0x80) >> 7) | ((bytes[5] & 0x3F) << 1)
		self.decimator_resolution = ((bytes[5] & 0xC0) >> 6) | ((bytes[6] & 0x7) << 2)
		self.decimator_output_level = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x3) << 5)
		self.amp_simulation_amplifier_type = (bytes[7] >> 2) & 0x3
		self.routing = (bytes[7] >> 4) & 0x1

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX121 Decimator - Amp Simulation object into raw data'''
		bytes = [0] * 8
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.decimator_pre_lpf & 0x1) << 5) | ((self.decimator_sampling_frequency << 6) & 0xC0)
		bytes[4] = ((self.decimator_sampling_frequency >> 2) & 0x7F) | ((self.decimator_high_damp << 7) & 0x80)
		bytes[5] = ((self.decimator_high_damp >> 1) & 0x3F) | ((self.decimator_resolution << 6) & 0xC0)
		bytes[6] = ((self.decimator_resolution >> 2) & 0x7) | ((self.decimator_output_level << 3) & 0xF8)
		bytes[7] = ((self.decimator_output_level >> 5) & 0x3) | ((self.amp_simulation_amplifier_type & 0x3) << 2) | ((self.routing & 0x1) << 4)
		return struct.pack('<8B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX121 Decimator - Amp Simulation object'''
		json = []
		json.append(('name','FX121'))
		json.append(('description','Decimator - Amp Simulation'))
		json.append(['decimator_pre_lpf',self.decimator_pre_lpf])
		json.append(['decimator_sampling_frequency',self.decimator_sampling_frequency])
		json.append(['decimator_high_damp',self.decimator_high_damp])
		json.append(['decimator_resolution',self.decimator_resolution])
		json.append(['decimator_output_level',self.decimator_output_level])
		json.append(['amp_simulation_amplifier_type',self.amp_simulation_amplifier_type])
		json.append(['routing',self.routing])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def decimator_pre_lpf(self):
		'''(Decimator) Pre LPF'''
		return self._decimator_pre_lpf

	@decimator_pre_lpf.setter
	def decimator_pre_lpf(self,value):
		if 0 <= value <= 1:
			self._decimator_pre_lpf = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def decimator_sampling_frequency(self):
		'''(Decimator) Sampling Frequency'''
		return self._decimator_sampling_frequency

	@decimator_sampling_frequency.setter
	def decimator_sampling_frequency(self,value):
		if 10 <= value <= 480:
			self._decimator_sampling_frequency = value
		else:
			raise ValueError('%s not 10 <= value <= 480' % value)

#-------------------------------------------------------------------------------
	@property
	def decimator_high_damp(self):
		'''(Decimator) High Damp'''
		return self._decimator_high_damp

	@decimator_high_damp.setter
	def decimator_high_damp(self,value):
		if 0 <= value <= 100:
			self._decimator_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def decimator_resolution(self):
		'''(Decimator) Resolution'''
		return self._decimator_resolution

	@decimator_resolution.setter
	def decimator_resolution(self,value):
		if 4 <= value <= 24:
			self._decimator_resolution = value
		else:
			raise ValueError('%s not 4 <= value <= 24' % value)

#-------------------------------------------------------------------------------
	@property
	def decimator_output_level(self):
		'''(Decimator) Output Level'''
		return self._decimator_output_level

	@decimator_output_level.setter
	def decimator_output_level(self,value):
		if 0 <= value <= 100:
			self._decimator_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def amp_simulation_amplifier_type(self):
		'''(Amp Simulation) Amplifier Type'''
		return self._amp_simulation_amplifier_type

	@amp_simulation_amplifier_type.setter
	def amp_simulation_amplifier_type(self,value):
		if 0 <= value <= 2:
			self._amp_simulation_amplifier_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def routing(self):
		'''Routing'''
		return self._routing

	@routing.setter
	def routing(self,value):
		if 0 <= value <= 1:
			self._routing = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

