#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx123.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX123(object):
	category = '''Mono-Mono'''
	description = '''Amp Simulation - Tremolo'''
	short_description = '''AmpSim - Tremolo'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX123 Amp Simulation - Tremolo object'''
		bytes = struct.unpack('<8B',raw[:8])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.amp_simulation_amplifier_type = (bytes[3] >> 5) & 0x3
		self.tremolo_lfo_waveform = ((bytes[3] & 0x80) >> 7) | ((bytes[4] & 0x3) << 1)
		data = ((bytes[4] & 0xFC) >> 2) | ((bytes[5] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.tremolo_lfo_shape = data
		self.tremolo_lfo_frequency = ((bytes[5] & 0xFC) >> 2) | ((bytes[6] & 0x3) << 6)
		self.depth = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x1) << 6)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX123 Amp Simulation - Tremolo object into raw data'''
		bytes = [0] * 8
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.amp_simulation_amplifier_type & 0x3) << 5) | ((self.tremolo_lfo_waveform << 7) & 0x80)
		bytes[4] = ((self.tremolo_lfo_waveform >> 1) & 0x3) | ((self.tremolo_lfo_shape << 2) & 0xFC)
		bytes[5] = ((self.tremolo_lfo_shape >> 6) & 0x3) | ((self.tremolo_lfo_frequency << 2) & 0xFC)
		bytes[6] = ((self.tremolo_lfo_frequency >> 6) & 0x3) | ((self.depth << 2) & 0xFC)
		bytes[7] = (self.depth >> 6) & 0x1
		return struct.pack('<8B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX123 Amp Simulation - Tremolo object'''
		json = []
		json.append(('name','FX123'))
		json.append(('description','Amp Simulation - Tremolo'))
		json.append(['amp_simulation_amplifier_type',self.amp_simulation_amplifier_type])
		json.append(['tremolo_lfo_waveform',self.tremolo_lfo_waveform])
		json.append(['tremolo_lfo_shape',self.tremolo_lfo_shape])
		json.append(['tremolo_lfo_frequency',self.tremolo_lfo_frequency])
		json.append(['depth',self.depth])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def amp_simulation_amplifier_type(self):
		'''(Amp Simulation) Amplifier Type'''
		return self._amp_simulation_amplifier_type

	@amp_simulation_amplifier_type.setter
	def amp_simulation_amplifier_type(self,value):
		if 0 <= value <= 2:
			self._amp_simulation_amplifier_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def tremolo_lfo_waveform(self):
		'''(Tremolo) LFO Waveform'''
		return self._tremolo_lfo_waveform

	@tremolo_lfo_waveform.setter
	def tremolo_lfo_waveform(self,value):
		if 0 <= value <= 4:
			self._tremolo_lfo_waveform = value
		else:
			raise ValueError('%s not 0 <= value <= 4' % value)

#-------------------------------------------------------------------------------
	@property
	def tremolo_lfo_shape(self):
		'''(Tremolo) LFO Shape'''
		return self._tremolo_lfo_shape

	@tremolo_lfo_shape.setter
	def tremolo_lfo_shape(self,value):
		if -100 <= value <= 100:
			self._tremolo_lfo_shape = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def tremolo_lfo_frequency(self):
		'''(Tremolo) LFO Frequency'''
		return self._tremolo_lfo_frequency

	@tremolo_lfo_frequency.setter
	def tremolo_lfo_frequency(self,value):
		if 1 <= value <= 230:
			self._tremolo_lfo_frequency = value
		else:
			raise ValueError('%s not 1 <= value <= 230' % value)

#-------------------------------------------------------------------------------
	@property
	def depth(self):
		'''Depth'''
		return self._depth

	@depth.setter
	def depth(self,value):
		if 0 <= value <= 100:
			self._depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

