#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx161.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX161(object):
	category = '''Mono//Mono'''
	description = '''Overdrive/Hi.Gain // Multitap BPM Delay'''
	short_description = '''OD/HG // BPMDly'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX161 Overdrive/Hi.Gain // Multitap BPM Delay object'''
		bytes = struct.unpack('<25B',raw[:25])
		self.od_hi_gain_wet_dry = (bytes[0] >> 1) & 0x7F
		self.od_hi_gain_wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.od_hi_gain_wet_dry_mod_intensity = data
		self.od_hi_gain_pan = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.multitap_bpm_delay_wet_dry = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.multitap_bpm_delay_wet_dry_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		data = ((bytes[6] & 0xFE) >> 1) | ((bytes[7] & 0x1) << 7)
		if data & 0x80:
			data -= 0x100
		self.multitap_bpm_delay_wet_dry_mod_intensity = data
		self.multitap_bpm_delay_pan = bytes[8] & 0x7F
		self.od_hi_gain_hi_gain_on = (bytes[8] >> 7) & 0x1
		self.od_hi_gain_drive = bytes[9] & 0x7F
		self.od_hi_gain_output_level = ((bytes[9] & 0x80) >> 7) | ((bytes[10] & 0x1F) << 1)
		self.od_hi_gain_output_level_mod_source = ((bytes[10] & 0xE0) >> 5) | ((bytes[11] & 0x7) << 3)
		data = ((bytes[11] & 0xF8) >> 3) | ((bytes[12] & 0x3) << 5)
		if data & 0x40:
			data -= 0x80
		self.od_hi_gain_output_level_mod_intensity = data
		self.od_hi_gain_eq_low_cutoff = (bytes[12] >> 2) & 0x3F
		data = bytes[13] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_low_gain = data
		self.od_hi_gain_eq_mid1_cutoff = ((bytes[13] & 0xC0) >> 6) | ((bytes[14] & 0xF) << 2)
		self.od_hi_gain_eq_mid1_q = (bytes[14] >> 4) & 0xF
		data = bytes[15] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid1_gain = data
		self.od_hi_gain_eq_mid2_cutoff = ((bytes[15] & 0xC0) >> 6) | ((bytes[16] & 0xF) << 2)
		self.od_hi_gain_eq_mid2_q = (bytes[16] >> 4) & 0xF
		data = bytes[17] & 0x3F
		if data & 0x20:
			data -= 0x40
		self.od_hi_gain_eq_mid2_gain = data
		self.multitap_bpm_delay_bpm = ((bytes[17] & 0xC0) >> 6) | (bytes[18] << 2) | ((bytes[19] & 0x1F) << 10)
		self.multitap_bpm_delay_tap1_base_note = ((bytes[19] & 0xE0) >> 5) | ((bytes[20] & 0x1) << 3)
		self.multitap_bpm_delay_tap1_times = (bytes[20] >> 1) & 0x3F
		self.multitap_bpm_delay_tap2_base_note = ((bytes[20] & 0x80) >> 7) | ((bytes[21] & 0x7) << 1)
		self.multitap_bpm_delay_tap2_times = ((bytes[21] & 0xF8) >> 3) | ((bytes[22] & 0x1) << 5)
		self.multitap_bpm_delay_tap1_level = (bytes[22] >> 1) & 0x7F
		data = bytes[23] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.multitap_bpm_delay_tap2_feedback = data
		self.multitap_bpm_delay_high_damp = bytes[24] & 0x7F

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX161 Overdrive/Hi.Gain // Multitap BPM Delay object into raw data'''
		bytes = [0] * 25
		bytes[0] = (self.od_hi_gain_wet_dry & 0x7F) << 1
		bytes[1] = (self.od_hi_gain_wet_dry_mod_source & 0x3F) | ((self.od_hi_gain_wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.od_hi_gain_wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.od_hi_gain_pan << 5) & 0xE0
		bytes[4] = ((self.od_hi_gain_pan >> 3) & 0xF) | ((self.multitap_bpm_delay_wet_dry << 4) & 0xF0)
		bytes[5] = ((self.multitap_bpm_delay_wet_dry >> 4) & 0x7) | ((self.multitap_bpm_delay_wet_dry_mod_source << 3) & 0xF8)
		bytes[6] = ((self.multitap_bpm_delay_wet_dry_mod_source >> 5) & 0x1) | ((self.multitap_bpm_delay_wet_dry_mod_intensity << 1) & 0xFE)
		bytes[7] = (self.multitap_bpm_delay_wet_dry_mod_intensity >> 7) & 0x1
		bytes[8] = (self.multitap_bpm_delay_pan & 0x7F) | ((self.od_hi_gain_hi_gain_on & 0x1) << 7)
		bytes[9] = (self.od_hi_gain_drive & 0x7F) | ((self.od_hi_gain_output_level << 7) & 0x80)
		bytes[10] = ((self.od_hi_gain_output_level >> 1) & 0x1F) | ((self.od_hi_gain_output_level_mod_source << 5) & 0xE0)
		bytes[11] = ((self.od_hi_gain_output_level_mod_source >> 3) & 0x7) | ((self.od_hi_gain_output_level_mod_intensity << 3) & 0xF8)
		bytes[12] = ((self.od_hi_gain_output_level_mod_intensity >> 5) & 0x3) | ((self.od_hi_gain_eq_low_cutoff & 0x3F) << 2)
		bytes[13] = (self.od_hi_gain_eq_low_gain & 0x3F) | ((self.od_hi_gain_eq_mid1_cutoff << 6) & 0xC0)
		bytes[14] = ((self.od_hi_gain_eq_mid1_cutoff >> 2) & 0xF) | ((self.od_hi_gain_eq_mid1_q & 0xF) << 4)
		bytes[15] = (self.od_hi_gain_eq_mid1_gain & 0x3F) | ((self.od_hi_gain_eq_mid2_cutoff << 6) & 0xC0)
		bytes[16] = ((self.od_hi_gain_eq_mid2_cutoff >> 2) & 0xF) | ((self.od_hi_gain_eq_mid2_q & 0xF) << 4)
		bytes[17] = (self.od_hi_gain_eq_mid2_gain & 0x3F) | ((self.multitap_bpm_delay_bpm << 6) & 0xC0)
		bytes[18] = (self.multitap_bpm_delay_bpm >> 2) & 0xFF
		bytes[19] = ((self.multitap_bpm_delay_bpm >> 31) & 0xA) | ((self.multitap_bpm_delay_tap1_base_note << 5) & 0xE0)
		bytes[20] = ((self.multitap_bpm_delay_tap1_base_note >> 3) & 0x1) | ((self.multitap_bpm_delay_tap1_times & 0x3F) << 1) | ((self.multitap_bpm_delay_tap2_base_note << 7) & 0x80)
		bytes[21] = ((self.multitap_bpm_delay_tap2_base_note >> 1) & 0x7) | ((self.multitap_bpm_delay_tap2_times << 3) & 0xF8)
		bytes[22] = ((self.multitap_bpm_delay_tap2_times >> 5) & 0x1) | ((self.multitap_bpm_delay_tap1_level & 0x7F) << 1)
		bytes[23] = self.multitap_bpm_delay_tap2_feedback & 0xFF
		bytes[24] = self.multitap_bpm_delay_high_damp & 0x7F
		return struct.pack('<25B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX161 Overdrive/Hi.Gain // Multitap BPM Delay object'''
		json = []
		json.append(('name','FX161'))
		json.append(('description','Overdrive/Hi.Gain // Multitap BPM Delay'))
		json.append(['od_hi_gain_wet_dry',self.od_hi_gain_wet_dry])
		json.append(['od_hi_gain_wet_dry_mod_source',self.od_hi_gain_wet_dry_mod_source])
		json.append(['od_hi_gain_wet_dry_mod_intensity',self.od_hi_gain_wet_dry_mod_intensity])
		json.append(['od_hi_gain_pan',self.od_hi_gain_pan])
		json.append(['multitap_bpm_delay_wet_dry',self.multitap_bpm_delay_wet_dry])
		json.append(['multitap_bpm_delay_wet_dry_mod_source',self.multitap_bpm_delay_wet_dry_mod_source])
		json.append(['multitap_bpm_delay_wet_dry_mod_intensity',self.multitap_bpm_delay_wet_dry_mod_intensity])
		json.append(['multitap_bpm_delay_pan',self.multitap_bpm_delay_pan])
		json.append(['od_hi_gain_hi_gain_on',self.od_hi_gain_hi_gain_on])
		json.append(['od_hi_gain_drive',self.od_hi_gain_drive])
		json.append(['od_hi_gain_output_level',self.od_hi_gain_output_level])
		json.append(['od_hi_gain_output_level_mod_source',self.od_hi_gain_output_level_mod_source])
		json.append(['od_hi_gain_output_level_mod_intensity',self.od_hi_gain_output_level_mod_intensity])
		json.append(['od_hi_gain_eq_low_cutoff',self.od_hi_gain_eq_low_cutoff])
		json.append(['od_hi_gain_eq_low_gain',self.od_hi_gain_eq_low_gain])
		json.append(['od_hi_gain_eq_mid1_cutoff',self.od_hi_gain_eq_mid1_cutoff])
		json.append(['od_hi_gain_eq_mid1_q',self.od_hi_gain_eq_mid1_q])
		json.append(['od_hi_gain_eq_mid1_gain',self.od_hi_gain_eq_mid1_gain])
		json.append(['od_hi_gain_eq_mid2_cutoff',self.od_hi_gain_eq_mid2_cutoff])
		json.append(['od_hi_gain_eq_mid2_q',self.od_hi_gain_eq_mid2_q])
		json.append(['od_hi_gain_eq_mid2_gain',self.od_hi_gain_eq_mid2_gain])
		json.append(['multitap_bpm_delay_bpm',self.multitap_bpm_delay_bpm])
		json.append(['multitap_bpm_delay_bpm_display',self.multitap_bpm_delay_bpm_display])
		json.append(['multitap_bpm_delay_tap1_base_note',self.multitap_bpm_delay_tap1_base_note])
		json.append(['multitap_bpm_delay_tap1_base_note_display',self.multitap_bpm_delay_tap1_base_note_display])
		json.append(['multitap_bpm_delay_tap1_times',self.multitap_bpm_delay_tap1_times])
		json.append(['multitap_bpm_delay_tap2_base_note',self.multitap_bpm_delay_tap2_base_note])
		json.append(['multitap_bpm_delay_tap2_base_note_display',self.multitap_bpm_delay_tap2_base_note_display])
		json.append(['multitap_bpm_delay_tap2_times',self.multitap_bpm_delay_tap2_times])
		json.append(['multitap_bpm_delay_tap1_level',self.multitap_bpm_delay_tap1_level])
		json.append(['multitap_bpm_delay_tap2_feedback',self.multitap_bpm_delay_tap2_feedback])
		json.append(['multitap_bpm_delay_high_damp',self.multitap_bpm_delay_high_damp])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry(self):
		'''(OD/Hi-Gain) Wet/Dry'''
		return self._od_hi_gain_wet_dry

	@od_hi_gain_wet_dry.setter
	def od_hi_gain_wet_dry(self,value):
		if 0 <= value <= 100:
			self._od_hi_gain_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_source(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Source'''
		return self._od_hi_gain_wet_dry_mod_source

	@od_hi_gain_wet_dry_mod_source.setter
	def od_hi_gain_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_wet_dry_mod_intensity(self):
		'''(OD/Hi-Gain) Wet/Dry Mod.Intensity'''
		return self._od_hi_gain_wet_dry_mod_intensity

	@od_hi_gain_wet_dry_mod_intensity.setter
	def od_hi_gain_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._od_hi_gain_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_pan(self):
		'''(OD/Hi-Gain) Pan'''
		return self._od_hi_gain_pan

	@od_hi_gain_pan.setter
	def od_hi_gain_pan(self,value):
		if 0 <= value <= 127:
			self._od_hi_gain_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry(self):
		'''(Multitap BPM Delay) Wet/Dry'''
		return self._multitap_bpm_delay_wet_dry

	@multitap_bpm_delay_wet_dry.setter
	def multitap_bpm_delay_wet_dry(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry_mod_source(self):
		'''(Multitap BPM Delay) Wet/Dry Mod.Source'''
		return self._multitap_bpm_delay_wet_dry_mod_source

	@multitap_bpm_delay_wet_dry_mod_source.setter
	def multitap_bpm_delay_wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._multitap_bpm_delay_wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_wet_dry_mod_intensity(self):
		'''(Multitap BPM Delay) Wet/Dry Mod.Intensity'''
		return self._multitap_bpm_delay_wet_dry_mod_intensity

	@multitap_bpm_delay_wet_dry_mod_intensity.setter
	def multitap_bpm_delay_wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._multitap_bpm_delay_wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_pan(self):
		'''(Multitap BPM Delay) Pan'''
		return self._multitap_bpm_delay_pan

	@multitap_bpm_delay_pan.setter
	def multitap_bpm_delay_pan(self,value):
		if 0 <= value <= 127:
			self._multitap_bpm_delay_pan = value
		else:
			raise ValueError('%s not 0 <= value <= 127' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_hi_gain_on(self):
		'''(OD/Hi-Gain) Hi-Gain On'''
		return self._od_hi_gain_hi_gain_on

	@od_hi_gain_hi_gain_on.setter
	def od_hi_gain_hi_gain_on(self,value):
		if 0 <= value <= 1:
			self._od_hi_gain_hi_gain_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_drive(self):
		'''(OD/Hi-Gain) Drive'''
		return self._od_hi_gain_drive

	@od_hi_gain_drive.setter
	def od_hi_gain_drive(self,value):
		if 1 <= value <= 100:
			self._od_hi_gain_drive = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level(self):
		'''(OD/Hi-Gain) Output Level'''
		return self._od_hi_gain_output_level

	@od_hi_gain_output_level.setter
	def od_hi_gain_output_level(self,value):
		if 0 <= value <= 50:
			self._od_hi_gain_output_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_source(self):
		'''(OD/Hi-Gain) Output Level Mod.Source'''
		return self._od_hi_gain_output_level_mod_source

	@od_hi_gain_output_level_mod_source.setter
	def od_hi_gain_output_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._od_hi_gain_output_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_output_level_mod_intensity(self):
		'''(OD/Hi-Gain) Output Level Mod.Intensity'''
		return self._od_hi_gain_output_level_mod_intensity

	@od_hi_gain_output_level_mod_intensity.setter
	def od_hi_gain_output_level_mod_intensity(self,value):
		if -50 <= value <= 50:
			self._od_hi_gain_output_level_mod_intensity = value
		else:
			raise ValueError('%s not -50 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_cutoff(self):
		'''(OD/Hi-Gain) EQ Low Cutoff'''
		return self._od_hi_gain_eq_low_cutoff

	@od_hi_gain_eq_low_cutoff.setter
	def od_hi_gain_eq_low_cutoff(self,value):
		if 0 <= value <= 49:
			self._od_hi_gain_eq_low_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 49' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_low_gain(self):
		'''(OD/Hi-Gain) EQ Low Gain'''
		return self._od_hi_gain_eq_low_gain

	@od_hi_gain_eq_low_gain.setter
	def od_hi_gain_eq_low_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_low_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid1 Cutoff'''
		return self._od_hi_gain_eq_mid1_cutoff

	@od_hi_gain_eq_mid1_cutoff.setter
	def od_hi_gain_eq_mid1_cutoff(self,value):
		if 1 <= value <= 63:
			self._od_hi_gain_eq_mid1_cutoff = value
		else:
			raise ValueError('%s not 1 <= value <= 63' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_q(self):
		'''(OD/Hi-Gain) EQ Mid1 Q'''
		return self._od_hi_gain_eq_mid1_q

	@od_hi_gain_eq_mid1_q.setter
	def od_hi_gain_eq_mid1_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid1_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid1_gain(self):
		'''(OD/Hi-Gain) EQ Mid1 Gain'''
		return self._od_hi_gain_eq_mid1_gain

	@od_hi_gain_eq_mid1_gain.setter
	def od_hi_gain_eq_mid1_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid1_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_cutoff(self):
		'''(OD/Hi-Gain) EQ Mid2 Cutoff'''
		return self._od_hi_gain_eq_mid2_cutoff

	@od_hi_gain_eq_mid2_cutoff.setter
	def od_hi_gain_eq_mid2_cutoff(self,value):
		if 0 <= value <= 55:
			self._od_hi_gain_eq_mid2_cutoff = value
		else:
			raise ValueError('%s not 0 <= value <= 55' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_q(self):
		'''(OD/Hi-Gain) EQ Mid2 Q'''
		return self._od_hi_gain_eq_mid2_q

	@od_hi_gain_eq_mid2_q.setter
	def od_hi_gain_eq_mid2_q(self,value):
		if 0 <= value <= 15:
			self._od_hi_gain_eq_mid2_q = value
		else:
			raise ValueError('%s not 0 <= value <= 15' % value)

#-------------------------------------------------------------------------------
	@property
	def od_hi_gain_eq_mid2_gain(self):
		'''(OD/Hi-Gain) EQ Mid2 Gain'''
		return self._od_hi_gain_eq_mid2_gain

	@od_hi_gain_eq_mid2_gain.setter
	def od_hi_gain_eq_mid2_gain(self,value):
		if -18 <= value <= 18:
			self._od_hi_gain_eq_mid2_gain = value
		else:
			raise ValueError('%s not -18 <= value <= 18' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_bpm(self):
		'''(Multitap BPM Delay) BPM'''
		return self._multitap_bpm_delay_bpm

	@multitap_bpm_delay_bpm.setter
	def multitap_bpm_delay_bpm(self,value):
		if 3999 <= value <= 30000:
			self._multitap_bpm_delay_bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def multitap_bpm_delay_bpm_display(self):
		'''(Multitap BPM Delay) BPM display value'''
		return bpm_decode(self._multitap_bpm_delay_bpm)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_base_note(self):
		'''(Multitap BPM Delay) Tap1 Base Note'''
		return self._multitap_bpm_delay_tap1_base_note

	@multitap_bpm_delay_tap1_base_note.setter
	def multitap_bpm_delay_tap1_base_note(self,value):
		if 0 <= value <= 9:
			self._multitap_bpm_delay_tap1_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def multitap_bpm_delay_tap1_base_note_display(self):
		'''(Multitap BPM Delay) Tap1 Base Note display value'''
		return base_note_dict[self._multitap_bpm_delay_tap1_base_note]

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_times(self):
		'''(Multitap BPM Delay) Tap1 Times'''
		return self._multitap_bpm_delay_tap1_times

	@multitap_bpm_delay_tap1_times.setter
	def multitap_bpm_delay_tap1_times(self,value):
		if 1 <= value <= 32:
			self._multitap_bpm_delay_tap1_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_base_note(self):
		'''(Multitap BPM Delay) Tap2 Base Note'''
		return self._multitap_bpm_delay_tap2_base_note

	@multitap_bpm_delay_tap2_base_note.setter
	def multitap_bpm_delay_tap2_base_note(self,value):
		if 0 <= value <= 9:
			self._multitap_bpm_delay_tap2_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def multitap_bpm_delay_tap2_base_note_display(self):
		'''(Multitap BPM Delay) Tap2 Base Note display value'''
		return base_note_dict[self._multitap_bpm_delay_tap2_base_note]

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_times(self):
		'''(Multitap BPM Delay) Tap2 Times'''
		return self._multitap_bpm_delay_tap2_times

	@multitap_bpm_delay_tap2_times.setter
	def multitap_bpm_delay_tap2_times(self,value):
		if 1 <= value <= 32:
			self._multitap_bpm_delay_tap2_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap1_level(self):
		'''(Multitap BPM Delay) Tap1 Level'''
		return self._multitap_bpm_delay_tap1_level

	@multitap_bpm_delay_tap1_level.setter
	def multitap_bpm_delay_tap1_level(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_tap1_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_tap2_feedback(self):
		'''(Multitap BPM Delay) Tap2 Feedback'''
		return self._multitap_bpm_delay_tap2_feedback

	@multitap_bpm_delay_tap2_feedback.setter
	def multitap_bpm_delay_tap2_feedback(self,value):
		if -100 <= value <= 100:
			self._multitap_bpm_delay_tap2_feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def multitap_bpm_delay_high_damp(self):
		'''(Multitap BPM Delay) High Damp'''
		return self._multitap_bpm_delay_high_damp

	@multitap_bpm_delay_high_damp.setter
	def multitap_bpm_delay_high_damp(self,value):
		if 0 <= value <= 100:
			self._multitap_bpm_delay_high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

