#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx173.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX173(object):
	category = '''Double Size'''
	description = '''PianoBody/Damper Simulation'''
	short_description = '''PianoBody/Damper'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX173 PianoBody/Damper Simulation object'''
		bytes = struct.unpack('<9B',raw[:9])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.sound_board_depth = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.damper_depth = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.damper_mod_source = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x1) << 5)
		self.tone = (bytes[6] >> 1) & 0x7F
		self.mid_shape = bytes[7] & 0x3F
		data = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0x3) << 2)
		if data & 0x8:
			data -= 0x10
		self.tune = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX173 PianoBody/Damper Simulation object into raw data'''
		bytes = [0] * 9
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.sound_board_depth << 5) & 0xE0
		bytes[4] = ((self.sound_board_depth >> 3) & 0xF) | ((self.damper_depth << 4) & 0xF0)
		bytes[5] = ((self.damper_depth >> 4) & 0x7) | ((self.damper_mod_source << 3) & 0xF8)
		bytes[6] = ((self.damper_mod_source >> 5) & 0x1) | ((self.tone & 0x7F) << 1)
		bytes[7] = (self.mid_shape & 0x3F) | ((self.tune << 6) & 0xC0)
		bytes[8] = (self.tune >> 2) & 0x3
		return struct.pack('<9B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX173 PianoBody/Damper Simulation object'''
		json = []
		json.append(('name','FX173'))
		json.append(('description','PianoBody/Damper Simulation'))
		json.append(['sound_board_depth',self.sound_board_depth])
		json.append(['damper_depth',self.damper_depth])
		json.append(['damper_mod_source',self.damper_mod_source])
		json.append(['tone',self.tone])
		json.append(['mid_shape',self.mid_shape])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['tune',self.tune])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def sound_board_depth(self):
		'''Sound Board Depth'''
		return self._sound_board_depth

	@sound_board_depth.setter
	def sound_board_depth(self,value):
		if 0 <= value <= 100:
			self._sound_board_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def damper_depth(self):
		'''Damper Depth'''
		return self._damper_depth

	@damper_depth.setter
	def damper_depth(self,value):
		if 0 <= value <= 100:
			self._damper_depth = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def damper_mod_source(self):
		'''Damper Mod.Source'''
		return self._damper_mod_source

	@damper_mod_source.setter
	def damper_mod_source(self,value):
		if 0 <= value <= 37:
			self._damper_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def tone(self):
		'''Tone'''
		return self._tone

	@tone.setter
	def tone(self,value):
		if 1 <= value <= 100:
			self._tone = value
		else:
			raise ValueError('%s not 1 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def mid_shape(self):
		'''Mid Shape'''
		return self._mid_shape

	@mid_shape.setter
	def mid_shape(self,value):
		if 0 <= value <= 36:
			self._mid_shape = value
		else:
			raise ValueError('%s not 0 <= value <= 36' % value)

#-------------------------------------------------------------------------------
	@property
	def tune(self):
		'''Tune'''
		return self._tune

	@tune.setter
	def tune(self,value):
		if -5 <= value <= 5:
			self._tune = value
		else:
			raise ValueError('%s not -5 <= value <= 5' % value)

