#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx174.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX174(object):
	category = '''Double Size'''
	description = '''Vocoder'''
	short_description = '''Vocoder'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX174 Vocoder object'''
		bytes = struct.unpack('<11B',raw[:11])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.carrier_trim = ((bytes[2] & 0xC0) >> 6) | ((bytes[3] & 0x1F) << 2)
		self.modulator_source = (bytes[3] >> 5) & 0x3
		self.modulator_select = ((bytes[3] & 0x80) >> 7) | ((bytes[4] & 0x1) << 1)
		self.modulator_trim = (bytes[4] >> 1) & 0x7F
		data = bytes[5] & 0x7
		if data & 0x4:
			data -= 0x8
		self.formant_shift = data
		self.response = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		self.noise_level = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x1) << 6)
		self.noise_level_mod_source = (bytes[7] >> 1) & 0x3F
		data = ((bytes[7] & 0x80) >> 7) | ((bytes[8] & 0x7F) << 1)
		if data & 0x80:
			data -= 0x100
		self.noise_level_mod_intensity = data
		data = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0xF) << 1)
		if data & 0x10:
			data -= 0x20
		self.low_gain = data
		data = ((bytes[9] & 0xF0) >> 4) | ((bytes[10] & 0x1) << 4)
		if data & 0x10:
			data -= 0x20
		self.high_gain = data
		self.modulator_high_mix = (bytes[10] >> 1) & 0x7F

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX174 Vocoder object into raw data'''
		bytes = [0] * 11
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = ((self.wet_dry_mod_intensity >> 2) & 0x3F) | ((self.carrier_trim << 6) & 0xC0)
		bytes[3] = ((self.carrier_trim >> 2) & 0x1F) | ((self.modulator_source & 0x3) << 5) | ((self.modulator_select << 7) & 0x80)
		bytes[4] = ((self.modulator_select >> 1) & 0x1) | ((self.modulator_trim & 0x7F) << 1)
		bytes[5] = (self.formant_shift & 0x7) | ((self.response << 3) & 0xF8)
		bytes[6] = ((self.response >> 5) & 0x3) | ((self.noise_level << 2) & 0xFC)
		bytes[7] = ((self.noise_level >> 6) & 0x1) | ((self.noise_level_mod_source & 0x3F) << 1) | ((self.noise_level_mod_intensity << 7) & 0x80)
		bytes[8] = ((self.noise_level_mod_intensity >> 1) & 0x7F) | ((self.low_gain << 7) & 0x80)
		bytes[9] = ((self.low_gain >> 1) & 0xF) | ((self.high_gain << 4) & 0xF0)
		bytes[10] = ((self.high_gain >> 4) & 0x1) | ((self.modulator_high_mix & 0x7F) << 1)
		return struct.pack('<11B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX174 Vocoder object'''
		json = []
		json.append(('name','FX174'))
		json.append(('description','Vocoder'))
		json.append(['carrier_trim',self.carrier_trim])
		json.append(['modulator_trim',self.modulator_trim])
		json.append(['formant_shift',self.formant_shift])
		json.append(['response',self.response])
		json.append(['noise_level',self.noise_level])
		json.append(['noise_level_mod_source',self.noise_level_mod_source])
		json.append(['noise_level_mod_intensity',self.noise_level_mod_intensity])
		json.append(['low_gain',self.low_gain])
		json.append(['high_gain',self.high_gain])
		json.append(['modulator_high_mix',self.modulator_high_mix])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['modulator_source',self.modulator_source])
		json.append(['modulator_select',self.modulator_select])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def carrier_trim(self):
		'''Carrier Trim'''
		return self._carrier_trim

	@carrier_trim.setter
	def carrier_trim(self,value):
		if 0 <= value <= 100:
			self._carrier_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def modulator_source(self):
		'''Modulator Source'''
		return self._modulator_source

	@modulator_source.setter
	def modulator_source(self,value):
		if 0 <= value <= 2:
			self._modulator_source = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def modulator_select(self):
		'''Modulator Select'''
		return self._modulator_select

	@modulator_select.setter
	def modulator_select(self,value):
		if 0 <= value <= 2:
			self._modulator_select = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def modulator_trim(self):
		'''Modulator Trim'''
		return self._modulator_trim

	@modulator_trim.setter
	def modulator_trim(self,value):
		if 0 <= value <= 100:
			self._modulator_trim = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def formant_shift(self):
		'''Formant Shift'''
		return self._formant_shift

	@formant_shift.setter
	def formant_shift(self,value):
		if -2 <= value <= 2:
			self._formant_shift = value
		else:
			raise ValueError('%s not -2 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def response(self):
		'''Response'''
		return self._response

	@response.setter
	def response(self,value):
		if 0 <= value <= 100:
			self._response = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def noise_level(self):
		'''Noise Level'''
		return self._noise_level

	@noise_level.setter
	def noise_level(self,value):
		if 0 <= value <= 100:
			self._noise_level = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def noise_level_mod_source(self):
		'''Noise Level Mod.Source'''
		return self._noise_level_mod_source

	@noise_level_mod_source.setter
	def noise_level_mod_source(self,value):
		if 0 <= value <= 37:
			self._noise_level_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def noise_level_mod_intensity(self):
		'''Noise Level Mod.Intensity'''
		return self._noise_level_mod_intensity

	@noise_level_mod_intensity.setter
	def noise_level_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._noise_level_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_gain(self):
		'''Low Gain'''
		return self._low_gain

	@low_gain.setter
	def low_gain(self,value):
		if -12 <= value <= 12:
			self._low_gain = value
		else:
			raise ValueError('%s not -12 <= value <= 12' % value)

#-------------------------------------------------------------------------------
	@property
	def high_gain(self):
		'''High Gain'''
		return self._high_gain

	@high_gain.setter
	def high_gain(self,value):
		if -12 <= value <= 12:
			self._high_gain = value
		else:
			raise ValueError('%s not -12 <= value <= 12' % value)

#-------------------------------------------------------------------------------
	@property
	def modulator_high_mix(self):
		'''Modulator High Mix'''
		return self._modulator_high_mix

	@modulator_high_mix.setter
	def modulator_high_mix(self,value):
		if 0 <= value <= 100:
			self._modulator_high_mix = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

