#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx177.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX177(object):
	category = '''Double Size'''
	description = '''Guitar Amp Clean Combo'''
	short_description = '''G.Amp Clean Combo'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX177 Guitar Amp Clean Combo object'''
		bytes = struct.unpack('<11B',raw[:11])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.gain = ((bytes[3] & 0xE0) >> 5) | ((bytes[4] & 0xF) << 3)
		self.volume = ((bytes[4] & 0xF0) >> 4) | ((bytes[5] & 0x7) << 4)
		self.bass = ((bytes[5] & 0xF8) >> 3) | ((bytes[6] & 0x3) << 5)
		self.middle = ((bytes[6] & 0xFC) >> 2) | ((bytes[7] & 0x1) << 6)
		self.treble = (bytes[7] >> 1) & 0x7F
		self.presence = bytes[8] & 0x7F
		self.volume_mod_source = ((bytes[8] & 0x80) >> 7) | ((bytes[9] & 0x1F) << 1)
		data = ((bytes[9] & 0xE0) >> 5) | ((bytes[10] & 0x1F) << 3)
		if data & 0x80:
			data -= 0x100
		self.volume_mod_intensity = data

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX177 Guitar Amp Clean Combo object into raw data'''
		bytes = [0] * 11
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.gain << 5) & 0xE0
		bytes[4] = ((self.gain >> 3) & 0xF) | ((self.volume << 4) & 0xF0)
		bytes[5] = ((self.volume >> 4) & 0x7) | ((self.bass << 3) & 0xF8)
		bytes[6] = ((self.bass >> 5) & 0x3) | ((self.middle << 2) & 0xFC)
		bytes[7] = ((self.middle >> 6) & 0x1) | ((self.treble & 0x7F) << 1)
		bytes[8] = (self.presence & 0x7F) | ((self.volume_mod_source << 7) & 0x80)
		bytes[9] = ((self.volume_mod_source >> 1) & 0x1F) | ((self.volume_mod_intensity << 5) & 0xE0)
		bytes[10] = (self.volume_mod_intensity >> 3) & 0x1F
		return struct.pack('<11B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX177 Guitar Amp Clean Combo object'''
		json = []
		json.append(('name','FX177'))
		json.append(('description','Guitar Amp Clean Combo'))
		json.append(['gain',self.gain])
		json.append(['volume',self.volume])
		json.append(['bass',self.bass])
		json.append(['volume_mod_source',self.volume_mod_source])
		json.append(['middle',self.middle])
		json.append(['volume_mod_intensity',self.volume_mod_intensity])
		json.append(['treble',self.treble])
		json.append(['presence',self.presence])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def gain(self):
		'''Gain'''
		return self._gain

	@gain.setter
	def gain(self,value):
		if 0 <= value <= 100:
			self._gain = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def volume(self):
		'''Volume'''
		return self._volume

	@volume.setter
	def volume(self,value):
		if 0 <= value <= 100:
			self._volume = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass(self):
		'''Bass'''
		return self._bass

	@bass.setter
	def bass(self,value):
		if 0 <= value <= 100:
			self._bass = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def middle(self):
		'''Middle'''
		return self._middle

	@middle.setter
	def middle(self,value):
		if 0 <= value <= 100:
			self._middle = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def treble(self):
		'''Treble'''
		return self._treble

	@treble.setter
	def treble(self,value):
		if 0 <= value <= 100:
			self._treble = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def presence(self):
		'''Presence'''
		return self._presence

	@presence.setter
	def presence(self,value):
		if 0 <= value <= 100:
			self._presence = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def volume_mod_source(self):
		'''Volume Mod.Source'''
		return self._volume_mod_source

	@volume_mod_source.setter
	def volume_mod_source(self,value):
		if 0 <= value <= 37:
			self._volume_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def volume_mod_intensity(self):
		'''Volume Mod.Intensity'''
		return self._volume_mod_intensity

	@volume_mod_intensity.setter
	def volume_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._volume_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

