#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx181.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

#===============================================================================
class FX181(object):
	category = '''Double Size'''
	description = '''Bass Tube Amp Model + Cabinet'''
	short_description = '''BassTubeAmp+Cab.'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX181 Bass Tube Amp Model + Cabinet object'''
		bytes = struct.unpack('<12B',raw[:12])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.bass_amp_model_tube_drive_amp_type = (bytes[3] >> 5) & 0x3
		self.bass_amp_model_tube_drive_gain = ((bytes[3] & 0x80) >> 7) | ((bytes[4] & 0x3F) << 1)
		self.bass_amp_model_tube_drive_volume = ((bytes[4] & 0xC0) >> 6) | ((bytes[5] & 0x1F) << 2)
		self.bass_amp_model_tube_drive_volume_mod_source = ((bytes[5] & 0xE0) >> 5) | ((bytes[6] & 0x7) << 3)
		data = ((bytes[6] & 0xF8) >> 3) | ((bytes[7] & 0x7) << 5)
		if data & 0x80:
			data -= 0x100
		self.bass_amp_model_tube_drive_volume_mod_intensity = data
		self.bass_amp_model_tube_drive_bass = ((bytes[7] & 0xF8) >> 3) | ((bytes[8] & 0x3) << 5)
		self.bass_amp_model_tube_drive_middle = ((bytes[8] & 0xFC) >> 2) | ((bytes[9] & 0x1) << 6)
		self.bass_amp_model_tube_drive_treble = (bytes[9] >> 1) & 0x7F
		self.bass_amp_model_tube_drive_presence = bytes[10] & 0x7F
		self.cabinet_cabinet_on = (bytes[10] >> 7) & 0x1
		self.cabinet_cabinet_type = bytes[11] & 0xF

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX181 Bass Tube Amp Model + Cabinet object into raw data'''
		bytes = [0] * 12
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = ((self.bass_amp_model_tube_drive_amp_type & 0x3) << 5) | ((self.bass_amp_model_tube_drive_gain << 7) & 0x80)
		bytes[4] = ((self.bass_amp_model_tube_drive_gain >> 1) & 0x3F) | ((self.bass_amp_model_tube_drive_volume << 6) & 0xC0)
		bytes[5] = ((self.bass_amp_model_tube_drive_volume >> 2) & 0x1F) | ((self.bass_amp_model_tube_drive_volume_mod_source << 5) & 0xE0)
		bytes[6] = ((self.bass_amp_model_tube_drive_volume_mod_source >> 3) & 0x7) | ((self.bass_amp_model_tube_drive_volume_mod_intensity << 3) & 0xF8)
		bytes[7] = ((self.bass_amp_model_tube_drive_volume_mod_intensity >> 5) & 0x7) | ((self.bass_amp_model_tube_drive_bass << 3) & 0xF8)
		bytes[8] = ((self.bass_amp_model_tube_drive_bass >> 5) & 0x3) | ((self.bass_amp_model_tube_drive_middle << 2) & 0xFC)
		bytes[9] = ((self.bass_amp_model_tube_drive_middle >> 6) & 0x1) | ((self.bass_amp_model_tube_drive_treble & 0x7F) << 1)
		bytes[10] = (self.bass_amp_model_tube_drive_presence & 0x7F) | ((self.cabinet_cabinet_on & 0x1) << 7)
		bytes[11] = self.cabinet_cabinet_type & 0xF
		return struct.pack('<12B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX181 Bass Tube Amp Model + Cabinet object'''
		json = []
		json.append(('name','FX181'))
		json.append(('description','Bass Tube Amp Model + Cabinet'))
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['bass_amp_model_tube_drive_amp_type',self.bass_amp_model_tube_drive_amp_type])
		json.append(['bass_amp_model_tube_drive_gain',self.bass_amp_model_tube_drive_gain])
		json.append(['bass_amp_model_tube_drive_volume',self.bass_amp_model_tube_drive_volume])
		json.append(['bass_amp_model_tube_drive_volume_mod_source',self.bass_amp_model_tube_drive_volume_mod_source])
		json.append(['bass_amp_model_tube_drive_volume_mod_intensity',self.bass_amp_model_tube_drive_volume_mod_intensity])
		json.append(['bass_amp_model_tube_drive_bass',self.bass_amp_model_tube_drive_bass])
		json.append(['bass_amp_model_tube_drive_middle',self.bass_amp_model_tube_drive_middle])
		json.append(['bass_amp_model_tube_drive_treble',self.bass_amp_model_tube_drive_treble])
		json.append(['bass_amp_model_tube_drive_presence',self.bass_amp_model_tube_drive_presence])
		json.append(['cabinet_cabinet_on',self.cabinet_cabinet_on])
		json.append(['cabinet_cabinet_type',self.cabinet_cabinet_type])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_tube_drive_amp_type(self):
		'''(Bass Amp Model Tube Drive) Amp Type'''
		return self._bass_amp_model_tube_drive_amp_type

	@bass_amp_model_tube_drive_amp_type.setter
	def bass_amp_model_tube_drive_amp_type(self,value):
		if 0 <= value <= 2:
			self._bass_amp_model_tube_drive_amp_type = value
		else:
			raise ValueError('%s not 0 <= value <= 2' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_tube_drive_gain(self):
		'''(Bass Amp Model Tube Drive) Gain'''
		return self._bass_amp_model_tube_drive_gain

	@bass_amp_model_tube_drive_gain.setter
	def bass_amp_model_tube_drive_gain(self,value):
		if 0 <= value <= 100:
			self._bass_amp_model_tube_drive_gain = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_tube_drive_volume(self):
		'''(Bass Amp Model Tube Drive) Volume'''
		return self._bass_amp_model_tube_drive_volume

	@bass_amp_model_tube_drive_volume.setter
	def bass_amp_model_tube_drive_volume(self,value):
		if 0 <= value <= 100:
			self._bass_amp_model_tube_drive_volume = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_tube_drive_volume_mod_source(self):
		'''(Bass Amp Model Tube Drive) Volume Mod.Source'''
		return self._bass_amp_model_tube_drive_volume_mod_source

	@bass_amp_model_tube_drive_volume_mod_source.setter
	def bass_amp_model_tube_drive_volume_mod_source(self,value):
		if 0 <= value <= 37:
			self._bass_amp_model_tube_drive_volume_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_tube_drive_volume_mod_intensity(self):
		'''(Bass Amp Model Tube Drive) Volume Mod.Intensity'''
		return self._bass_amp_model_tube_drive_volume_mod_intensity

	@bass_amp_model_tube_drive_volume_mod_intensity.setter
	def bass_amp_model_tube_drive_volume_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._bass_amp_model_tube_drive_volume_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_tube_drive_bass(self):
		'''(Bass Amp Model Tube Drive) Bass'''
		return self._bass_amp_model_tube_drive_bass

	@bass_amp_model_tube_drive_bass.setter
	def bass_amp_model_tube_drive_bass(self,value):
		if 0 <= value <= 100:
			self._bass_amp_model_tube_drive_bass = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_tube_drive_middle(self):
		'''(Bass Amp Model Tube Drive) Middle'''
		return self._bass_amp_model_tube_drive_middle

	@bass_amp_model_tube_drive_middle.setter
	def bass_amp_model_tube_drive_middle(self,value):
		if 0 <= value <= 100:
			self._bass_amp_model_tube_drive_middle = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_tube_drive_treble(self):
		'''(Bass Amp Model Tube Drive) Treble'''
		return self._bass_amp_model_tube_drive_treble

	@bass_amp_model_tube_drive_treble.setter
	def bass_amp_model_tube_drive_treble(self,value):
		if 0 <= value <= 100:
			self._bass_amp_model_tube_drive_treble = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bass_amp_model_tube_drive_presence(self):
		'''(Bass Amp Model Tube Drive) Presence'''
		return self._bass_amp_model_tube_drive_presence

	@bass_amp_model_tube_drive_presence.setter
	def bass_amp_model_tube_drive_presence(self,value):
		if 0 <= value <= 100:
			self._bass_amp_model_tube_drive_presence = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def cabinet_cabinet_on(self):
		'''(Cabinet) Cabinet On'''
		return self._cabinet_cabinet_on

	@cabinet_cabinet_on.setter
	def cabinet_cabinet_on(self,value):
		if 0 <= value <= 1:
			self._cabinet_cabinet_on = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def cabinet_cabinet_type(self):
		'''(Cabinet) Cabinet Type'''
		return self._cabinet_cabinet_type

	@cabinet_cabinet_type.setter
	def cabinet_cabinet_type(self,value):
		if 0 <= value <= 11:
			self._cabinet_cabinet_type = value
		else:
			raise ValueError('%s not 0 <= value <= 11' % value)

