#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx190.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX190(object):
	category = '''Double Size'''
	description = '''Hold Delay'''
	short_description = '''Hold Delay'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX190 Hold Delay object'''
		bytes = struct.unpack('<15B',raw[:15])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.bpm_midi_sync = (bytes[5] >> 2) & 0x1
		self.bpm = ((bytes[5] & 0xF8) >> 3) | (bytes[6] << 5) | ((bytes[7] & 0x3) << 13)
		self.bpm_base_note = (bytes[7] >> 2) & 0xF
		self.bpm_times = ((bytes[7] & 0xC0) >> 6) | ((bytes[8] & 0xF) << 2)
		self.rec_control_source = ((bytes[8] & 0xF0) >> 4) | ((bytes[9] & 0x3) << 4)
		self.reset_control_source = (bytes[9] >> 2) & 0x3F
		self.manual_rec_control = bytes[10] & 0x1
		self.manual_reset_control = (bytes[10] >> 1) & 0x1
		data = ((bytes[10] & 0xFC) >> 2) | ((bytes[11] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.pan = data
		self.pan_mod_source = (bytes[11] >> 2) & 0x3F
		data = bytes[12] & 0xFF
		if data & 0x80:
			data -= 0x100
		self.pan_mod_intensity = data
		self.loop_time = ((bytes[13] & 0xFC) >> 2) | ((bytes[14] & 0xFF) << 6)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX190 Hold Delay object into raw data'''
		bytes = [0] * 15
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[5] = ((self.bpm_midi_sync & 0x1) << 2) | ((self.bpm << 3) & 0xF8)
		bytes[6] = (self.bpm >> 5) & 0xFF
		bytes[7] = ((self.bpm >> 3) & 0xD) | ((self.bpm_base_note & 0xF) << 2) | ((self.bpm_times << 6) & 0xC0)
		bytes[8] = ((self.bpm_times >> 2) & 0xF) | ((self.rec_control_source << 4) & 0xF0)
		bytes[9] = ((self.rec_control_source >> 4) & 0x3) | ((self.reset_control_source & 0x3F) << 2)
		bytes[10] = (self.manual_rec_control & 0x1) | ((self.manual_reset_control & 0x1) << 1) | ((self.pan << 2) & 0xFC)
		bytes[11] = ((self.pan >> 6) & 0x3) | ((self.pan_mod_source & 0x3F) << 2)
		bytes[12] = self.pan_mod_intensity & 0xFF
		bytes[13] = (self.loop_time << 2) & 0xFC
		bytes[14] = (self.loop_time >> 6) & 0xFF
		return struct.pack('<15B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX190 Hold Delay object'''
		json = []
		json.append(('name','FX190'))
		json.append(('description','Hold Delay'))
		json.append(['rec_control_source',self.rec_control_source])
		json.append(['manual_rec_control',self.manual_rec_control])
		json.append(['reset_control_source',self.reset_control_source])
		json.append(['manual_reset_control',self.manual_reset_control])
		json.append(['pan',self.pan])
		json.append(['pan_mod_source',self.pan_mod_source])
		json.append(['pan_mod_intensity',self.pan_mod_intensity])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		json.append(['loop_time',self.loop_time])
		json.append(['bpm_midi_sync',self.bpm_midi_sync])
		json.append(['bpm',self.bpm])
		json.append(['bpm_display',self.bpm_display])
		json.append(['bpm_base_note',self.bpm_base_note])
		json.append(['bpm_base_note_display',self.bpm_base_note_display])
		json.append(['bpm_times',self.bpm_times])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bpm_midi_sync(self):
		'''BPM/MIDI Sync'''
		return self._bpm_midi_sync

	@bpm_midi_sync.setter
	def bpm_midi_sync(self,value):
		if 0 <= value <= 1:
			self._bpm_midi_sync = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def bpm(self):
		'''BPM'''
		return self._bpm

	@bpm.setter
	def bpm(self,value):
		if 3999 <= value <= 30000:
			self._bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def bpm_display(self):
		'''BPM display value'''
		return bpm_decode(self._bpm)

#-------------------------------------------------------------------------------
	@property
	def bpm_base_note(self):
		'''BPM Base Note'''
		return self._bpm_base_note

	@bpm_base_note.setter
	def bpm_base_note(self,value):
		if 0 <= value <= 9:
			self._bpm_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def bpm_base_note_display(self):
		'''BPM Base Note display value'''
		return base_note_dict[self._bpm_base_note]

#-------------------------------------------------------------------------------
	@property
	def bpm_times(self):
		'''BPM Times'''
		return self._bpm_times

	@bpm_times.setter
	def bpm_times(self,value):
		if 1 <= value <= 32:
			self._bpm_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def rec_control_source(self):
		'''Rec Control Source'''
		return self._rec_control_source

	@rec_control_source.setter
	def rec_control_source(self,value):
		if 0 <= value <= 37:
			self._rec_control_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def reset_control_source(self):
		'''Reset Control Source'''
		return self._reset_control_source

	@reset_control_source.setter
	def reset_control_source(self,value):
		if 0 <= value <= 37:
			self._reset_control_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def manual_rec_control(self):
		'''Manual Rec Control'''
		return self._manual_rec_control

	@manual_rec_control.setter
	def manual_rec_control(self,value):
		if 0 <= value <= 1:
			self._manual_rec_control = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def manual_reset_control(self):
		'''Manual Reset Control'''
		return self._manual_reset_control

	@manual_reset_control.setter
	def manual_reset_control(self,value):
		if 0 <= value <= 1:
			self._manual_reset_control = value
		else:
			raise ValueError('%s not 0 <= value <= 1' % value)

#-------------------------------------------------------------------------------
	@property
	def pan(self):
		'''Pan'''
		return self._pan

	@pan.setter
	def pan(self,value):
		if -100 <= value <= 100:
			self._pan = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def pan_mod_source(self):
		'''Pan Mod.Source'''
		return self._pan_mod_source

	@pan_mod_source.setter
	def pan_mod_source(self,value):
		if 0 <= value <= 37:
			self._pan_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def pan_mod_intensity(self):
		'''Pan Mod.Intensity'''
		return self._pan_mod_intensity

	@pan_mod_intensity.setter
	def pan_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._pan_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def loop_time(self):
		'''Loop Time'''
		return self._loop_time

	@loop_time.setter
	def loop_time(self,value):
		if 0 <= value <= 10800:
			self._loop_time = value
		else:
			raise ValueError('%s not 0 <= value <= 10800' % value)

