#!@PYTHON@
# -*- coding: utf-8 -*-
#
# fx191.py -- classes for Korg Krome Music Workstation
#
# (c) 2014  Brian M. Ames <bmames@apk.net>

import math
import struct
from modsrc import module_sources

def bpm_decode(bpm):
	return 'MIDI' if bpm==0xF9F else '%3.2f' % (bpm*0.01)

base_note_dict = {0: '1/32', 1: '1/24', 2: '1/16', 3: '1/12', 4: '1/8', 5: '1/6', 6: '1/4', 7: '1/3', 8: '1/2', 9: '1/1'}

#===============================================================================
class FX191(object):
	category = '''Double Size'''
	description = '''Left, Center, Right Beats per Minute Long Delay'''
	short_description = '''LCR BPM Long Dly'''
#-------------------------------------------------------------------------------
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a FX191 Left, Center, Right Beats per Minute Long Delay object'''
		bytes = struct.unpack('<19B',raw[:19])
		self.wet_dry = (bytes[0] >> 1) & 0x7F
		self.wet_dry_mod_source = bytes[1] & 0x3F
		data = ((bytes[1] & 0xC0) >> 6) | ((bytes[2] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.wet_dry_mod_intensity = data
		self.bpm = ((bytes[3] & 0xE0) >> 5) | (bytes[4] << 3) | ((bytes[5] & 0xF) << 11)
		self.l_delay_base_note = (bytes[5] >> 4) & 0xF
		self.l_delay_times = bytes[6] & 0x3F
		self.l_delay_level = ((bytes[6] & 0xC0) >> 6) | ((bytes[7] & 0xF) << 2)
		self.c_delay_base_note = (bytes[7] >> 4) & 0xF
		self.c_delay_times = bytes[8] & 0x3F
		self.c_delay_level = ((bytes[8] & 0xC0) >> 6) | ((bytes[9] & 0xF) << 2)
		self.r_delay_base_note = (bytes[9] >> 4) & 0xF
		self.r_delay_times = bytes[10] & 0x3F
		self.r_delay_level = ((bytes[10] & 0xC0) >> 6) | ((bytes[11] & 0xF) << 2)
		data = ((bytes[11] & 0xF0) >> 4) | ((bytes[12] & 0xF) << 4)
		if data & 0x80:
			data -= 0x100
		self.feedback = data
		self.feedback_mod_source = ((bytes[12] & 0xF0) >> 4) | ((bytes[13] & 0x3) << 4)
		data = ((bytes[13] & 0xFC) >> 2) | ((bytes[14] & 0x3) << 6)
		if data & 0x80:
			data -= 0x100
		self.feedback_mod_intensity = data
		self.high_damp = ((bytes[14] & 0xFC) >> 2) | ((bytes[15] & 0x1) << 6)
		self.low_damp = (bytes[15] >> 1) & 0x7F
		self.input_trim_mod_source = bytes[16] & 0x3F
		data = ((bytes[16] & 0xC0) >> 6) | ((bytes[17] & 0x3F) << 2)
		if data & 0x80:
			data -= 0x100
		self.input_trim_mod_intensity = data
		self.spread = ((bytes[17] & 0xC0) >> 6) | ((bytes[18] & 0xF) << 2)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a FX191 Left, Center, Right Beats per Minute Long Delay object into raw data'''
		bytes = [0] * 19
		bytes[0] = (self.wet_dry & 0x7F) << 1
		bytes[1] = (self.wet_dry_mod_source & 0x3F) | ((self.wet_dry_mod_intensity << 6) & 0xC0)
		bytes[2] = (self.wet_dry_mod_intensity >> 2) & 0x3F
		bytes[3] = (self.bpm << 5) & 0xE0
		bytes[4] = (self.bpm >> 3) & 0xFF
		bytes[5] = ((self.bpm >> 15) & 0xB) | ((self.l_delay_base_note & 0xF) << 4)
		bytes[6] = (self.l_delay_times & 0x3F) | ((self.l_delay_level << 6) & 0xC0)
		bytes[7] = ((self.l_delay_level >> 2) & 0xF) | ((self.c_delay_base_note & 0xF) << 4)
		bytes[8] = (self.c_delay_times & 0x3F) | ((self.c_delay_level << 6) & 0xC0)
		bytes[9] = ((self.c_delay_level >> 2) & 0xF) | ((self.r_delay_base_note & 0xF) << 4)
		bytes[10] = (self.r_delay_times & 0x3F) | ((self.r_delay_level << 6) & 0xC0)
		bytes[11] = ((self.r_delay_level >> 2) & 0xF) | ((self.feedback << 4) & 0xF0)
		bytes[12] = ((self.feedback >> 4) & 0xF) | ((self.feedback_mod_source << 4) & 0xF0)
		bytes[13] = ((self.feedback_mod_source >> 4) & 0x3) | ((self.feedback_mod_intensity << 2) & 0xFC)
		bytes[14] = ((self.feedback_mod_intensity >> 6) & 0x3) | ((self.high_damp << 2) & 0xFC)
		bytes[15] = ((self.high_damp >> 6) & 0x1) | ((self.low_damp & 0x7F) << 1)
		bytes[16] = (self.input_trim_mod_source & 0x3F) | ((self.input_trim_mod_intensity << 6) & 0xC0)
		bytes[17] = ((self.input_trim_mod_intensity >> 2) & 0x3F) | ((self.spread << 6) & 0xC0)
		bytes[18] = (self.spread >> 2) & 0xF
		return struct.pack('<19B',*bytes)

#-------------------------------------------------------------------------------
	def toJSON(self):
		'''Return JSON representation of a FX191 Left, Center, Right Beats per Minute Long Delay object'''
		json = []
		json.append(('name','FX191'))
		json.append(('description','Left, Center, Right Beats per Minute Long Delay'))
		json.append(['bpm',self.bpm])
		json.append(['bpm_display',self.bpm_display])
		json.append(['l_delay_base_note',self.l_delay_base_note])
		json.append(['l_delay_base_note_display',self.l_delay_base_note_display])
		json.append(['l_delay_times',self.l_delay_times])
		json.append(['l_delay_level',self.l_delay_level])
		json.append(['c_delay_base_note',self.c_delay_base_note])
		json.append(['c_delay_base_note_display',self.c_delay_base_note_display])
		json.append(['c_delay_times',self.c_delay_times])
		json.append(['c_delay_level',self.c_delay_level])
		json.append(['r_delay_base_note',self.r_delay_base_note])
		json.append(['r_delay_base_note_display',self.r_delay_base_note_display])
		json.append(['r_delay_times',self.r_delay_times])
		json.append(['r_delay_level',self.r_delay_level])
		json.append(['feedback',self.feedback])
		json.append(['feedback_mod_source',self.feedback_mod_source])
		json.append(['feedback_mod_intensity',self.feedback_mod_intensity])
		json.append(['high_damp',self.high_damp])
		json.append(['low_damp',self.low_damp])
		json.append(['input_trim_mod_source',self.input_trim_mod_source])
		json.append(['input_trim_mod_intensity',self.input_trim_mod_intensity])
		json.append(['spread',self.spread])
		json.append(['wet_dry',self.wet_dry])
		json.append(['wet_dry_mod_source',self.wet_dry_mod_source])
		json.append(['wet_dry_mod_intensity',self.wet_dry_mod_intensity])
		return tuple(json)

#-------------------------------------------------------------------------------
	@property
	def wet_dry(self):
		'''Wet/Dry'''
		return self._wet_dry

	@wet_dry.setter
	def wet_dry(self,value):
		if 0 <= value <= 100:
			self._wet_dry = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_source(self):
		'''Wet/Dry Mod.Source'''
		return self._wet_dry_mod_source

	@wet_dry_mod_source.setter
	def wet_dry_mod_source(self,value):
		if 0 <= value <= 37:
			self._wet_dry_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def wet_dry_mod_intensity(self):
		'''Wet/Dry Mod.Intensity'''
		return self._wet_dry_mod_intensity

	@wet_dry_mod_intensity.setter
	def wet_dry_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._wet_dry_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def bpm(self):
		'''BPM'''
		return self._bpm

	@bpm.setter
	def bpm(self,value):
		if 3999 <= value <= 30000:
			self._bpm = value
		else:
			raise ValueError('%s not 3999 <= value <= 30000' % value)

	@property
	def bpm_display(self):
		'''BPM display value'''
		return bpm_decode(self._bpm)

#-------------------------------------------------------------------------------
	@property
	def l_delay_base_note(self):
		'''L Delay Base Note'''
		return self._l_delay_base_note

	@l_delay_base_note.setter
	def l_delay_base_note(self,value):
		if 0 <= value <= 9:
			self._l_delay_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def l_delay_base_note_display(self):
		'''L Delay Base Note display value'''
		return base_note_dict[self._l_delay_base_note]

#-------------------------------------------------------------------------------
	@property
	def l_delay_times(self):
		'''L Delay Times'''
		return self._l_delay_times

	@l_delay_times.setter
	def l_delay_times(self,value):
		if 1 <= value <= 32:
			self._l_delay_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def l_delay_level(self):
		'''L Delay Level'''
		return self._l_delay_level

	@l_delay_level.setter
	def l_delay_level(self,value):
		if 0 <= value <= 50:
			self._l_delay_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def c_delay_base_note(self):
		'''C Delay Base Note'''
		return self._c_delay_base_note

	@c_delay_base_note.setter
	def c_delay_base_note(self,value):
		if 0 <= value <= 9:
			self._c_delay_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def c_delay_base_note_display(self):
		'''C Delay Base Note display value'''
		return base_note_dict[self._c_delay_base_note]

#-------------------------------------------------------------------------------
	@property
	def c_delay_times(self):
		'''C Delay Times'''
		return self._c_delay_times

	@c_delay_times.setter
	def c_delay_times(self,value):
		if 1 <= value <= 32:
			self._c_delay_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def c_delay_level(self):
		'''C Delay Level'''
		return self._c_delay_level

	@c_delay_level.setter
	def c_delay_level(self,value):
		if 0 <= value <= 50:
			self._c_delay_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_base_note(self):
		'''R Delay Base Note'''
		return self._r_delay_base_note

	@r_delay_base_note.setter
	def r_delay_base_note(self,value):
		if 0 <= value <= 9:
			self._r_delay_base_note = value
		else:
			raise ValueError('%s not 0 <= value <= 9' % value)

	@property
	def r_delay_base_note_display(self):
		'''R Delay Base Note display value'''
		return base_note_dict[self._r_delay_base_note]

#-------------------------------------------------------------------------------
	@property
	def r_delay_times(self):
		'''R Delay Times'''
		return self._r_delay_times

	@r_delay_times.setter
	def r_delay_times(self,value):
		if 1 <= value <= 32:
			self._r_delay_times = value
		else:
			raise ValueError('%s not 1 <= value <= 32' % value)

#-------------------------------------------------------------------------------
	@property
	def r_delay_level(self):
		'''R Delay Level'''
		return self._r_delay_level

	@r_delay_level.setter
	def r_delay_level(self,value):
		if 0 <= value <= 50:
			self._r_delay_level = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback(self):
		'''Feedback'''
		return self._feedback

	@feedback.setter
	def feedback(self,value):
		if -100 <= value <= 100:
			self._feedback = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_source(self):
		'''Feedback Mod.Source'''
		return self._feedback_mod_source

	@feedback_mod_source.setter
	def feedback_mod_source(self,value):
		if 0 <= value <= 37:
			self._feedback_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def feedback_mod_intensity(self):
		'''Feedback Mod.Intensity'''
		return self._feedback_mod_intensity

	@feedback_mod_intensity.setter
	def feedback_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._feedback_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def high_damp(self):
		'''High Damp'''
		return self._high_damp

	@high_damp.setter
	def high_damp(self,value):
		if 0 <= value <= 100:
			self._high_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def low_damp(self):
		'''Low Damp'''
		return self._low_damp

	@low_damp.setter
	def low_damp(self,value):
		if 0 <= value <= 100:
			self._low_damp = value
		else:
			raise ValueError('%s not 0 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_source(self):
		'''Input Trim Mod.Source'''
		return self._input_trim_mod_source

	@input_trim_mod_source.setter
	def input_trim_mod_source(self,value):
		if 0 <= value <= 37:
			self._input_trim_mod_source = value
		else:
			raise ValueError('%s not 0 <= value <= 37' % value)

#-------------------------------------------------------------------------------
	@property
	def input_trim_mod_intensity(self):
		'''Input Trim Mod.Intensity'''
		return self._input_trim_mod_intensity

	@input_trim_mod_intensity.setter
	def input_trim_mod_intensity(self,value):
		if -100 <= value <= 100:
			self._input_trim_mod_intensity = value
		else:
			raise ValueError('%s not -100 <= value <= 100' % value)

#-------------------------------------------------------------------------------
	@property
	def spread(self):
		'''Spread'''
		return self._spread

	@spread.setter
	def spread(self,value):
		if 0 <= value <= 50:
			self._spread = value
		else:
			raise ValueError('%s not 0 <= value <= 50' % value)

