#!@PYTHON@
# -*- coding: utf-8 -*-
#
# sysex.py -- classes for Korg Krome Music Workstation
# 
# (c) 2014  Brian M. Ames <bmames@apk.net>

import struct
from table1 import ProgramParameters
from mode import mode_dict

#===============================================================================
def convert_7_bit_to_8_bit(data):
#	print len(data)
	result = []
	if isinstance(data,basestring):
		while len(data) >= 8:
			v = list(struct.unpack('>8B',data[:8]))
			data = data[8:]
			bit8 = v.pop(0)
			while v:
				result.append(chr(v.pop(0) | ((bit8 & 1) << 7)))
				bit8 >>= 1
		v = list(struct.unpack('>%dB'%len(data),data))
		bit8 = v.pop(0)
		while v:
			result.append(chr(v.pop(0) | ((bit8 & 1) << 7)))
			bit8 >>= 1
	elif isinstance(data,list):
		while len(data) >= 8:
			v = data[:8]
			data = data[8:]
			bit8 = v.pop(0)
			while v:
				result.append(chr(v.pop(0) | ((bit8 & 1) << 7)))
				bit8 >>= 1
		v = data
		bit8 = v.pop(0)
		while v:
			result.append(chr(v.pop(0) | ((bit8 & 1) << 7)))
			bit8 >>= 1
	return ''.join(result)

#===============================================================================
class DeviceInquiry(object):
	'''      DEVICE INQUlRY ( When received this message, transmits INQUlRY MESSAGE REPLY )
      [ F0,7E,nn,06,01,F7 ] 3rd byte nn : Channel = 0 - F : Global Channel
                                                  = 7F    : Any Channel'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0x7F):
		self.globchnl = globchnl

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = self.globchnl & 0x7F
		return struct.pack('>6B',0xF0, 0x7E, byte2, 0x06, 0x01, 0xF7)

#===============================================================================
class InquiryReply(object):
	''''''
#-------------------------------------------------------------------------------		
	def __init__(self,data=None):
		if data is None:
			self.device_id = None
			self.manufacturer_id = None
			self.device_family = None
			self.device_type = None
			self.version = None
		else:
			self.device_id = data[2]
			self.manufacturer_id = data[5:-9]
			self.device_family = (data[-8] << 7) | data[-9]
			self.device_type = (data[-4] << 7) | data[-5]
			self.version = (data[-2] << 21) | (data[-3] << 14) | (data[-4] << 7) | data[-5]

#-------------------------------------------------------------------------------		
	def __str__(self):
		return 'InquiryReply id %d manufacturer %s family %d type %d version %d' % (self.device_id,['0x%02X' % m for m in self.manufacturer_id],self.device_family,self.device_type,self.version)

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = self.globchnl & 0x7F
		return struct.pack('>6B',0xF0, 0x7E, byte2, 0x06, 0x01, 0xF7)

#===============================================================================
class GMSystemOn(object):
	'''      GM System On ( Receive when in Sequencer mode )
      [ F0,7E,nn,09,01,F7 ] 3rd byte nn : Channel = 0 - F : Global Channel
                                                  = 7F    : Any Channel'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0):
		self.globchnl = globchnl

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = self.globchnl & 0x7F
		return struct.pack('>6B',0xF0, 0x7E, byte2, 0x09, 0x01, 0xF7)

#===============================================================================
class SampleDumpRequest(object):
	'''      DEVICE INQUlRY ( When received this message, transmits INQUlRY MESSAGE REPLY )
      [ F0,7E,nn,06,01,F7 ] 3rd byte nn : Channel = 0 - F : Global Channel
                                                  = 7F    : Any Channel'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0x7F):
		self.globchnl = globchnl

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = self.globchnl & 0x7F
		return struct.pack('>6B',0xF0, 0x7E, byte2, 0x03, 0x00, 0xF7)

#===============================================================================
class DataDumpRequest(object):
	'''      DEVICE INQUlRY ( When received this message, transmits INQUlRY MESSAGE REPLY )
      [ F0,7E,nn,06,01,F7 ] 3rd byte nn : Channel = 0 - F : Global Channel
                                                  = 7F    : Any Channel'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0x7F):
		self.globchnl = globchnl

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = self.globchnl & 0x7F
#		return struct.pack('>6B',0xF0, 0x7E, byte2, 0x08, 0x00, 0xF7)
		return struct.pack('>7B',0xF0, 0x42, 0x30, 0x24, 0x0F, 0x00, 0xF7)

#===============================================================================
class MasterVolume(object):
	'''      Master Volume
      [ F0,7F,0g,04,01,vv,mm,F7 ] 3rd byte  g : Global Channel
                                  6th byte vv : Value(LSB)
                                  7th byte mm : Value(MSB)
                                           mm,vv = 00,00 - 7F,7F : Min - Max'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0,value=0):
		self.globchnl = globchnl
		self.value = value

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = self.globchnl & 0x0F
		byte5 = self.value & 0x7F
		byte6 = (self.value >> 7) & 0x7F
		return struct.pack('>8B',0xF0, 0x7F, byte2, 0x04, 0x01, byte5, byte6, 0xF7)

#===============================================================================
class MasterBalance(object):
	'''      Master Balance
      [ F0,7F,0g,04,02,vv,mm,F7 ] 3rd byte  g : Global Channel
                                  6th byte vv : Value(LSB)
                                  7th byte mm : Value(MSB)
                                  mm,vv = 00,00:Left, 40,00:Center, 7F,7F:Right'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0,value=0):
		self.globchnl = globchnl
		self.value = value

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = self.globchnl & 0x0F
		byte5 = self.value & 0x7F
		byte6 = (self.value >> 7) & 0x7F
		return struct.pack('>8B',0xF0, 0x7F, byte2, 0x04, 0x02, byte5, byte6, 0xF7)

#===============================================================================
class MasterFineTune(object):
	'''      Master Fine Tune ( Control Master Tune(cent) in Global )
      [ F0,7F,0g,04,03,vv,mm,F7 ] 3rd byte  g : Global Channel
                                  6th byte vv : Value(LSB)
                                  7th byte mm : Value(MSB)
                                           mm,vv = 20,00:-50, 40,00:+00, 60,00:+50'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0,value=0):
		self.globchnl = globchnl
		self.value = value

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = self.globchnl & 0x0F
		byte5 = self.value & 0x7F
		byte6 = (self.value >> 7) & 0x7F
		return struct.pack('>8B',0xF0, 0x7F, byte2, 0x04, 0x03, byte5, byte6, 0xF7)

#===============================================================================
class MasterCoarseTune(object):
	'''Master Coarse Tune ( Control Transpose (chromatic step) in Global )
      [ F0,7F,0g,04,04,vv,mm,F7 ] 3rd byte g : Global Channel
                                  6th byte vv : Value(LSB)
                                  7th byte mm : Value(MSB)
                                           mm,vv = 34,00:-12, 40,00:+00, 4C,00:+12'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0,value=0):
		self.globchnl = globchnl
		self.value = value

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = self.globchnl & 0x0F
		byte5 = self.value & 0x7F
		byte6 = (self.value >> 7) & 0x7F
		return struct.pack('>8B',0xF0, 0x7F, byte2, 0x04, 0x04, byte5, byte6, 0xF7)

#===============================================================================
class ModeRequest(object):
	'''(1) MODE REQUEST                                                        R
        F0, 42, 3g, 00, 01, 15  Excl Header
        12                      Function
        F7                      End of Excl
(Receives this message, and transmits Func=42 message)'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0):
		self.globchnl = globchnl

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		return struct.pack('>8B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x12, 0xF7)

#===============================================================================
class ModeData(object):
	'''(2) MODE DATA                                                           T
        F0, 42, 3g, 00, 01, 15  Excl Header
        42                      Function
        0000 0mmm               Mode                    (*2)
        0000 0000               reserved
        0sss ssss               data1
        0ddd dddd               data2
        00                      Reserved
        F7                      End of Excl
(Receives FUNC=12 message, and transmits this message & data)

    ss : bit 0,1 = 0 : Note Receive is EVEN,  = 1 : ODD,            = 2 : ALL
         bit 3,4 = 0 : Seq Clock is internal, = 1 : External(MIDI), = 2 : External(USB)

    dd : bit 0   = 0 : Programs Memory is not protected,            = 1 : protected
         bit 1   = 0 : Combinations Memory is not protected,        = 1 : protected
         bit 2   = 0 : Sequence Memory is not protected,            = 1 : protected
         bit 3   = 0 : Drum Kits Memory is not protected,           = 1 : protected
         bit 4   = 0 : Arp Patterns Memory is not protected,        = 1 : protected'''

#-------------------------------------------------------------------------------		
	def __init__(self,raw=None):
		if raw is None:
			pass
		else:
			self.unpack(raw)

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('>6s7B',raw)) # 13 
		self._excl_header = v.pop(0)
		self.function = v.pop(0)
		byte7 = v.pop(0)
		self.mode = byte7 & 0x07
		self._reserved8 = v.pop(0)
		byte9 = v.pop(0)
		self.note_received = byte9 & 0x03
		self.seq_clock = (byte9 >> 3) & 0x03
		byte10 = v.pop(0)
		self.programs_memory_protected = byte10 & 0x01
		self.combinations_memory_protected = (byte10 >> 1) & 0x01
		self.sequence_memory_protected = (byte10 >> 2) & 0x01
		self.drum_kits_memory_protected = (byte10 >> 3) & 0x01
		self.arp_patterns_memory_protected = (byte10 >> 4) & 0x01
		self._reserved11 = v.pop(0)
		self._end_of_excl = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self._excl_header)
		v.append(self.function)
		byte7 = 0
		byte7 |= self.mode & 0x07
		v.append(byte7)
		v.append(self._reserved8)
		byte9 = 0
		byte9 |= self.note_received & 0x03
		byte9 |= (self.seq_clock & 0x03) << 3
		byte10 = 0
		byte10 |= self.programs_memory_protected & 0x01
		byte10 |= (self.combinations_memory_protected & 0x01) << 1
		byte10 |= (self.sequence_memory_protected & 0x01) << 2
		byte10 |= (self.drum_kits_memory_protected & 0x01) << 3
		byte10 |= (self.arp_patterns_memory_protected & 0x01) << 4
		v.append(byte10)
		v.append(self._reserved11)
		v.append(self._end_of_excl)
		return struct.pack('>6s7B',*v) # 13

#-------------------------------------------------------------------------------
	@property
	def function(self):
		'''Function'''
		return self._function

	@function.setter
	def function(self,value):
		if 0 <= value <= 127:
			self._function = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def mode(self):
		'''Mode'''
		return self._mode

	@mode.setter
	def mode(self,value):
		if 0 <= value <= 7:
			self._mode = value
		else:
			raise ValueError('not 0 <= value <= 7')

#-------------------------------------------------------------------------------
	@property
	def note_received(self):
		'''Note Receive 0 : EVEN, 1 : ODD, 2 : ALL'''
		return self._note_received

	@note_received.setter
	def note_received(self,value):
		if 0 <= value <= 2:
			self._note_received = value
		else:
			raise ValueError('not 0 <= value <= 2')

#-------------------------------------------------------------------------------
	@property
	def seq_clock(self):
		'''Seq Clock 0 : internal, 1 : External(MIDI), 2 : External(USB)'''
		return self._seq_clock

	@seq_clock.setter
	def seq_clock(self,value):
		if 0 <= value <= 2:
			self._seq_clock = value
		else:
			raise ValueError('not 0 <= value <= 2')

#-------------------------------------------------------------------------------
	@property
	def programs_memory_protected(self):
		'''Programs Memory 0 : not protected, 1 : protected'''
		return self._programs_memory_protected

	@programs_memory_protected.setter
	def programs_memory_protected(self,value):
		if 0 <= value <= 1:
			self._programs_memory_protected = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def combinations_memory_protected(self):
		'''Combinations Memory 0 : not protected, 1 : protected'''
		return self._combinations_memory_protected

	@combinations_memory_protected.setter
	def combinations_memory_protected(self,value):
		if 0 <= value <= 1:
			self._combinations_memory_protected = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def sequence_memory_protected(self):
		'''Sequence Memory 0 : not protected, 1 : protected'''
		return self._sequence_memory_protected

	@sequence_memory_protected.setter
	def sequence_memory_protected(self,value):
		if 0 <= value <= 1:
			self._sequence_memory_protected = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def drum_kits_memory_protected(self):
		'''Drum Kits Memory 0 : not protected, 1 : protected'''
		return self._drum_kits_memory_protected

	@drum_kits_memory_protected.setter
	def drum_kits_memory_protected(self,value):
		if 0 <= value <= 1:
			self._drum_kits_memory_protected = value
		else:
			raise ValueError('not 0 <= value <= 1')

#-------------------------------------------------------------------------------
	@property
	def arp_patterns_memory_protected(self):
		'''Arp Patterns Memory 0 : not protected, 1 : protected'''
		return self._arp_patterns_memory_protected

	@arp_patterns_memory_protected.setter
	def arp_patterns_memory_protected(self,value):
		if 0 <= value <= 1:
			self._arp_patterns_memory_protected = value
		else:
			raise ValueError('not 0 <= value <= 1')

#===============================================================================
class ModeChange(object):
	'''(3) MODE CHANGE                                                         R , T
        F0, 42, 3g, 00, 01, 15  Excl Header
        4E                      Function
        0000 0mmm               Mode                    (*2)
        F7                      End of Excl
(Receives this message & data, changes the Mode, and transmits Func=24 message)
When the Mode is changed by SW, transmits this message & data.'''
	def __init__(self,globchnl=0,mode=0):
		self.globchnl = globchnl
		self.mode = mode

#-------------------------------------------------------------------------------		
	def __repr__(self,globchnl=0,mode=0):
		return '%s(globchnl=%s,mode=%s)' % (self.__class__.__name__,self.globchnl,self.mode)

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte7 = self.mode & 0x07
		return struct.pack('>9B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x4E, byte7, 0xF7)

#===============================================================================
class ParameterChange(object):
	'''(4) PARAMETER CHANGE                                                    R , T
        F0, 42, 3g, 00, 01, 15  Excl Header
        41                      Function
        0000 0mmm               Mode                    (*2)
        00                      Parameter ID (MSB)
        0ppp pppp               Parameter ID (LSB)      (TABLE 1,2,5,10,11)
        00                      Parameter SUB ID (MSB)
        0qqq qqqq               Parameter SUB ID (LSB)  (TABLE 1,2,5,10,11)
        0000 vvvv               Value (Value bit28-31)
        0vvv vvvv               Value (Value bit21-27)
        0vvv vvvv               Value (Value bit14-20)
        0vvv vvvv               Value (Value bit7-13)
        0vvv vvvv               Value (Value bit0-6)
        F7                      End of Excl
(Even if it receives this message, a reply does not return)
When the Parameter No. is changed by SW, transmits this message & data.'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0,mode=0,id=0,subid=0,value=0,offset=-1,bit=7,bits=8,parameter=None,rng=(0,0x7F)):
		self.globchnl = globchnl
		self.mode = mode
		self.id = id
		self.subid = subid
		self.value = value
		self.offset = offset
		self.bit = bit
		self.bits = bits
		self.parameter = parameter
		self.rng = rng

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte7 = self.mode & 0x07
		byte9 = self.id & 0x7F
		byte11 = self.subid & 0x7F
		byte12 = (self.value >> 28) & 0x0F
		byte13 = (self.value >> 21) & 0x7F
		byte14 = (self.value >> 14) & 0x7F
		byte15 = (self.value >> 7) & 0x7F
		byte16 = self.value & 0x7F
		return struct.pack('>18B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x41, byte7, 0x00, byte9, 0x00, byte11, byte12, byte13, byte14, byte15, byte16, 0xF7)

#-------------------------------------------------------------------------------		
	def __repr__(self,globchnl=0,mode=0):
#		return '%s(globchnl=%s,mode=%s,id=%s,subid=%s,value=%s) # mode=%s' % (self.__class__.__name__,self.globchnl,self.mode,self.id,self.subid,self.value,mode_dict[self.mode])
		return "\t\t\t\t\t\t0x%02X : {'offset':%s,'bit':%s,'bits':%s,'parameter':'%s','range':%s,'bias':0,'step':None,'mode':0x%02X,'id':0x%02X,'subid':0x%02X},# 0x%02X" % (self.subid,self.offset,self.bit,self.bits,self.parameter,self.rng,self.mode,self.id,self.subid,self.value)

#===============================================================================
class DrumKitParameterChange(object):
	'''(5) DRUM KIT PARAMETER CHANGE                                           R , T
        F0, 42, 3g, 00, 01, 15  Excl Header
        53                      Function
        0sss ssss               Index No.               (s = 00-7F ( : C-1-G9))
        0bbb bbbb               Drum Kit Bank           (*3)
        000k kkkk               Drum Kit No.            (k = 00-0F(INT), 00-1F(USER))
        00                      Parameter No.(MSB)
        0ppp pppp               Parameter No.(LSB)      (TABLE 6)
        0000 vvvv               Value (Value bit28-31)
        0vvv vvvv               Value (Value bit21-27)
        0vvv vvvv               Value (Value bit14-20)
        0vvv vvvv               Value (Value bit7-13)
        0vvv vvvv               Value (Value bit0-6)
        F7                      End of Excl
(Even if it receives this message, a reply does not return)'''
	def __init__(self,globchnl=0,index=0,bank=0,kit=0,param=0,value=0):
		self.globchnl = globchnl
		self.index = index  # MIDI note number 00-7F ( : C-1-G9)
		self.bank = bank
		self.kit = kit
		self.param = param
		self.value = value

#-------------------------------------------------------------------------------		
	def __str__(self):
		return 'DrumKitParameterChange index=%s bank=%s kit=%s param=0x%02X value=%s' % (self.index,self.bank,self.kit,self.param,self.value)
		
#-------------------------------------------------------------------------------		
	def __repr__(self):
		return 'DrumKitParameterChange(index=%s,bank=%s,kit=0x%02X,param=%s,value=%s)' % (self.index,self.bank,self.kit,self.param,self.value)
		
#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte7 = self.index & 0x7F
		byte8 = self.bank & 0x7F
		byte9 = self.kit & 0x1F
		byte11 = self.param & 0x7F
		byte12 = (self.value >> 28) & 0x0F
		byte13 = (self.value >> 21) & 0x7F
		byte14 = (self.value >> 14) & 0x7F
		byte15 = (self.value >> 7) & 0x7F
		byte16 = self.value & 0x7F
		return struct.pack('>18B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x53, byte7, byte8, byte9, 0x00, byte11, byte12, byte13, byte14, byte15, byte16, 0xF7)

#===============================================================================
class ArpeggioParameterChange(object):
	'''(6) ARPEGGIO PATTERN PARAMETER CHANGE                                   R , T
        F0, 42, 3g, 00, 01, 15  Excl Header
        54                      Function
        0000 000b               Arpeggio AorB   b = 0 : Arpeggio A, 1 : Arpeggio B
        0000 aaaa               Pattern No. MSB (bit10-7)
        0aaa aaaa               Pattern No. LSB (bit 6-0) a = 000-403 ( : 0-1027)
        0sss ssss               Step No.        ss = 00-2F ( : 00-47)
        0ttt tttt               Tone No.        tt = 00-0B ( : 00-11)
        00                      Parameter No. (MSB)
        0ppp pppp               Parameter No. (LSB)     (TABLE 4)
        0000 vvvv               Value (Value bit28-31)
        0vvv vvvv               Value (Value bit21-27)
        0vvv vvvv               Value (Value bit14-20)
        0vvv vvvv               Value (Value bit7-13)
        0vvv vvvv               Value (Value bit0-6)
        F7                      End of Excl
(Even if it receives this message, a reply does not return)'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0,a_or_b=0,pattern=0,step=0,tone=0,param=0,value=0):
		self.globchnl = globchnl
		self.a_or_b = a_or_b
		self.pattern = pattern
		self.step = step
		self.tone = tone
		self.param = param
		self.value = value

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte7 = self.a_or_b & 0x01
		byte8 = (self.pattern >> 7) & 0x0F
		byte9 = self.pattern & 0x7F
		byte10 = self.step & 0x7F
		byte11 = self.tone & 0x7F
		byte12 = 0x00
		byte13 = self.param & 0x7F
		byte14 = (self.value >> 28) & 0x0F
		byte15 = (self.value >> 21) & 0x7F
		byte16 = (self.value >> 14) & 0x7F
		byte17 = (self.value >> 7) & 0x7F
		byte18 = self.value & 0x7F
		return struct.pack('>20B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x54, byte7, byte8, byte9, byte10, byte11, byte12, byte13, byte14, byte15, byte16, byte17, byte18, 0xF7)

#===============================================================================
class ProgramParameterDumpRequest(object):
	'''(7) PROGRAM PARAMETER DUMP REQUEST                                      R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        00                      Object(Program)
        0bbb bbbb               Program Bank            (*3)
        00                      Reserved
        0ppp pppp               Program No.
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=00 or Func=24 message)'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0,bank=0,program=0):
		self.globchnl = globchnl
		self.bank = bank
		self.program = program

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte8 = self.bank & 0x7F
		byte10 = self.program & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x00, byte8, 0x00, byte10, 0xF7)

#===============================================================================
class CombinationParameterDumpRequest(object):
	'''(8) COMBINATION PARAMETER DUMP REQUEST                                  R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        01                      Object(Combination)
        0bbb bbbb               Combination Bank        (*3)
        00                      Reserved
        0ccc cccc               Combination No.
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=01 or Func=24 message)'''
	def __init__(self,globchnl=0,bank=0,combi=0):
		self.globchnl = globchnl
		self.bank = bank
		self.combi = combi

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte8 = self.bank & 0x7F
		byte10 = self.combi & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x01, byte8, 0x00, byte10, 0xF7)

#===============================================================================
class SongTimbreSetDataDumpRequest(object):
	'''(9) SONG TIMBRE SET DATA DUMP REQUEST                                   R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        02                      Object(Song Timbre Set)
        00                      Reserved
        00                      Reserved
        0ccc cccc               Song No.        cc = 00-7F ( : 0-127)
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=02 or Func=24 message)'''
	def __init__(self,globchnl=0,song=0):
		self.globchnl = globchnl
		self.song = song

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte10 = self.song & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x02, 0x00, 0x00, byte10, 0xF7)

#===============================================================================
class GlobalDataDumpRequest(object):
	'''(10) GLOBAL DATA DUMP REQUEST                                           R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        03                      Object(Global)
        00                      Reserved
        00                      Reserved
        00                      Reserved
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=03 or Func=24 message)'''
	def __init__(self,globchnl=0):
		self.globchnl = globchnl

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x03, 0x00, 0x00, 0x00, 0xF7)

#===============================================================================
class DrumKitParameterDumpRequest(object):
	'''(11) DRUM KIT PARAMETER DUMP REQUEST                                    R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        04                      Object(Drum Kit)
        0bbb bbbb               Drum Kit Bank           (*3)
        00                      Reserved
        000k kkkk               Drum Kit No.            (k = 00-0F(INT), 00-1F(USER))
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=04 or Func=24 message)'''
	def __init__(self,globchnl=0,bank=0,kit=0):
		self.globchnl = globchnl
		self.bank = bank
		self.kit = kit

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte8 = self.bank & 0x7F
		byte10 = self.kit & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x04, byte8, 0x00, byte10, 0xF7)

#===============================================================================
class SongControlDataDumpRequest(object):
	'''(12) SONG CONTROL DATA DUMP REQUEST                                     R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        08                      Object(Song Control)
        00                      Reserved
        00                      Reserved
        0ccc cccc               Song No.        cc = 00-7F ( : 0-127)
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=08 or Func=24 message)'''
	def __init__(self,globchnl=0,song=0):
		self.globchnl = globchnl
		self.song = song

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte10 = self.song & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x08, 0x00, 0x00, byte10, 0xF7)

#===============================================================================
class SongEventDataDumpRequest(object):
	'''(13) SONG EVENT DATA DUMP REQUEST                                       R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        09                      Object(Song Event)
        00                      Reserved
        00                      Reserved
        0ccc cccc               Song No.        cc = 00-7F ( : 0-127)
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=09 or Func=24 message)'''
	def __init__(self,globchnl=0,song=0):
		self.globchnl = globchnl
		self.song = song

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte10 = self.song & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x09, 0x00, 0x00, byte10, 0xF7)

#===============================================================================
class DrumTrackPatternDataDumpRequest(object):
	'''(14) DRUM TRACK PATTERN DATA DUMP REQUEST                               R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        0B                      Object(Drum Track Pattern)
        00                      Reserved
        0000 0kkk               Pattern No. MSB
        0kkk kkkk               Pattern No. LSB           k = 0-3E7 ( : 0-999)
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=0B&0C or Func=24 message)'''
	def __init__(self,globchnl=0,pattern=0):
		self.globchnl = globchnl
		self.pattern = pattern

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte9 = (self.pattern >> 7) & 0x07
		byte10 = self.pattern & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x0B, 0x00, byte9, byte10, 0xF7)

#===============================================================================
class CueListParameterDumpRequest(object):
	'''CUELIST PARAMETER DUMP REQUEST                                R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        0D                      Object(Program Bank)
        00                      Reserved
        00                      Reserved
        000c cccc               CueList No.
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=0F or Func=24 message)'''
	def __init__(self,globchnl=0,cuelist=0):
		self.globchnl = globchnl
		self.cuelist = cuelist

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte10 = self.cuelist & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x0D, 0x00, 0x00, byte10, 0xF7)

#===============================================================================
class ProgramBankParameterDumpRequest(object):
	'''(15) PROGRAM BANK PARAMETER DUMP REQUEST                                R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        0F                      Object(Program Bank)
        0bbb bbbb               Program Bank            (*3)
        00                      Reserved
        00                      Reserved
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=0F or Func=24 message)'''
	def __init__(self,globchnl=0,bank=0):
		self.globchnl = globchnl
		self.bank = bank

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte8 = self.bank & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x0F, byte8, 0x00, 0x00, 0xF7)

#===============================================================================
class CombinationBankParameterDumpRequest(object):
	'''(16) COMBINATION BANK PARAMETER DUMP REQUEST                            R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        11                      Object(Combination Bank)
        0bbb bbbb               Combination Bank        (*3)
        00                      Reserved
        00                      Reserved
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=11 or Func=24 message)'''
	def __init__(self,globchnl=0,bank=0):
		self.globchnl = globchnl
		self.bank = bank

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte8 = self.bank & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x11, byte8, 0x00, 0x00, 0xF7)

#===============================================================================
class DrumKitBankParameterDumpRequest(object):
	'''(17) DRUM KIT BANK PARAMETER DUMP REQUEST                               R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        12                      Object(Drum Kit Bank)
        0bbb bbbb               Drum Kit Bank           (*3)
        00                      Reserved
        00                      Reserved
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=12 or Func=24 message)'''
	def __init__(self,globchnl=0,bank=0):
		self.globchnl = globchnl
		self.bank = bank

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte8 = self.bank & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x12, byte8, 0x00, 0x00, 0xF7)

#===============================================================================
class ArpeggioPatternParameterDumpRequest(object):
	'''(18) ARPEGGIO PATTERN PARAMETER DUMP REQUEST                            R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        16                      Object(Arpeggio Pattern)
        00                      Reserved
        0000 aaaa               Pattern No. MSB (bit10-7)
        0aaa aaaa               Pattern No. LSB (bit 6-0) a = 000-403 ( : 0-1027)
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=16 or Func=24 message)'''
	def __init__(self,globchnl=0,pattern=0):
		self.globchnl = globchnl
		self.pattern = pattern

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte9 = (self.pattern >> 7) & 0x0F
		byte10 = self.pattern & 0x7F
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x16, 0x00, byte9, byte10, 0xF7)

#===============================================================================
class ArpeggioPatternAllParameterDumpRequest(object):
	'''(19) ARPEGGIO PATTERN ALL PARAMETER DUMP REQUEST                        R
        F0, 42, 3g, 00, 01, 15  Excl Header
        72                      Function
        17                      Object(Arpeggio Pattern All)
        00                      Reserved
        00                      Reserved
        00                      Reserved
        F7                      End of Excl
(Receives this message, and transmits Func=73/Object=17 or Func=24 message)'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0,pattern=0):
		self.globchnl = globchnl
		self.pattern = pattern

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		return struct.pack('>12B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x72, 0x17, 0x00, 0x00, 0x00, 0xF7)

#===============================================================================
class CurrentProgramParameterDumpRequest(object):
	'''(35) CURRENT PROGRAM PARAMETER DUMP REQUEST                                      R
        F0, 42, 3g, 00, 01, 15  Excl Header
        74                      Function
        00                      Object(Program)
        F7                      End of Excl
(Receives this message, and transmits Func=75/Object=00 or Func=24 message)'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0):
		self.globchnl = globchnl

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		return struct.pack('>9B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x74, 0x00, 0xF7)

#===============================================================================
class CurrentCombinationParameterDumpRequest(object):
	'''(35) CURRENT COMBINATION PARAMETER DUMP REQUEST                                  R
        F0, 42, 3g, 00, 01, 15  Excl Header
        74                      Function
        01                      Object(Combination)
        F7                      End of Excl
(Receives this message, and transmits Func=75/Object=01 or Func=24 message)'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0):
		self.globchnl = globchnl

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		return struct.pack('>9B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x74, 0x01, 0xF7)

#===============================================================================
class CurrentSongTimbreSetDataDumpRequest(object):
	'''(9) SONG TIMBRE SET DATA DUMP REQUEST                                   R
        F0, 42, 3g, 00, 01, 15  Excl Header
        74                      Function
        02                      Object(Song Timbre Set)
        F7                      End of Excl
(Receives this message, and transmits Func=75/Object=02 or Func=24 message)'''
#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0):
		self.globchnl = globchnl

#-------------------------------------------------------------------------------		
	def pack(self):
		byte2 = 0x30 | (self.globchnl & 0x0F)
		return struct.pack('>9B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x74, 0x02, 0xF7)

#===============================================================================
class ProgramParameterDump(object):

#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0,pattern=0):
		self.globchnl = globchnl
		self.pattern = pattern

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a  object'''
		v = list(struct.unpack('>6s6B2154sB',raw)) # 2167 
		self._excl_header = v.pop(0)
		self.function = v.pop(0)
		self.object = v.pop(0)
		self.bank = v.pop(0)
		self._reserved9 = v.pop(0)
		self.program = v.pop(0)
		self.version = v.pop(0)
		self.data = v.pop(0)
		self._end_of_excl = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		v = list()
		v.append(self._excl_header)
		v.append(self.function)
		v.append(self.object)
		v.append(self.bank)
		v.append(self._reserved9)
		v.append(self.program)
		v.append(self.version)
		v.append(self.data.pack())
		v.append(self._end_of_excl)
		return struct.pack('>6s6B2154sB',*v) # 2167

#-------------------------------------------------------------------------------
	@property
	def function(self):
		'''Function'''
		return self._function

	@function.setter
	def function(self,value):
		if 0 <= value <= 127:
			self._function = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def object(self):
		'''Object(Program)'''
		return self._object

	@object.setter
	def object(self,value):
		if 0 <= value <= 127:
			self._object = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def bank(self):
		'''Program Bank'''
		return self._bank

	@bank.setter
	def bank(self,value):
		if 0 <= value <= 127:
			self._bank = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def program(self):
		'''Program Number'''
		return self._program

	@program.setter
	def program(self,value):
		if 0 <= value <= 127:
			self._program = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def version(self):
		'''Object Version'''
		return self._version

	@version.setter
	def version(self,value):
		if 0 <= value <= 127:
			self._version = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def data(self):
		'''Data (*1, TABLE 1)'''
		return self._data

	@data.setter
	def data(self,value):
		if isinstance(value,ProgramParameters):
			self._data = value
		elif isinstance(value,basestring):
			self._data = ProgramParameters(convert_7_bit_to_8_bit(value))
		else:
			raise TypeError('value must of type ProgramParameters')

#===============================================================================
class ResultReport(object):
	message_dict = {
					0 : 'Result OK',
					1 : 'Received Data Length is wrong',
					2 : 'Dest Memory is protected',
					3 : 'Dest Bank/Prog/Param is not exist',
					4 : 'The mode is wrong',
					5 : 'Memory over flow',
					40 : 'Another type error'}

#-------------------------------------------------------------------------------		
	def __init__(self,globchnl=0,code=0):
		self.globchnl = globchnl
		self.code = code

#-------------------------------------------------------------------------------		
	def __str__(self):
		return 'Result: %d %s' % (self._code,self.message)
		self.globchnl = globchnl
		self.code = code

#-------------------------------------------------------------------------------
	def unpack(self,raw):
		'''Unpack raw data into a ResultReport object'''
		v = list(struct.unpack('>6s3B',raw)) # 9
		self._excl_header = v.pop(0)
		self.function = v.pop(0)
		self.code = v.pop(0)
		self._end_of_excl = v.pop(0)

#-------------------------------------------------------------------------------
	def pack(self):
		'''Pack a  object into raw data'''
		byte2 = 0x30 | (self.globchnl & 0x0F)
		byte7 = self.code & 0x7F
		return struct.pack('>9B',0xF0, 0x42, byte2, 0x00, 0x01, 0x15, 0x24, byte7, 0xF7)

#-------------------------------------------------------------------------------
	@property
	def function(self):
		'''Function'''
		return self._function

	@function.setter
	def function(self,value):
		if 0 <= value <= 127:
			self._function = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def code(self):
		'''Result Code'''
		return self._code

	@code.setter
	def code(self,value):
		if 0 <= value <= 127:
			self._code = value
		else:
			raise ValueError('not 0 <= value <= 127')

#-------------------------------------------------------------------------------
	@property
	def message(self):
		'''Result Message'''
		return ResultReport.message_dict[self._code]


